@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2006 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2008 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -21,10 +21,6 @@
 
 #include "dvdisaster.h"
 
-#ifdef WITH_LOGFILE_YES
- #include <time.h>
-#endif
-
 #ifdef SYS_MINGW
  #include <windows.h>
  #include <tlhelp32.h>
@@ -136,7 +132,7 @@ void WriteSignature()
    /* processing of error conditions not necessary */
 
    sprintf(loc, "%s\\signature", Closure->binDir);
-   if(!(file = fopen(loc, "wb")))
+   if(!(file = portable_fopen(loc, "wb")))
      return;
    fwrite(sig, 20, 1, file);
    fclose(file);
@@ -151,7 +147,7 @@ int VerifySignature()
    int result;
 
    sprintf(loc, "%s\\signature", Closure->binDir);
-   if(!(file = fopen(loc, "rb")))
+   if(!(file = portable_fopen(loc, "rb")))
      return FALSE;
 
    fread(buf, 20, 1, file);
@@ -166,20 +162,20 @@ int VerifySignature()
 
 
 static void get_base_dirs()
-{  struct stat mystat;
-#ifdef WITH_LOGFILE_YES
-   time_t now;
-#endif
+{  
 #ifdef SYS_MINGW
    char *appdata;
 #endif
 
-   /*** The source directory is supposed to hold the most recent files,
+   /*** Unless completely disabled through a configure option, the
+	source directory is supposed to hold the most recent files,
 	so try this first. Not necessary under Windows as it will always
 	use the directory the binary has been called from. */
 
+#ifdef WITH_EMBEDDED_SRC_PATH_YES
+
 #ifndef SYS_MINGW
-   if(!stat(SRCDIR, &mystat))
+   if(DirStat(SRCDIR))
    {  Closure->binDir = g_strdup(SRCDIR);
       Closure->docDir = g_strdup_printf("%s/documentation",SRCDIR);
       Verbose("Using paths from SRCDIR = %s\n", SRCDIR);
@@ -187,16 +183,18 @@ static void get_base_dirs()
    } 
 #endif
 
+#endif /* WITH_EMBEDDED_SRC_PATH_YES */
+
    /*** Otherwise try the installation directory. 
-	On Linux this is a hardcoded directory.
+	On Unices this is a hardcoded directory.
 	Windows has binary distributions with no prior known installation place,
 	but luckily it provides a way for figuring out that location. */
 
-#if defined(SYS_LINUX) || defined(SYS_FREEBSD) || defined(SYS_DARWIN) || defined(SYS_UNKNOWN)
-   if(!stat(BINDIR, &mystat))
+#ifndef SYS_MINGW
+   if(DirStat(BINDIR))
      Closure->binDir = g_strdup(BINDIR);
 
-   if(!stat(DOCDIR, &mystat))
+   if(DirStat(DOCDIR))
      Closure->docDir = g_strdup(DOCDIR);
    Verbose("Using hardcoded BINDIR = %s, DOCDIR = %s\n", BINDIR, DOCDIR);
 #endif
@@ -212,11 +210,12 @@ static void get_base_dirs()
    /*** The location of the dotfile depends on the operating system. 
 	Under Unix the users home directory is used. */
 
-#if defined(SYS_LINUX) || defined(SYS_FREEBSD) || defined(SYS_DARWIN) || defined(SYS_UNKNOWN)
+#ifndef SYS_MINGW
+ #ifdef WITH_EMBEDDED_SRC_PATH_YES
 find_dotfile:
+ #endif /* WITH_EMBEDDED_SRC_PATH_YES */
 
    Closure->dotFile = g_strdup_printf("%s/.dvdisaster", g_getenv("HOME"));
-   Closure->logFile = g_strdup_printf("%s/.dvdisaster.log", g_getenv("HOME"));
 #endif
 
 #ifdef SYS_MINGW
@@ -230,26 +229,31 @@ find_dotfile:
 	   appdata ? appdata : "NULL");
 
    if(appdata)
-   {  char *our_dir = g_strdup_printf("%s\\dvdisaster", appdata);
+   {  Closure->appData = g_strdup_printf("%s\\dvdisaster", appdata);
 
-      if(!stat(appdata, &mystat)) /* CSIDL_APPDATA present? */
+      if(DirStat(appdata)) /* CSIDL_APPDATA present? */
       { 
-	 Verbose("- dotfile path : %s\n", our_dir);
+	 Verbose("- dotfile path : %s\n", Closure->appData);
 
-	 if(!stat(our_dir, &mystat))
-	 {  Closure->dotFile = g_strdup_printf("%s\\.dvdisaster", our_dir);
-	    Closure->logFile = g_strdup_printf("%s\\logfile.txt", our_dir);
+	 if(DirStat(Closure->appData))
+	 {  Closure->dotFile = g_strdup_printf("%s\\.dvdisaster", Closure->appData);
 	    Verbose("- dotfile path : present\n");
 	 }
-	 else if(!mkdir(our_dir)) /* Note: Windows! */
-	 {  Closure->dotFile = g_strdup_printf("%s\\.dvdisaster", our_dir);
-	    Closure->logFile = g_strdup_printf("%s\\logfile.txt", our_dir);
+	 else if(!portable_mkdir(Closure->appData)) /* Note: Windows! */
+	 {  Closure->dotFile = g_strdup_printf("%s\\.dvdisaster", Closure->appData);
 	    Verbose("- dotfile path : - created -\n");
 	 }
+	 else 
+	 {  g_free(Closure->appData);
+	    Closure->appData = NULL;
+	 }
+      }
+      else 
+      {	 Verbose("- dotfile path : *can not be used*\n");
+	 g_free(Closure->appData);
+	 Closure->appData = NULL;
       }
-      else Verbose("- dotfile path : *can not be used*\n");
 
-      g_free(our_dir);
       g_free(appdata);
    }
 
@@ -257,29 +261,110 @@ find_dotfile:
 
    if(!Closure->dotFile)
       Closure->dotFile = g_strdup_printf("%s\\.dvdisaster", Closure->binDir);
-
-   if(!Closure->logFile)
-      Closure->logFile = g_strdup_printf("%s\\logfile.txt", Closure->binDir);
 #endif
 
    Verbose("\nUsing file locations:\n"
 	   "- Bin dir: %s\n"
 	   "- Doc dir: %s\n"
-	   "- dotfile: %s\n"
-	   "- logfile: %s\n\n",
+	   "- AppData: %s\n"
+	   "- dotfile: %s\n\n",
 	   Closure->binDir,
 	   Closure->docDir,
-	   Closure->dotFile,
-	   Closure->logFile);   
+	   Closure->appData,
+	   Closure->dotFile);   
+}
 
-#ifdef WITH_LOGFILE_YES
-   if(!stat(Closure->logFile, &mystat))
-     unlink(Closure->logFile);
+/***
+ *** Set/get color values
+ ***/
 
-   time(&now);
-   PrintLogFile("dvdisaster-%s logging started at %s\n",
-		Closure->cookedVersion, ctime(&now));
-#endif
+/*
+ * Update color string for the <span color="#f00baa">...</span> string
+ */
+
+void UpdateMarkup(char **string, GdkColor *color)
+{  int hexval;
+ 
+   hexval  = (color->red  << 8) & 0xff0000;
+   hexval |=  color->green      & 0xff00;
+   hexval |= (color->blue >> 8) & 0xff;
+
+   if(*string) g_free(*string);
+   *string = g_strdup_printf("color=\"#%06x\"", hexval);
+}
+
+/*
+ * Default color values
+ */
+
+void DefaultColors()
+{
+   Closure->redText->red        = 0xffff;
+   Closure->redText->green      = 0;
+   Closure->redText->blue       = 0;
+
+   Closure->greenText->red      = 0;
+   Closure->greenText->green    = 0x8000;
+   Closure->greenText->blue     = 0;
+
+   Closure->barColor->red       = 0xffff;
+   Closure->barColor->green     = 0;
+   Closure->barColor->blue      = 0;
+
+   Closure->logColor->red       = 0xffff;
+   Closure->logColor->green     = 0;
+   Closure->logColor->blue      = 0xffff;
+
+   Closure->curveColor->red     = 0;
+   Closure->curveColor->green   = 0;
+   Closure->curveColor->blue    = 0xffff;
+
+   Closure->redSector->red      = 0xffff;
+   Closure->redSector->green    = 0;
+   Closure->redSector->blue     = 0;
+
+   Closure->yellowSector->red   = 0xffff;
+   Closure->yellowSector->green = 0xc000;
+   Closure->yellowSector->blue  = 0;
+      
+   Closure->greenSector->red    = 0;
+   Closure->greenSector->green  = 0xdb00;
+   Closure->greenSector->blue   = 0;
+
+   Closure->darkSector->red     = 0;
+   Closure->darkSector->green   = 0x8000;
+   Closure->darkSector->blue    = 0;
+
+   Closure->blueSector->red     = 0;
+   Closure->blueSector->green   = 0;
+   Closure->blueSector->blue    = 0xffff;
+
+   Closure->whiteSector->red    = 0xffff;
+   Closure->whiteSector->green  = 0xffff;
+   Closure->whiteSector->blue   = 0xffff;
+
+   UpdateMarkup(&Closure->redMarkup, Closure->redText);
+   UpdateMarkup(&Closure->greenMarkup, Closure->greenText);
+}
+
+static void save_colors(FILE *dotfile, char *symbol, GdkColor *color)
+{  char *blanks="                    ";
+   char *pad;
+   int len=strlen(symbol);
+
+   if(len>19) pad=blanks+19;
+   else       pad=blanks+len;
+
+   fprintf(dotfile, "%s:%s%02x%02x%02x\n", symbol, pad,
+	   color->red>>8, color->green>>8, color->blue>>8);
+}
+
+static void get_color(GdkColor *color, char *value)
+{  unsigned int hex = strtol(value, NULL, 16);
+   
+   color->red   = (hex>>8)&0xff00;
+   color->green = hex&0xff00;
+   color->blue  = (hex<<8)&0xff00;
 }
 
 /***
@@ -292,7 +377,7 @@ void ReadDotfile()
 {  FILE *dotfile;
    char line[MAX_LINE_LEN];
 
-   dotfile = fopen(Closure->dotFile, "rb");
+   dotfile = portable_fopen(Closure->dotFile, "rb");
    if(!dotfile)
       return;
 
@@ -345,32 +430,71 @@ void ReadDotfile()
 
       if(!strcmp(symbol, "adaptive-read"))   { Closure->adaptiveRead   = atoi(value); continue; }
       if(!strcmp(symbol, "auto-suffix"))     { Closure->autoSuffix  = atoi(value); continue; }
+      if(!strcmp(symbol, "bd-size1"))        { Closure->bdSize1 = Closure->savedBDSize1 = atoll(value); continue; }
+      if(!strcmp(symbol, "bd-size2"))        { Closure->bdSize2 = Closure->savedBDSize2 = atoll(value); continue; }
       if(!strcmp(symbol, "cache-size"))      { Closure->cacheMB     = atoi(value); continue; }
       if(!strcmp(symbol, "cd-size"))         { Closure->cdSize = Closure->savedCDSize = atoll(value); continue; }
+      if(!strcmp(symbol, "codec-threads"))   { Closure->codecThreads = atoi(value); continue; }
       if(!strcmp(symbol, "dao"))             { Closure->noTruncate  = atoi(value); continue; }
+      if(!strcmp(symbol, "defective-dump"))  { Closure->defectiveDump = atoi(value); continue; }
+      if(!strcmp(symbol, "defective-dir"))   { if(Closure->dDumpDir) g_free(Closure->dDumpDir);
+	                                       Closure->dDumpDir = g_strdup(value); continue; }
+      if(!strcmp(symbol, "defective-prefix")){ if(Closure->dDumpPrefix) g_free(Closure->dDumpPrefix); 
+	                                       Closure->dDumpPrefix = g_strdup(value); continue; }
       if(!strcmp(symbol, "dotfile-version")) { Closure->dotFileVersion = atoi(value); continue; }
       if(!strcmp(symbol, "dvd-size1"))       { Closure->dvdSize1 = Closure->savedDVDSize1 = atoll(value); continue; }
       if(!strcmp(symbol, "dvd-size2"))       { Closure->dvdSize2 = Closure->savedDVDSize2 = atoll(value); continue; }
+      if(!strcmp(symbol, "eject"))           { Closure->eject  = atoi(value); continue; }
       if(!strcmp(symbol, "fill-unreadable")) { Closure->fillUnreadable = atoi(value); 
 	                                       PrepareDeadSector();
 	                                       continue; 
                                              }
+      if(!strcmp(symbol, "ignore-fatal-sense")) { Closure->ignoreFatalSense  = atoi(value); continue; }
+      if(!strcmp(symbol, "internal-attempts"))  { Closure->internalAttempts = atoi(value); continue; }
       if(!strcmp(symbol, "jump"))            { Closure->sectorSkip  = atoi(value); continue; }
+      if(!strcmp(symbol, "log-file-enabled")){ Closure->logFileEnabled = atoi(value); continue; }
+      if(!strcmp(symbol, "log-file"))        { if(Closure->logFile) g_free(Closure->logFile);
+	                                       Closure->logFile  = g_strdup(value); continue; }
       if(!strcmp(symbol, "medium-size"))     { Closure->mediumSize  = atoll(value); continue; }
       if(!strcmp(symbol, "method-name"))     { if(Closure->methodName) g_free(Closure->methodName);
 	                                       Closure->methodName = g_strdup(value); continue; }
+      if(!strcmp(symbol, "max-read-attempts"))   { Closure->maxReadAttempts = atoi(value); continue; }
+      if(!strcmp(symbol, "min-read-attempts"))   { Closure->minReadAttempts = atoi(value); continue; }
       if(!strcmp(symbol, "query-size"))      { Closure->querySize  = atoi(value); continue; }
+      if(!strcmp(symbol, "raw-mode"))        { Closure->rawMode = atoi(value); continue; }
       if(!strcmp(symbol, "read-and-create")) { Closure->readAndCreate = atoi(value); continue; }
+      if(!strcmp(symbol, "read-medium"))     { Closure->readingPasses = atoi(value); continue; }
+      if(!strcmp(symbol, "read-raw"))        { Closure->readRaw = atoi(value); continue; }
       if(!strcmp(symbol, "redundancy"))      { if(Closure->redundancy) g_free(Closure->redundancy);
                                                Closure->redundancy  = g_strdup(value); continue; }
+      if(!strcmp(symbol, "reverse-cancel-ok")) { Closure->reverseCancelOK = atoi(value); continue; }
       if(!strcmp(symbol, "spinup-delay"))    { Closure->spinupDelay = atoi(value); continue; }
       if(!strcmp(symbol, "split-files"))     { Closure->splitFiles  = atoi(value); continue; }
       if(!strcmp(symbol, "unlink"))          { Closure->unlinkImage = atoi(value); continue; }
+      if(!strcmp(symbol, "verbose"))         { Closure->verbose = atoi(value); continue; }
       if(!strcmp(symbol, "welcome-msg"))     { Closure->welcomeMessage = atoi(value); continue; }
+
+      if(!strcmp(symbol, "positive-text"))   { get_color(Closure->greenText, value); 
+	                                       UpdateMarkup(&Closure->greenMarkup, Closure->greenText);
+	                                       continue; 
+                                             }
+      if(!strcmp(symbol, "negative-text"))   { get_color(Closure->redText, value);
+	                                       UpdateMarkup(&Closure->redMarkup, Closure->redText); 
+					       continue; 
+                                             }
+      if(!strcmp(symbol, "bar-color"))       { get_color(Closure->barColor, value); continue; }
+      if(!strcmp(symbol, "log-color"))       { get_color(Closure->logColor, value); continue; }
+      if(!strcmp(symbol, "curve-color"))     { get_color(Closure->curveColor, value); continue; }
+      if(!strcmp(symbol, "defective-sector")){ get_color(Closure->redSector, value); continue; }
+      if(!strcmp(symbol, "bad-checksum-sector")){ get_color(Closure->yellowSector, value); continue; }
+      if(!strcmp(symbol, "good-sector"))     { get_color(Closure->greenSector, value); continue; }
+      if(!strcmp(symbol, "ignored-sector"))  { get_color(Closure->blueSector, value); continue; }
+      if(!strcmp(symbol, "highlit-sector"))  { get_color(Closure->whiteSector, value); continue; }
+      if(!strcmp(symbol, "present-sector"))  { get_color(Closure->darkSector, value); continue; }
    }
 
    if(fclose(dotfile))
-     g_fprintf(stderr, _("Error closing configuration file %s: %s\n"), 
+     g_fprintf(stderr, "Error closing configuration file %s: %s\n", 
 	       Closure->dotFile, strerror(errno));
 }
 
@@ -388,9 +512,9 @@ static void update_dotfile()
 
    /*** Otherwise, save our session */
 
-   dotfile = fopen(Closure->dotFile, "wb");
+   dotfile = portable_fopen(Closure->dotFile, "wb");
    if(!dotfile)
-   {  g_fprintf(stderr, _("Could not open configuration file %s: %s\n"), 
+   {  g_fprintf(stderr, "Could not open configuration file %s: %s\n", 
 		Closure->dotFile, strerror(errno));
       return;
    }
@@ -401,36 +525,65 @@ static void update_dotfile()
 	       "# which will be overwritten each time dvdisaster is run.\n\n"),
 	     VERSION);
 
-   g_fprintf(dotfile, "last-device:     %s\n", Closure->device);
-   g_fprintf(dotfile, "last-image:      %s\n", Closure->imageName);
-   g_fprintf(dotfile, "last-ecc:        %s\n", Closure->eccName);
-   g_fprintf(dotfile, "browser:         %s\n\n", Closure->browser);
+   g_fprintf(dotfile, "last-device:       %s\n", Closure->device);
+   g_fprintf(dotfile, "last-image:        %s\n", Closure->imageName);
+   g_fprintf(dotfile, "last-ecc:          %s\n", Closure->eccName);
+   g_fprintf(dotfile, "bd-size1:          %lld\n", (long long int)Closure->bdSize1);
+   g_fprintf(dotfile, "bd-size2:          %lld\n", (long long int)Closure->bdSize2);
+   g_fprintf(dotfile, "browser:           %s\n\n", Closure->browser);
 
-   g_fprintf(dotfile, "adaptive-read:   %d\n", Closure->adaptiveRead);
-   g_fprintf(dotfile, "auto-suffix:     %d\n", Closure->autoSuffix);
-   g_fprintf(dotfile, "cache-size:      %d\n", Closure->cacheMB);
-   g_fprintf(dotfile, "cd-size:         %lld\n", (long long int)Closure->cdSize);
-   g_fprintf(dotfile, "dao:             %d\n", Closure->noTruncate);
-   g_fprintf(dotfile, "dotfile-version: %d\n", Closure->dotFileVersion);
-   g_fprintf(dotfile, "dvd-size1:       %lld\n", (long long int)Closure->dvdSize1);
-   g_fprintf(dotfile, "dvd-size2:       %lld\n", (long long int)Closure->dvdSize2);
-   g_fprintf(dotfile, "fill-unreadable: %d\n", Closure->fillUnreadable);
-   g_fprintf(dotfile, "jump:            %d\n", Closure->sectorSkip);
-   g_fprintf(dotfile, "medium-size:     %lld\n", (long long int)Closure->mediumSize);
-   g_fprintf(dotfile, "method-name:     %s\n", Closure->methodName);
-   g_fprintf(dotfile, "query-size:      %d\n", Closure->querySize);
-   g_fprintf(dotfile, "read-and-create: %d\n", Closure->readAndCreate);
+   g_fprintf(dotfile, "adaptive-read:     %d\n", Closure->adaptiveRead);
+   g_fprintf(dotfile, "auto-suffix:       %d\n", Closure->autoSuffix);
+   g_fprintf(dotfile, "cache-size:        %d\n", Closure->cacheMB);
+   g_fprintf(dotfile, "cd-size:           %lld\n", (long long int)Closure->cdSize);
+   g_fprintf(dotfile, "codec-threads:     %d\n", Closure->codecThreads);
+   g_fprintf(dotfile, "dao:               %d\n", Closure->noTruncate);
+   g_fprintf(dotfile, "defective-dump:    %d\n", Closure->defectiveDump);
+   g_fprintf(dotfile, "defective-dir:     %s\n", Closure->dDumpDir);
+   g_fprintf(dotfile, "defective-prefix:  %s\n", Closure->dDumpPrefix);
+   g_fprintf(dotfile, "dotfile-version:   %d\n", Closure->dotFileVersion);
+   g_fprintf(dotfile, "dvd-size1:         %lld\n", (long long int)Closure->dvdSize1);
+   g_fprintf(dotfile, "dvd-size2:         %lld\n", (long long int)Closure->dvdSize2);
+   g_fprintf(dotfile, "eject:             %d\n", Closure->eject);
+   g_fprintf(dotfile, "fill-unreadable:   %d\n", Closure->fillUnreadable);
+   g_fprintf(dotfile, "ignore-fatal-sense: %d\n", Closure->ignoreFatalSense);
+   g_fprintf(dotfile, "internal-attempts: %d\n", Closure->internalAttempts);
+   g_fprintf(dotfile, "jump:              %d\n", Closure->sectorSkip);
+   g_fprintf(dotfile, "log-file-enabled:  %d\n", Closure->logFileEnabled);
+   g_fprintf(dotfile, "log-file:          %s\n", Closure->logFile);
+   g_fprintf(dotfile, "medium-size:       %lld\n", (long long int)Closure->mediumSize);
+   g_fprintf(dotfile, "method-name:       %s\n", Closure->methodName);
+   g_fprintf(dotfile, "max-read-attempts: %d\n", Closure->maxReadAttempts);
+   g_fprintf(dotfile, "min-read-attempts: %d\n", Closure->minReadAttempts);
+   g_fprintf(dotfile, "query-size:        %d\n", Closure->querySize);
+   g_fprintf(dotfile, "raw-mode:          %d\n", Closure->rawMode);
+   g_fprintf(dotfile, "read-and-create:   %d\n", Closure->readAndCreate);
+   g_fprintf(dotfile, "read-medium:       %d\n", Closure->readingPasses);
+   g_fprintf(dotfile, "read-raw:          %d\n", Closure->readRaw);
    if(Closure->redundancy)
-     g_fprintf(dotfile, "redundancy:    %s\n", Closure->redundancy);
-   g_fprintf(dotfile, "spinup-delay:    %d\n", Closure->spinupDelay);
-   g_fprintf(dotfile, "split-files:     %d\n", Closure->splitFiles);
-   g_fprintf(dotfile, "unlink:          %d\n", Closure->unlinkImage);
-   g_fprintf(dotfile, "welcome-msg:     %d\n", Closure->welcomeMessage);
+     g_fprintf(dotfile, "redundancy:        %s\n", Closure->redundancy);
+   g_fprintf(dotfile, "reverse-cancel-ok: %d\n", Closure->reverseCancelOK);
+   g_fprintf(dotfile, "spinup-delay:      %d\n", Closure->spinupDelay);
+   g_fprintf(dotfile, "split-files:       %d\n", Closure->splitFiles);
+   g_fprintf(dotfile, "unlink:            %d\n", Closure->unlinkImage);
+   g_fprintf(dotfile, "verbose:           %d\n", Closure->verbose);
+   g_fprintf(dotfile, "welcome-msg:       %d\n\n", Closure->welcomeMessage);
+
+   save_colors(dotfile, "positive-text",      Closure->greenText);
+   save_colors(dotfile, "negative-text",      Closure->redText);
+   save_colors(dotfile, "bar-color",          Closure->barColor);
+   save_colors(dotfile, "log-color",          Closure->logColor);
+   save_colors(dotfile, "curve-color",        Closure->curveColor);
+   save_colors(dotfile, "defective-sector",   Closure->redSector);
+   save_colors(dotfile, "bad-checksum-sector",Closure->yellowSector);
+   save_colors(dotfile, "good-sector",        Closure->greenSector);
+   save_colors(dotfile, "ignored-sector",     Closure->blueSector);
+   save_colors(dotfile, "highlit-sector",     Closure->whiteSector);
+   save_colors(dotfile, "present-sector",     Closure->darkSector);
 
    if(fclose(dotfile))
-     g_fprintf(stderr, _("Error closing configuration file %s: %s\n"), 
+     g_fprintf(stderr, "Error closing configuration file %s: %s\n", 
 	       Closure->dotFile, strerror(errno));
-
 }
 
 /***
@@ -491,10 +644,17 @@ void InitClosure()
    Closure->deviceNodes = g_ptr_array_new();
    Closure->imageName   = g_strdup("medium.iso");
    Closure->eccName     = g_strdup("medium.ecc");
-   Closure->browser     = g_strdup("gnome-open");
+   Closure->browser     = g_strdup("xdg-open");
    Closure->methodList  = g_ptr_array_new();
    Closure->methodName  = g_strdup("RS01");
+   Closure->dDumpDir    = g_strdup("/tmp");
+   Closure->dDumpPrefix = g_strdup("sector-");
    Closure->cacheMB     = 32;
+   Closure->codecThreads = 1;
+   Closure->minReadAttempts = 1;
+   Closure->maxReadAttempts = 1;
+   Closure->rawMode     = 0x20;
+   Closure->internalAttempts = -1;
    Closure->sectorSkip  = 16;
    Closure->spinupDelay = 5;
    Closure->querySize   = 2;
@@ -506,29 +666,33 @@ void InitClosure()
    Closure->cdSize   = Closure->savedCDSize   = CDR_SIZE;
    Closure->dvdSize1 = Closure->savedDVDSize1 = DVD_SL_SIZE;
    Closure->dvdSize2 = Closure->savedDVDSize2 = DVD_DL_SIZE;
-
-   /*** Align the buffer at a 4096 boundary.
-	Might be needed by some SCSI drivers. */
-
-   Closure->scratchBufBase = g_malloc(32768+4096);
-   Closure->scratchBuf = Closure->scratchBufBase 
-                         + (4096 - ((unsigned long)Closure->scratchBufBase & 4095));
+   Closure->bdSize1  = Closure->savedBDSize1  = BD_SL_SIZE;
+   Closure->bdSize2  = Closure->savedBDSize2  = BD_DL_SIZE;
 
    Closure->logString = g_string_sized_new(1024);
 
    Closure->background = g_malloc0(sizeof(GdkColor));
-   Closure->black = g_malloc0(sizeof(GdkColor));
-   Closure->white = g_malloc0(sizeof(GdkColor));
-   Closure->red   = g_malloc0(sizeof(GdkColor));
-   Closure->yellow= g_malloc0(sizeof(GdkColor));
-   Closure->green = g_malloc0(sizeof(GdkColor));
-   Closure->darkgreen = g_malloc0(sizeof(GdkColor));
-   Closure->blue  = g_malloc0(sizeof(GdkColor));
-   Closure->grid  = g_malloc0(sizeof(GdkColor));
+   Closure->foreground = g_malloc0(sizeof(GdkColor));
+   Closure->grid       = g_malloc0(sizeof(GdkColor));
+
+   Closure->redText     = g_malloc0(sizeof(GdkColor));
+   Closure->greenText   = g_malloc0(sizeof(GdkColor));
+   Closure->barColor    = g_malloc0(sizeof(GdkColor));
+   Closure->logColor    = g_malloc0(sizeof(GdkColor));
+   Closure->curveColor  = g_malloc0(sizeof(GdkColor));
+   Closure->redSector   = g_malloc0(sizeof(GdkColor));
+   Closure->yellowSector= g_malloc0(sizeof(GdkColor));
+   Closure->greenSector = g_malloc0(sizeof(GdkColor));
+   Closure->blueSector  = g_malloc0(sizeof(GdkColor));
+   Closure->whiteSector = g_malloc0(sizeof(GdkColor));
+   Closure->darkSector  = g_malloc0(sizeof(GdkColor));
+
+   DefaultColors();
 
    memset(Closure->bs, '\b', 255);
 
    get_base_dirs();
+   DefaultLogFile();
 
 #ifdef SYS_MINGW
    OpenAspi();
@@ -593,28 +757,43 @@ void FreeClosure()
    cond_free(Closure->logFile);
    cond_free(Closure->binDir);
    cond_free(Closure->docDir);
+   cond_free(Closure->appData);
    cond_free(Closure->browser);
    cond_free(Closure->deadSector);
    cond_free(Closure->errorTitle);
-   cond_free(Closure->scratchBufBase);
+   cond_free(Closure->dDumpDir);
+   cond_free(Closure->dDumpPrefix);
 
    if(Closure->prefsContext)
      FreePreferences(Closure->prefsContext);
 
+   if(Closure->rawEditorContext)
+      FreeRawEditorContext(Closure->rawEditorContext);
+
    if(Closure->logString)
      g_string_free(Closure->logString, TRUE);
 
    if(Closure->drawGC)
      g_object_unref(Closure->drawGC);
-   cond_free(Closure->red);
-   cond_free(Closure->yellow);
-   cond_free(Closure->green);
-   cond_free(Closure->darkgreen);
-   cond_free(Closure->blue);
-   cond_free(Closure->black);
-   cond_free(Closure->white);
-   cond_free(Closure->grid);
+
    cond_free(Closure->background);
+   cond_free(Closure->foreground);
+   cond_free(Closure->grid);
+   cond_free(Closure->redText);
+   cond_free(Closure->greenText);
+   cond_free(Closure->barColor);
+   cond_free(Closure->logColor);
+   cond_free(Closure->curveColor);
+   cond_free(Closure->redSector);
+   cond_free(Closure->yellowSector);
+   cond_free(Closure->greenSector);
+   cond_free(Closure->blueSector);
+   cond_free(Closure->whiteSector);
+   cond_free(Closure->darkSector);
+
+   cond_free(Closure->redMarkup);
+   cond_free(Closure->greenMarkup);
+   cond_free(Closure->invisibleDash);
 
    if(Closure->readLinearCurve)
      FreeCurve(Closure->readLinearCurve);
