@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2006 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2008 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -122,11 +122,16 @@ typedef enum
    MODE_SEQUENCE, 
 
    MODE_BYTESET, 
+   MODE_COPY_SECTOR,
+   MODE_CMP_IMAGES,
    MODE_ERASE, 
    MODE_LIST_ASPI,
    MODE_MARKED_IMAGE,
+   MODE_MERGE_IMAGES,
    MODE_RANDOM_ERR, 
    MODE_RANDOM_IMAGE,
+   MODE_RAW_SECTOR,
+   MODE_READ_SECTOR,
    MODE_SEND_CDB,
    MODE_SHOW_SECTOR, 
    MODE_SIGN,
@@ -138,11 +143,21 @@ typedef enum
    MODIFIER_CACHE_SIZE, 
    MODIFIER_CLV_SPEED,    /* unused */ 
    MODIFIER_CAV_SPEED,    /* unused */
+   MODIFIER_CDUMP, 
    MODIFIER_DAO, 
    MODIFIER_DEBUG,
+   MODIFIER_DEFECTIVE_DUMP,
+   MODIFIER_EJECT,
    MODIFIER_FILL_UNREADABLE,
+   MODIFIER_IGNORE_FATAL_SENSE,
+   MODIFIER_INTERNAL_REREADS,
    MODIFIER_QUERY_SIZE,
    MODIFIER_RANDOM_SEED,
+   MODIFIER_READ_ATTEMPTS,
+   MODIFIER_READ_MEDIUM,
+   MODIFIER_READ_RAW,
+   MODIFIER_RAW_MODE,
+   MODIFIER_SCREEN_SHOT,
    MODIFIER_SIMULATE_DEFECTS,
    MODIFIER_SPEED_WARNING, 
    MODIFIER_SPINUP_DELAY, 
@@ -159,9 +174,12 @@ int main(int argc, char *argv[])
    char *read_range = NULL;
 #ifdef WITH_NLS_YES
    char *locale_test;
- #ifndef SYS_MINGW
+ #ifdef WITH_EMBEDDED_SRC_PATH_YES
+  #ifndef SYS_MINGW
    char src_locale_path[strlen(SRCDIR)+10];
- #else
+  #endif
+ #endif /* WITH_EMBEDDED_SRC_PATH_YES */
+ #ifdef SYS_MINGW
    char *bin_locale_path = NULL;
  #endif
 #endif
@@ -183,6 +201,11 @@ int main(int argc, char *argv[])
 
     InitClosure();
 
+    /*** Setup for multithreading */
+
+    g_thread_init(NULL);
+    Closure->mainThread = g_thread_self();
+
     /*** Setup the locale.
          Try the local source directory first (it may be more recent),
          then fall back to the global installation directory.
@@ -226,6 +249,15 @@ int main(int argc, char *argv[])
 #endif
 	     break;
 
+          case LANG_RUSSIAN:
+	     g_setenv("LANG", "ru_RU", 1);
+#ifdef WIN_CONSOLE
+	     g_setenv("OUTPUT_CHARSET", "CP855", 1);
+#else
+	     g_setenv("OUTPUT_CHARSET", "CP1251", 1);
+#endif
+	     break;
+
           case LANG_SWEDISH:
 	     g_setenv("LANG", "sv_SV", 1);
 #ifdef WIN_CONSOLE
@@ -245,9 +277,13 @@ int main(int argc, char *argv[])
 #ifndef SYS_MINGW
     /* Try local source directory first */
 
+#ifdef WITH_EMBEDDED_SRC_PATH_YES
     g_sprintf(src_locale_path,"%s/locale",SRCDIR);
     bindtextdomain("dvdisaster", src_locale_path);
 //printf("testing src %s\n", src_locale_path);
+
+#endif /* WITH_EMBEDDED_SRC_PATH_YES */
+
     /* TRANSLATORS: 
        This is a dummy entry which is supposed to translate into "ok".
        Please do not return anything else here. */
@@ -297,7 +333,7 @@ int main(int argc, char *argv[])
 	wrong packing. */
 
    if(sizeof(EccHeader) != 4096)
-     Stop(_("sizeof(EccHeader) is %d, but must be 4096.\n"),sizeof(EccHeader));
+     Stop("sizeof(EccHeader) is %d, but must be 4096.\n", sizeof(EccHeader));
 
    /*** Determine the default device (OS dependent!) */
 
@@ -314,33 +350,48 @@ int main(int argc, char *argv[])
       { {"adaptive-read", 0, 0, MODIFIER_ADAPTIVE_READ},
 	{"auto-suffix", 0, 0,  MODIFIER_AUTO_SUFFIX},
 	{"byteset", 1, 0, MODE_BYTESET },
+	{"copy-sector", 1, 0, MODE_COPY_SECTOR },
+	{"compare-images", 1, 0, MODE_CMP_IMAGES },
 	{"cache-size", 1, 0, MODIFIER_CACHE_SIZE },
 	{"cav", 1, 0, MODIFIER_CAV_SPEED },
+	{"cdump", 0, 0, MODIFIER_CDUMP },
 	{"clv", 1, 0, MODIFIER_CLV_SPEED },
 	{"create", 0, 0, 'c'},
 	{"dao", 0, 0, MODIFIER_DAO },
-	{"debug", 0, 0, MODIFIER_DEBUG }, 
+	{"debug", 0, 0, MODIFIER_DEBUG },
+	{"defective-dump", 1, 0, MODIFIER_DEFECTIVE_DUMP },
 	{"device", 0, 0, 'd'},
         {"ecc", 1, 0, 'e'},
+	{"eject", 0, 0, MODIFIER_EJECT },
 	{"erase", 1, 0, MODE_ERASE },
 	{"fill-unreadable", 1, 0, MODIFIER_FILL_UNREADABLE },
 	{"fix", 0, 0, 'f'},
 	{"help", 0, 0, 'h'},
+	{"ignore-fatal-sense", 0, 0, MODIFIER_IGNORE_FATAL_SENSE },
+	{"internal-rereads", 1, 0, MODIFIER_INTERNAL_REREADS },
         {"image", 1, 0, 'i'},
 	{"jump", 1, 0, 'j'},
 #ifdef SYS_MINGW
 	{"list", 0, 0, 'l' },
 #endif
 	{"marked-image", 1, 0, MODE_MARKED_IMAGE },
+	{"merge-images", 1, 0, MODE_MERGE_IMAGES },
 	{"method", 2, 0, 'm' },
-	{"query-size", 1, 0, MODIFIER_QUERY_SIZE },
         {"prefix", 1, 0, 'p'},
+	{"query-size", 1, 0, MODIFIER_QUERY_SIZE },
 	{"random-errors", 1, 0, MODE_RANDOM_ERR },
 	{"random-image", 1, 0, MODE_RANDOM_IMAGE },
 	{"random-seed", 1, 0, MODIFIER_RANDOM_SEED },
+	{"raw-mode", 1, 0, MODIFIER_RAW_MODE },
+	{"raw-sector", 1, 0, MODE_RAW_SECTOR},
 	{"read", 2, 0,'r'},
+	{"read-attempts", 1, 0, MODIFIER_READ_ATTEMPTS },
+	{"read-medium", 1, 0, MODIFIER_READ_MEDIUM },
+	{"read-sector", 1, 0, MODE_READ_SECTOR},
+	{"read-raw", 0, 0, MODIFIER_READ_RAW},
 	{"redundancy", 1, 0, 'n'},
 	{"scan", 2, 0,'s'},
+	{"screen-shot", 0, 0, MODIFIER_SCREEN_SHOT },
 	{"send-cdb", 1, 0, MODE_SEND_CDB},
 	{"show-sector", 1, 0, MODE_SHOW_SECTOR},
 	{"sign", 0, 0, MODE_SIGN},
@@ -349,6 +400,7 @@ int main(int argc, char *argv[])
 	{"spinup-delay", 1, 0, MODIFIER_SPINUP_DELAY},
 	{"split-files", 0, 0, MODIFIER_SPLIT_FILES},
 	{"test", 0, 0, 't'},
+        {"threads", 1, 0, 'x'},
 	{"truncate", 2, 0, MODIFIER_TRUNCATE},
 	{"unlink", 0, 0, 'u'},
        	{"verbose", 0, 0, 'v'},
@@ -358,7 +410,7 @@ int main(int argc, char *argv[])
       };
 
       c = getopt_long(argc, argv, 
-		      "cd:e:fhi:j:lm::n:p:r::s::tuv",
+		      "cd:e:fhi:j:lm::n:p:r::s::tuvx:",
 		      long_options, &option_index);
 
       if(c == -1) break;
@@ -424,6 +476,11 @@ int main(int argc, char *argv[])
          case 'v': Closure->verbose = TRUE;
 	           break;
 
+         case 'x': Closure->codecThreads = atoi(optarg);
+                   if(Closure->codecThreads < 1 || Closure->codecThreads > MAX_CODEC_THREADS)
+                     Stop(_("--threads must be 1..%d\n"), MAX_CODEC_THREADS);
+                   break;
+
          case  0 : break; /* flag argument */
 
          case MODIFIER_ADAPTIVE_READ:
@@ -434,17 +491,34 @@ int main(int argc, char *argv[])
 	   break;
          case MODIFIER_CACHE_SIZE:
 	   Closure->cacheMB = atoi(optarg);
-	   if(Closure->cacheMB <   1) 
-	     Stop(_("--cache-size must at least be 1MB; 16MB or higher is recommended.")); 
+	   if(Closure->cacheMB <   8) 
+	     Stop(_("--cache-size must at least be 8MB; 16MB or higher is recommended.")); 
 	   if(Closure->cacheMB > 2048) 
 	     Stop(_("--cache-size maximum is 2048MB.")); 
 	   break;
+         case MODIFIER_CDUMP:
+	   Closure->debugCDump = TRUE;
+	   break;
          case MODIFIER_DAO: 
 	   Closure->noTruncate = 1; 
 	   break;
+         case MODIFIER_EJECT: 
+	   Closure->eject = 1; 
+	   break;
          case MODIFIER_FILL_UNREADABLE:
 	   if(optarg) Closure->fillUnreadable = strtol(optarg, NULL, 0);
 	   break;
+         case MODIFIER_IGNORE_FATAL_SENSE:
+	   Closure->ignoreFatalSense = TRUE;
+	   break;
+	 case MODIFIER_INTERNAL_REREADS:
+	    if(optarg)
+	       Closure->internalAttempts = atoi(optarg);
+	    if(Closure->internalAttempts < 0) 
+	       Closure->internalAttempts = -1;
+	    if(Closure->internalAttempts > 10) 
+	       Closure->internalAttempts = 10;
+	    break;
          case MODIFIER_TRUNCATE: 
 	   if(optarg)                  /* debugging truncate mode */
 	   {  mode = MODE_TRUNCATE;
@@ -455,6 +529,19 @@ int main(int argc, char *argv[])
          case MODIFIER_DEBUG:
 	   Closure->debugMode = TRUE;
 	   break;
+         case MODIFIER_DEFECTIVE_DUMP:
+	 {  char *c;
+	    Closure->defectiveDump = TRUE;
+	    g_free(Closure->dDumpDir);
+	    Closure->dDumpDir = g_strdup(optarg);
+	    c = strrchr(Closure->dDumpDir, '/');
+	    if(c)
+	    {  *c = 0;
+	       g_free(Closure->dDumpPrefix);
+	       Closure->dDumpPrefix = g_strdup(c+1);
+	    }
+	 }
+	   break;
          case MODIFIER_QUERY_SIZE:
 	        if(!strcmp(optarg, "drive")) Closure->querySize = 0;
 	   else if(!strcmp(optarg, "udf"))   Closure->querySize = 1;
@@ -464,6 +551,41 @@ int main(int argc, char *argv[])
          case MODIFIER_RANDOM_SEED:
 	   if(optarg) Closure->randomSeed = atoi(optarg);
 	   break;
+         case MODIFIER_RAW_MODE:
+	    if(optarg) Closure->rawMode = strtol(optarg,NULL,16);
+	   break;
+         case MODIFIER_READ_ATTEMPTS:
+	   if(optarg) 
+	   {  char copy[strlen(optarg)+1];
+	      char *cpos;
+
+	      strcpy(copy, optarg);
+	      cpos = strchr(copy,'-');
+
+	      if(!cpos)
+	      {  Closure->minReadAttempts = Closure->maxReadAttempts = atoi(optarg); 
+	      }
+	      else
+	      {  *cpos = 0;
+		 Closure->minReadAttempts = atoi(copy);
+		 Closure->maxReadAttempts = atoi(cpos+1);
+	      }
+	      
+	      if(Closure->minReadAttempts < 1)
+		Closure->minReadAttempts = 1;
+	      if(Closure->maxReadAttempts < Closure->minReadAttempts)
+		Closure->maxReadAttempts = Closure->minReadAttempts;
+	   }
+	   break;
+         case MODIFIER_READ_MEDIUM:
+	   Closure->readingPasses = atoi(optarg);
+	   break;
+         case MODIFIER_READ_RAW:
+	   Closure->readRaw = TRUE;
+	   break;
+         case MODIFIER_SCREEN_SHOT:
+	   Closure->screenShotMode = TRUE;
+	   break;
          case MODIFIER_SIMULATE_DEFECTS:
 	   if(optarg) Closure->simulateDefects = atoi(optarg);
 	   else Closure->simulateDefects = 10;
@@ -485,7 +607,7 @@ int main(int argc, char *argv[])
 	   Closure->driveSpeed = -atoi(optarg);
 	   break;
          case MODIFIER_VERSION:
-	   PrintCLI(_("\ndvdisaster version %s\n\n"), VERSION);
+	   PrintCLI(_("\ndvdisaster version %s\n\n"), Closure->cookedVersion);
 	   FreeClosure();
 	   exit(EXIT_SUCCESS); 
 	   break;
@@ -493,6 +615,14 @@ int main(int argc, char *argv[])
 	   mode = MODE_BYTESET;
 	   debug_arg = g_strdup(optarg);
 	   break;
+	 case MODE_CMP_IMAGES:
+	   mode = MODE_CMP_IMAGES;
+	   debug_arg = g_strdup(optarg);
+	   break;
+	 case MODE_COPY_SECTOR:
+	   mode = MODE_COPY_SECTOR;
+	   debug_arg = g_strdup(optarg);
+	   break;
          case MODE_ERASE: 
 	   mode = MODE_ERASE;
 	   debug_arg = g_strdup(optarg);
@@ -501,6 +631,10 @@ int main(int argc, char *argv[])
 	   mode = MODE_MARKED_IMAGE;
 	   debug_arg = g_strdup(optarg);
 	   break;
+         case MODE_MERGE_IMAGES:
+	   mode = MODE_MERGE_IMAGES;
+	   debug_arg = g_strdup(optarg);
+	   break;
          case MODE_RANDOM_ERR:
 	   mode = MODE_RANDOM_ERR;
 	   debug_arg = g_strdup(optarg);
@@ -509,6 +643,14 @@ int main(int argc, char *argv[])
 	   mode = MODE_RANDOM_IMAGE;
 	   debug_arg = g_strdup(optarg);
 	   break;
+         case MODE_RAW_SECTOR:
+	   mode = MODE_RAW_SECTOR;
+	   debug_arg = g_strdup(optarg);
+	   break;
+         case MODE_READ_SECTOR:
+	   mode = MODE_READ_SECTOR;
+	   debug_arg = g_strdup(optarg);
+	   break;
          case MODE_SEND_CDB: 
 	   mode = MODE_SEND_CDB;
 	   debug_arg = g_strdup(optarg);
@@ -533,12 +675,17 @@ int main(int argc, char *argv[])
    if(!Closure->debugMode)
      switch(mode)
      {  case MODE_BYTESET:
+	case MODE_COPY_SECTOR:
+	case MODE_CMP_IMAGES:
         case MODE_ERASE:
+        case MODE_RANDOM_ERR:
+        case MODE_RANDOM_IMAGE:
+        case MODE_READ_SECTOR:
+        case MODE_RAW_SECTOR:
         case MODE_SEND_CDB:
         case MODE_SHOW_SECTOR:
         case MODE_MARKED_IMAGE:
-        case MODE_RANDOM_ERR:
-        case MODE_RANDOM_IMAGE:
+        case MODE_MERGE_IMAGES:
         case MODE_SIGN:
         case MODE_TRUNCATE:
         case MODE_ZERO_UNREADABLE:
@@ -625,6 +772,14 @@ int main(int argc, char *argv[])
          Byteset(debug_arg);
 	 break;
 
+      case MODE_CMP_IMAGES:
+	 MergeImages(debug_arg, FALSE);
+	 break;
+
+      case MODE_COPY_SECTOR:
+	 CopySector(debug_arg);
+	 break;
+
       case MODE_ERASE:
          Erase(debug_arg);
 	 break;
@@ -633,6 +788,14 @@ int main(int argc, char *argv[])
 	 SendCDB(debug_arg);
 	 break;
 
+      case MODE_RAW_SECTOR:
+	 RawSector(debug_arg);
+	 break;
+
+      case MODE_READ_SECTOR:
+	 ReadSector(debug_arg);
+	 break;
+
       case MODE_SHOW_SECTOR:
 	 ShowSector(debug_arg);
 	 break;
@@ -645,6 +808,10 @@ int main(int argc, char *argv[])
 	 RandomImage(Closure->imageName, debug_arg, 1);
 	 break;
 
+      case MODE_MERGE_IMAGES:
+	 MergeImages(debug_arg, TRUE);
+	 break;
+
       case MODE_RANDOM_IMAGE:
 	RandomImage(Closure->imageName, debug_arg, 0);
 	 break;
@@ -711,12 +878,20 @@ int main(int argc, char *argv[])
 	     "  -n,--redundancy n%%     - error correction file redundancy (in percent), or\n"
 	     "                           maximum error correction image size (in sectors)\n"
 	     "  -v,--verbose           - more diagnostic messages\n"
+//             "  -x, --threads n        - use n threads for en-/decoding (if supported by codec)\n"
 	     "  --adaptive-read        - use optimized strategy for reading damaged media\n"
 	     "  --auto-suffix          - automatically add .iso and .ecc file suffixes\n"
 	     "  --cache-size n         - image cache size in MB during -c mode (default: 32MB)\n"
 	     "  --dao                  - assume DAO disc; do not trim image end\n"
+	     "  --eject                - eject medium after successful read\n"
 	     "  --fill-unreadable n    - fill unreadable sectors with byte n\n"
-      	     "  --query-size n         - query drive/udf/ecc for image size (default: ecc)\n"         
+	     "  --ignore-fatal-sense   - continue reading after potentially fatal error conditon\n"
+	     "  --internal-rereads n   - drive may attempt n rereads before reporting an error\n"
+      	     "  --query-size n         - query drive/udf/ecc for image size (default: ecc)\n"   
+	     "  --raw-mode n           - mode for raw reading CD media (20 or 21)\n"
+	     "  --read-attempts n-m    - attempts n upto m reads of a defective sector\n"
+	     "  --read-medium n        - read the whole medium up to n times\n"
+	     "  --read-raw             - performs read in raw mode if possible\n"
 	     "  --speed-warning n      - print warning if speed changes by more than n percent\n"
 	     "  --spinup-delay n       - wait n seconds for drive to spin up\n"
 	     "  --split-files          - split image into files <= 2GB\n\n"));
@@ -725,14 +900,22 @@ int main(int argc, char *argv[])
       { PrintCLI(_("Debugging options (purposefully undocumented and possibly harmful)\n"
 	     "  --debug           - enables the following options\n"
 	     "  --byteset s,i,b   - set byte i in sector s to b\n"
+             "  --cdump           - creates C #include file dumps instead of hexdumps\n" 
+	     "  --compare-images a,b  - compare sectors in images a and b\n"
+	     "  --copy-sector a,n,b,m - copy sector n from image a to sector m in image b\n"
+	     "  --defective-dump p  creates C #include file dumps for unrecoverable sectors\n"
 	     "  --erase sector    - erase the given sector\n"
 	     "  --erase n-m       - erase sectors n - m, inclusively\n"
 	     "  --marked-image n  - create image with n marked random sectors\n"
+	     "  --merge-images a,b  merge image a with b (a receives sectors from b)\n"
 	     "  --random-errors r,e seed image with (correctable) random errors\n"
 	     "  --random-image n  - create image with n sectors of random numbers\n"
 	     "  --random-seed n   - random seed for built-in random number generator\n"
+	     "  --raw-sector n    - shows hexdump of the given raw sector from medium in drive\n"
+	     "  --read-sector n   - shows hexdump of the given sector from medium in drive\n"
+	     "  --screen-shot     - useful for generating screen shots\n"
 	     "  --send-cdb arg    - executes given cdb at drive; kills system if used wrong\n"
-	     "  --show-sector n   - shows hexdump of the given sector\n"
+	     "  --show-sector n   - shows hexdump of the given sector in an image file\n"
 	     "  --sim-defects n   - simulate n%% defective sectors on medium\n"
 	     "  --truncate n      - truncates image to n sectors\n"
 	     "  --zero-unreadable - replace the \"unreadable sector\" markers with zeros\n\n"));
@@ -754,6 +937,17 @@ int main(int argc, char *argv[])
 
    if(mode == MODE_NONE)
    {  
+
+      if(Closure->screenShotMode)
+      {  GPtrArray *a=Closure->deviceNames;
+	 int i;
+	 for(i=0; i<a->len; i++)
+	 {  char *p = g_ptr_array_index(a,i);
+	    g_free(p);
+	    a->pdata[i] = g_strdup(_("Optical drive 52X FW 1.02"));
+	 }
+      }
+
       Closure->guiMode = TRUE;
       ReadDotfile();
       CreateMainWindow(&argc, &argv);
