@@ -1,5 +1,6 @@
+
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2006 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2008 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -97,6 +98,18 @@
 #define CDR_SIZE         (351*1024)
 #define DVD_SL_SIZE      2295104  /* DVD+R/RW size used at least common denominator */
 #define DVD_DL_SIZE 	 4171712  /* also seen: 4148992 4173824  */
+#define BD_SL_SIZE      11826176
+#define BD_DL_SIZE	21826176  /* whats the right value here? */
+
+/* Maximum accepted media sizes (in 2K sectors) */
+
+#define MAX_CDR_SIZE     (100*60*75)     /* CDs can't have >100min w/o severe hacks  */
+#define MAX_DVD_SL_SIZE  2500000         /* I have to guess here */
+#define MAX_DVD_DL_SIZE  4600000         /* I have to guess here */
+
+/* Maximum number of parallel encoder/decoder threads */
+
+#define MAX_CODEC_THREADS 32             /* not including IO and GUI */
 
 /***
  *** Our global closure (encapsulation of global variables)
@@ -116,15 +129,25 @@ typedef struct _GlobalClosure
    gint64 readStart;    /* Range to read */
    gint64 readEnd;
    gint64 cdSize;       /* Maximum cd size (for RS02 type images) */
-   gint64 dvdSize1;     /* Maximum 1-layer dvd size (for RS02 type images) */
-   gint64 dvdSize2;     /* Maximum 2-layer dvd size (for RS02 type images) */
+   gint64 dvdSize1;     /* Maximum 1-layer dvd size (for augmented images) */
+   gint64 dvdSize2;     /* Maximum 2-layer dvd size (for augmented images) */
+   gint64 bdSize1;      /* Maximum 1-layer dvd size (for augmented images) */
+   gint64 bdSize2;      /* Maximum 2-layer dvd size (for augmented images) */
    gint64 savedCDSize;  /* Undo values for above */
    gint64 savedDVDSize1;
    gint64 savedDVDSize2;
-   gint64 mediumSize;   /* Maximum medium size (for RS02 type images) */
+   gint64 savedBDSize1;
+   gint64 savedBDSize2;
+   gint64 mediumSize;   /* Maximum medium size (for augmented images) */
    int cacheMB;         /* Cache setting for the parity codec, in megabytes */
+   int codecThreads;    /* Number of threads to use for RS encoders */
    int sectorSkip;      /* Number of sectors to skip after read error occurs */
    char *redundancy;    /* Error correction code redundancy */
+   int readRaw;         /* Read CD sectors raw + verify them */
+   int rawMode;         /* mode for mode page */
+   int minReadAttempts; /* minimum reading attempts */
+   int maxReadAttempts; /* maximal reading attempts */
+   int internalAttempts;/* read attempts by the drive itself */
    int adaptiveRead;    /* Use optimized strategy for reading defective images */
    int speedWarning;    /* Print warning if speed changes by more than given percentage */
    int fillUnreadable;  /* Byte value for filling unreadable sectors or -1 */
@@ -134,7 +157,9 @@ typedef struct _GlobalClosure
    int unlinkImage;     /* delete image after ecc file creation */
    int driveSpeed;      /* currently unused */
    int debugMode;       /* may activate additional features */
+   int debugCDump;      /* dump as #include file instead of hexdump */
    int verbose;         /* may activate additional messages */
+   int screenShotMode;  /* screen shot mode */
    int splitFiles;      /* limit image files to 2GB */
    int autoSuffix;      /* automatically extend files with suffices .iso/.ecc */
    int querySize;       /* what sources are used for image size queries */
@@ -143,12 +168,25 @@ typedef struct _GlobalClosure
    int welcomeMessage;  /* just print dvdisaster logo if FALSE */
    int dotFileVersion;  /* version of dotfile */
    int simulateDefects; /* if >0, this is the percentage of simulated media defects */
+   int defectiveDump;   /* dump non-recoverable sectors into given path */
+   char *dDumpDir;      /* directory for above */
+   char *dDumpPrefix;   /* file name prefix for above */
+   int reverseCancelOK; /* if TRUE the button order is reversed */
+   int eject;           /* eject medium on success */
+   int readingPasses;   /* try to read medium n times */
+   int pauseAfter;      /* pause after given amount of minutes */
+   int pauseDuration;   /* duration of pause in minutes */
+   int pauseEject;      /* Eject medium during pause */
+   int ignoreFatalSense;/* Continue reading after potential fatal sense errors */
   
    char *deadSector;    /* Copy of our "dead sector" marker. */
    char *dotFile;       /* path to .dvdisaster file */
    char *logFile;       /* path to logfile */
+   int  logFileEnabled; /* logfile enabled */
+   int  logFileStamped; /* time stamp written to log file */
    char *binDir;        /* place where the binary resides */
    char *docDir;        /* place where our documentation resides */
+   char *appData;       /* Windows specific */
    char *browser;       /* Name of preferred WEB browser */
 
    char bs[256];        /* A string of 255 backspace characters */
@@ -167,9 +205,6 @@ typedef struct _GlobalClosure
    unsigned long (*SendASPI32Command)(void*);
 #endif
 
-   unsigned char *scratchBufBase;  /* scratch buffer for use with SCSI commands; */
-   unsigned char *scratchBuf;      /* 32768 bytes aligned at a 4096 byte boundary */
-
    guint32 *crcCache;              /* sectorwise CRC32 for last image read */
    char    *crcImageName;          /* file name of cached image */
    unsigned char md5Cache[16];     /* md5sum of last image read */
@@ -191,6 +226,8 @@ typedef struct _GlobalClosure
    GtkTooltips *tooltips;    /* our global tooltips structure */
    GdkPixbuf *windowIcon;    /* main window icon */
 
+   GtkWidget *driveCombo;    /* combo box for drive selection */
+
    GtkWidget *imageFileSel;  /* image file selector */
    GtkWidget *imageEntry;    /* image name entry field */
    GtkWidget *eccFileSel;    /* ecc file selector */
@@ -213,16 +250,34 @@ typedef struct _GlobalClosure
    GtkWindow *prefsWindow;
    void *prefsContext;       /* local data for the preferences window */
 
+   /*** The raw editor window */
+
+   void *rawEditorContext;
+
+   /*** The medium info window */
+
+   GtkWidget *mediumWindow;   /* Dialog for the medium info window */
+   GtkWidget *mediumDrive;
+   void *mediumInfoContext;   /* private data */
+
    /*** Common stuff for drawing curves and spirals */
 
    GdkGC     *drawGC;
-   GdkColor  *background,*grid,*black,*white;
-   GdkColor  *red,*yellow,*green,*darkgreen,*blue;
-   gint      lastPercent;
-   gint      lastSegment;
-   gint      lastPlotted;
-   gint      lastPlottedY;
-   gint      percent; 
+   GdkColor  *background,*foreground,*grid;
+   GdkColor  *redText;
+   char      *redMarkup;
+   GdkColor  *greenText;
+   char      *greenMarkup;
+   GdkColor  *barColor;
+   GdkColor  *logColor;
+   GdkColor  *curveColor;
+   GdkColor  *redSector;
+   GdkColor  *yellowSector;
+   GdkColor  *greenSector;
+   GdkColor  *blueSector;
+   GdkColor  *whiteSector;
+   GdkColor  *darkSector;
+   char      *invisibleDash;
 
    /*** Widgets for the linear reading/scanning action */
 
@@ -311,6 +366,8 @@ typedef struct _EccInfo
 #define MFLAG_DEVEL (1<<0)    /* for methodFlags[3] */
 #define MFLAG_RC    (1<<1)                      
 
+#define MFLAG_DATA_MD5 (1<<0)  /* specific to RS03 */
+
 typedef struct _EccHeader
 {  gint8 cookie[12];           /* "*dvdisaster*" */
    gint8 method[4];            /* e.g. "RS01" */
@@ -333,6 +390,12 @@ typedef struct _EccHeader
            for a copy of the first ecc blocks CRC sums. */
 } EccHeader;
 
+/***
+ *** forward declarations
+ ***/
+
+struct _RawBuffer *rawbuffer_forward;
+
 /***
  *** dvdisaster.c
  ***/
@@ -365,6 +428,8 @@ void FreeBitmap(Bitmap*);
  ***/
 
 void InitClosure(void);
+void UpdateMarkup(char**, GdkColor*);
+void DefaultColors(void);
 void ClearCrcCache(void);
 void FreeClosure(void);
 void ReadDotfile(void);
@@ -376,6 +441,29 @@ int  VerifySignature(void);
  ***/
 
 guint32 Crc32(unsigned char*, int);
+guint32 EDCCrc32(unsigned char*, int);
+
+/***
+ *** crcbuf.c
+ ***/
+
+typedef struct _CrcBuf
+{  guint32 *crcbuf;
+   gint64 size;
+   Bitmap *valid;
+} CrcBuf;
+
+enum
+{  CRC_UNKNOWN,
+   CRC_BAD,
+   CRC_GOOD
+};
+
+CrcBuf *GetCRCFromRS01(EccInfo*);
+CrcBuf *GetCRCFromRS02(void*, void*, LargeFile*);
+void FreeCrcBuf(CrcBuf*);
+
+int CheckAgainstCrcBuffer(CrcBuf*, gint64, unsigned char*);
 
 /***
  *** curve.c
@@ -390,19 +478,24 @@ typedef struct _Curve
 {  GtkWidget *widget;   /* drawing area which is hosting us */
    PangoLayout *layout;
    gdouble *fvalue;     /* floating point curve values */
-   gint    *ivalue;     /* integer curve values */
+   gint    *ivalue;     /* integer bar curve values */
+   gint    *lvalue;     /* logarithmic integer curve */
+   gint    enable;      /* which of the above should be drawn */
    gint lastValueIdx;   /* end of value array */
    gint leftX,rightX;   /* Position of left and right y axis */
-   gint topY,bottomY;   /* Top and bottom end of y axes */
+   gint topY,bottomY;   /* Top and bottom end of i/f y axes */
+   gint topLY,bottomLY; /* Top and bottom end of l y axes */
    char *leftLabel;     /* Label of left coordinate axis */
+   char *leftLogLabel;  /* Label of left log coordinate axis */
    char *leftFormat;    /* Format of left coordinate axis numbering */
    int bottomFormat;    /* what kind of data are we displaying? */
    int margin;
-   int maxX,maxY;       /* current max value at y axis */
+   int maxX,maxY,logMaxY; /* current max value at y axis */
 } Curve;
 
-#define REDRAW_ICURVE (1<<0)
-#define REDRAW_FCURVE (1<<1)
+#define DRAW_ICURVE (1<<0)
+#define DRAW_FCURVE (1<<1)
+#define DRAW_LCURVE (1<<2)
 
 Curve* CreateCurve(GtkWidget*, char*, char*, int, int);
 void ZeroCurve(Curve*);
@@ -412,17 +505,23 @@ void UpdateCurveGeometry(Curve*, char*, int);
 
 int  CurveX(Curve*, gdouble);
 int  CurveY(Curve*, gdouble);
+int  CurveLogY(Curve*, gdouble);
 void RedrawAxes(Curve*);
-void RedrawCurve(Curve*, int, int);
+void RedrawCurve(Curve*, int);
 
 /***
  *** debug.c
  ***/
 
+void HexDump(unsigned char*, int, int);
+void CopySector(char*);
 void Byteset(char*);
 void Erase(char*);
+void MergeImages(char*, int);
 void RandomError(char*, char*);
 void RandomImage(char*, char*, int);
+void RawSector(char*);
+void ReadSector(char*);
 void SendCDB(char*);
 void ShowSector(char*);
 Bitmap* SimulateDefects(gint64);
@@ -476,33 +575,77 @@ void CreateFixWindow(GtkWidget*);
  * be #included from galois-inlines.h
  */  
 
-#define SYMBOLSIZE 8
-#define FIELDSIZE (1<<SYMBOLSIZE)
-#define FIELDMAX (FIELDSIZE-1)
-#define ALPHA0 FIELDMAX
+/* Galois field parameters for 8bit symbol Reed-Solomon code */
 
-#define GENERATOR_POLY 0x187    /* 1 + X + X**2 + X**7 + X**8 */
-#define FIRST_ROOT 112          /* same choices as in CCSDS */                 
-#define PRIM_ELEM 11                  
-#define PRIMTH_ROOT 116         /* prim-th root of 1 */           
+#define GF_SYMBOLSIZE 8
+#define GF_FIELDSIZE (1<<GF_SYMBOLSIZE)
+#define GF_FIELDMAX (GF_FIELDSIZE-1)
+#define GF_ALPHA0 GF_FIELDMAX
+
+/* RS polynomial parameters for RS01/RS02 codec */
+
+#define RS_GENERATOR_POLY 0x187    /* 1 + X + X**2 + X**7 + X**8 */
+#define RS_FIRST_ROOT 112          /* same choices as in CCSDS */                 
+#define RS_PRIM_ELEM 11                  
+#define RS_PRIMTH_ROOT 116         /* prim-th root of 1 */           
+
+/* Lookup tables for Galois field arithmetic */
 
 typedef struct _GaloisTables
-{  gint32 nroots;        /* degree of generator polynomial */
-   gint32 ndata;         /* data bytes per ecc block */
-
-   gint32 *index_of;     /* log */
-   gint32 *alpha_to;     /* inverse log */
-   gint32 *enc_alpha_to; /* inverse log optimized for encoder */
-   gint32 *gpoly;        /* generator polynomial */
+{  gint32 gfGenerator;  /* GF generator polynomial */ 
+   gint32 *indexOf;     /* log */
+   gint32 *alphaTo;     /* inverse log */
+   gint32 *encAlphaTo; /* inverse log optimized for encoder */
 } GaloisTables;
 
-GaloisTables* CreateGaloisTables(int);
+/* Lookup and working tables for the ReedSolomon codecs */
+
+typedef struct _ReedSolomonTables
+{  GaloisTables *gfTables;/* from above */
+   gint32 *gpoly;        /* RS code generator polynomial */
+   gint32 fcr;           /* first consecutive root of RS generator polynomial */
+   gint32 primElem;      /* primitive field element */
+   gint32 nroots;        /* degree of RS generator polynomial */
+   gint32 ndata;         /* data bytes per ecc block */
+} ReedSolomonTables;
+
+GaloisTables* CreateGaloisTables(gint32);
 void FreeGaloisTables(GaloisTables*);
 
+ReedSolomonTables *CreateReedSolomonTables(GaloisTables*, gint32, gint32, int);
+void FreeReedSolomonTables(ReedSolomonTables*);
+
 /***
  *** help-dialogs.c
  ***/
 
+/* Creating labels with links to online help */
+
+typedef struct _LabelWithOnlineHelp
+{  GtkWidget *helpWindow;
+   GtkWidget *normalLabel;
+   GtkWidget *linkBox;
+   GtkWidget *linkLabel;
+   GtkWidget *vbox;
+   GPtrArray *lastSizes;  /* for breaking expose loops between the help windows */
+  
+   char *windowTitle;
+   char *normalText;
+   char *highlitText;
+   int inside;
+   int outerPadding;   /* Padding between window and outer vbox */
+} LabelWithOnlineHelp;
+
+LabelWithOnlineHelp* CreateLabelWithOnlineHelp(char*, char*);
+LabelWithOnlineHelp* CloneLabelWithOnlineHelp(LabelWithOnlineHelp*, char*);
+void FreeLabelWithOnlineHelp(LabelWithOnlineHelp*);
+void SetOnlineHelpLinkText(LabelWithOnlineHelp*, char*);
+void AddHelpListItem(LabelWithOnlineHelp*, char*, ...);
+void AddHelpParagraph(LabelWithOnlineHelp*, char*, ...);
+void AddHelpWidget(LabelWithOnlineHelp*, GtkWidget*);
+
+/* Specific online help dialogs */
+
 GtkWidget* ShowTextfile(char*, char*, char*, GtkScrolledWindow**, GtkTextBuffer**);
 void ShowGPL();
 void ShowLog();
@@ -513,6 +656,19 @@ void AboutText(GtkWidget*, char*, ...);
 void AboutLink(GtkWidget*, char*, char*);
 void AboutTextWithLink(GtkWidget*, char*, char*);
 
+/***
+ *** heuristic-lec.c
+ ***/
+
+void UpdateByteCounts(struct _RawBuffer*);
+void CalculatePQLoad(struct _RawBuffer*);
+void UpdatePQParityList(struct _RawBuffer*, unsigned char*);
+
+int HeuristicLEC(unsigned char*, struct _RawBuffer*, unsigned char*);
+int SearchPlausibleSector(struct _RawBuffer*, int);
+int BruteForceSearchPlausibleSector(struct _RawBuffer*);
+int AckHeuristic(struct _RawBuffer*);
+
 /***
  *** icon-factory.c
  ***/
@@ -533,6 +689,54 @@ int LargeTruncate(LargeFile*, gint64);
 int LargeStat(char*, gint64*);
 int LargeUnlink(char*);
 
+int DirStat(char*);
+FILE *portable_fopen(char*, char*);
+int portable_mkdir(char*);
+
+/*** 
+ *** l-ec.c
+ ***/
+
+#define N_P_VECTORS   86      /* 43 16bit p vectors */
+#define P_VECTOR_SIZE 26      /* using RS(26,24) ECC */
+
+#define N_Q_VECTORS   52      /* 26 16bit q vectors */
+#define Q_VECTOR_SIZE 45      /* using RS(45,43) ECC */
+
+#define P_PADDING 229         /* padding values for */
+#define Q_PADDING 210         /* shortened RS code  */
+
+int PToByteIndex(int, int);
+int QToByteIndex(int, int);
+void ByteIndexToP(int, int*, int*);
+void ByteIndexToQ(int, int*, int*);
+
+void PrintVector(unsigned char*, int, int);
+
+void GetPVector(unsigned char*, unsigned char*, int);
+void SetPVector(unsigned char*, unsigned char*, int);
+void FillPVector(unsigned char*, unsigned char, int);
+void AndPVector(unsigned char*, unsigned char, int);
+void OrPVector(unsigned char*, unsigned char, int);
+
+void GetQVector(unsigned char*, unsigned char*, int);
+void SetQVector(unsigned char*, unsigned char*, int);
+void FillQVector(unsigned char*, unsigned char, int);
+void AndQVector(unsigned char*, unsigned char, int);
+void OrQVector(unsigned char*, unsigned char, int);
+
+int DecodePQ(ReedSolomonTables*, unsigned char*, int, int*, int);
+
+int CountC2Errors(unsigned char*);
+
+/***
+ *** logfile.c
+ ***/
+
+void DefaultLogFile();
+void VPrintLogFile(char*, va_list);
+void PrintLogFile(char*, ...);
+
 /***
  *** main-window.c
  ***/
@@ -553,6 +757,12 @@ typedef enum
 void CreateMainWindow(int*, char***);
 void ContinueWithAction(int);
 
+/***
+ *** medium-info.c
+ ***/
+
+void CreateMediumInfoWindow(void);
+
 /***
  *** memtrack.c
  ***/
@@ -662,7 +872,6 @@ void CalcSectors(gint64, gint64*, int*);
 
 void PrintCLI(char*, ...);
 void PrintLog(char*, ...);
-void PrintLogFile(char*, ...);
 void Verbose(char*, ...);
 void PrintTimeToLog(GTimer*, char*, ...);
 void PrintProgress(char*, ...);
@@ -691,6 +900,9 @@ void SwitchAndSetFootline(GtkWidget*, int, GtkWidget*, char*, ...);
 void ReverseCancelOK(GtkDialog*);
 void TimedInsensitive(GtkWidget*, int);
 
+int GetLabelWidth(GtkLabel*, char*, ...);
+void LockLabelSize(GtkLabel*, char*, ...);
+
 /***
  *** preferences.c
  ***/
@@ -701,14 +913,56 @@ void HidePreferences(void);
 void FreePreferences(void*);
 
 void UpdatePrefsQuerySize(void);
+void RegisterPreferencesHelpWindow(LabelWithOnlineHelp*);
 
 /***
- *** print_sense.c
+ *** print-sense.c
  ***/
 
 void RememberSense(int, int, int);
 char *GetSenseString(int, int, int, int);
 char* GetLastSenseString(int);
+void GetLastSense(int*, int*, int*);
+
+/***
+ *** random.c
+ ***/
+
+#define	MY_RAND_MAX	2147483647
+
+gint32  Random(void);
+void    SRandom(gint32);
+guint32 Random32(void);
+
+/***
+ *** raw-editor.c
+ ***/
+
+void CreateRawEditor(void);
+void FreeRawEditorContext(void*);
+
+
+/***
+ *** raw-sector-cache.c
+ ***/
+
+typedef struct _dsh
+{  unsigned char mediumFP[16];       /* Medium fingerprint */
+   gint64 lba;                       /* LBA of this sector */
+   gint32 sectorSize;                /* Sector size in bytes */
+   gint32 properties;                /* Flags for future use */
+   gint32 dshFormat;                 /* Format of this file */
+   gint32 nSectors;                  /* Number of sectors in this file */
+} DefectiveSectorHeader;
+
+enum                                 /* for ->properties above */
+{  DSH_HAS_FINGERPRINT = (1<<0),
+   DSH_XA_MODE         = (1<<1)
+};
+
+int SaveDefectiveSector(struct _RawBuffer*, int);
+int TryDefectiveSectorCache(struct _RawBuffer*, unsigned char*);
+void ReadDefectiveSectorFile(DefectiveSectorHeader *, struct _RawBuffer*, char*);
 
 /*** 
  *** read-linear.c
@@ -723,23 +977,16 @@ void ReadMediumLinear(gpointer);
 void ResetLinearReadWindow();
 void CreateLinearReadWindow(GtkWidget*);
 
-void InitializeCurve(int, gint64, gint64, gint64);
-void AddCurveValues(int, double, int);
-
-/***
- *** random.c
- ***/
-
-#define	MY_RAND_MAX	2147483647
-
-gint32  Random(void);
-void    SRandom(gint32);
-guint32 Random32(void);
+void InitializeCurve(void*, int, int);
+void AddCurveValues(void*, int, int, int);
+void MarkExistingSectors(void);
 
 /*** 
  *** read-adaptive.c
  ***/
 
+void GetReadingRange(gint64, gint64*, gint64*);
+
 void ReadMediumAdaptive(gpointer);
 
 /***
@@ -757,9 +1004,88 @@ void SetAdaptiveReadSubtitle(char*);
 void SetAdaptiveReadFootline(char*, GdkColor*);
 void UpdateAdaptiveResults(gint64, gint64, gint64, int);
 void ChangeSegmentColor(GdkColor*, int);
-void ChangeSpiralCursor(int);
 void RemoveFillMarkers();
 
+/***
+ *** recover-raw.c
+ ***/
+
+#define MAX_RAW_TRANSFER_SIZE (2352+296)  /* main channel plus C2 vector and mask */
+#define CD_RAW_DUMP_SIZE MAX_RAW_TRANSFER_SIZE
+#define CD_RAW_SECTOR_SIZE 2352  
+#define CD_RAW_C2_SECTOR_SIZE (2352+294)  /* main channel plus C2 vector */
+
+typedef struct _RawBuffer
+{  GaloisTables *gt;          /* for L-EC Reed-Solomon */
+   ReedSolomonTables *rt;
+   struct _AlignedBuffer *workBuf; /* working buffer for READ CD */
+   unsigned char *zeroSector; /* a raw sector containing just zeros. */
+   unsigned char **rawBuf;    /* buffer for raw read attempts */
+   int samplesRead;           /* number of samples read */
+   int samplesMax;            /* maximum number of samples we can store */
+   int sampleSize;            /* size of samples */
+   int dataOffset;            /* offset to user data in frame */
+   int xaMode;                /* frame is in XA21 mode */
+   int recommendedAttempts;   /* number of retries recommended by reading heuristics */
+
+   unsigned char *recovered;  /* working buffer for cd frame recovery */
+   unsigned char *byteState;  /* state of error correction */
+   unsigned char *reference;  /* NULL or the correct sector (for debugging purposes) */
+   unsigned char *byteCount;  /* stores how many different bytes were read for each position in a sector */   
+   gint64 lba;                /* sector number were currently working on */
+
+   guint8 mediumFP[16];       /* medium fingerprint for raw sector cache validation */
+   int validFP;               /* indicates valid fingerprint */
+
+   unsigned char *pParity1[N_P_VECTORS];
+   unsigned char *pParity2[N_P_VECTORS];
+   int pParityN[N_P_VECTORS][2];
+
+   unsigned char *qParity1[N_Q_VECTORS];
+   unsigned char *qParity2[N_Q_VECTORS];
+   int qParityN[N_Q_VECTORS][2];
+
+   int *pLoad,*qLoad;
+
+   /* data structures for the smart_lec */
+
+   unsigned char **pList[N_P_VECTORS];  /* List of accepting P vectors */
+   int pn[N_P_VECTORS];
+
+   unsigned char **qList[N_Q_VECTORS];  /* List of accepting Q vectors */
+   int qn[N_Q_VECTORS];
+
+   int bestFrame;                      /* Frame with lowest failures */
+   int bestP1, bestP2, bestQ1, bestQ2;
+
+} RawBuffer;
+
+enum                          /* values for byteState */
+{  FRAME_BYTE_UNKNOWN,        /* state of byte is unknown */
+   FRAME_BYTE_ERROR,          /* byte is wrong (= erasure for ecc) */
+   FRAME_BYTE_GOOD            /* byte is correct */
+};
+
+RawBuffer* CreateRawBuffer(int);
+void ReallocRawBuffer(RawBuffer*, int);
+void ResetRawBuffer(RawBuffer*);
+void FreeRawBuffer(RawBuffer*);
+
+void DumpSector(RawBuffer*, char*);
+
+#define STRICT_MSF_CHECK FALSE
+#define SLOPPY_MSF_CHECK TRUE
+
+int MSFtoLBA(unsigned char, unsigned char, unsigned char);
+int CheckEDC(unsigned char*, int);
+int CheckMSF(unsigned char*, int, int);
+void InitializeCDFrame(unsigned char*, int, int, int);
+
+void UpdateFrameStats(RawBuffer*);
+int ValidateRawSector(RawBuffer*, unsigned char*, char*);
+int IterativeLEC(RawBuffer*);
+int TryCDFrameRecovery(RawBuffer*, unsigned char*);
+
 /*** 
  *** scsi-layer.c
  ***
@@ -767,6 +1093,19 @@ void RemoveFillMarkers();
  * scsi wrapper dependent declarations.
  */
 
+/* Maximum number of sectors per request */
+
+#define MAX_CLUSTER_SIZE (32*2048)
+#define MAX_CLUSTER_SECTORS 32
+
+typedef struct _AlignedBuffer
+{  unsigned char *base;
+   unsigned char *buf;
+} AlignedBuffer;
+
+AlignedBuffer *CreateAlignedBuffer(int);
+void FreeAlignedBuffer(AlignedBuffer*);
+
 void OpenAspi(void);
 void CloseAspi(void);
 void ListAspiDrives(void);
@@ -775,7 +1114,26 @@ char* DefaultDevice(void);
 gint64 CurrentImageSize(void);
 gint64 CurrentImageCapacity(void);
 
-int SendReadCDB(char*, unsigned char*, int, int);
+int SendReadCDB(char*, unsigned char*, unsigned char*, int, int);
+
+/***
+ *** rs-encoder.c
+ ***/
+
+typedef struct _ReedSolomonEncoder
+{  GaloisTables *gfTables;
+   ReedSolomonTables *rsTables;
+
+   guint64 chunkSize;
+   int shiftPtr;
+} ReedSolomonEncoder;
+
+ReedSolomonEncoder *CreateRSEncoder(ReedSolomonTables*, guint64);
+void FreeReedSolomonEncoder(ReedSolomonEncoder*);
+
+void ResetReedSolomonEncoder(ReedSolomonEncoder*, unsigned char*);
+void EncodeNextLayer(ReedSolomonEncoder*, unsigned char*, unsigned char*, guint64);
+
 
 /***
  *** show-manual.c
@@ -783,6 +1141,20 @@ int SendReadCDB(char*, unsigned char*, int, int);
 
 void ShowHTML(char*);
 
+/***
+ *** smart-lec.c
+ ***/
+
+#define SMART_LEC_MESSAGE_SIZE 256
+
+void CollectGoodVectors(RawBuffer*);
+void PrintPQStats(RawBuffer*);
+int SmartLEC(RawBuffer*);
+
+void *PrepareIterativeSmartLEC(RawBuffer*);
+void SmartLECIteration(void*, char*);
+void EndIterativeSmartLEC(void*);
+
 /***
  *** spiral.c
  ***/
@@ -809,6 +1181,8 @@ void FreeSpiral(Spiral*);
 void DrawSpiral(Spiral*);
 void DrawSpiralSegment(Spiral*, GdkColor*, int);
 void DrawSpiralLabel(Spiral*, PangoLayout*, char*, GdkColor*, int, int);
+
+void ChangeSpiralCursor(Spiral*, int);
 void MoveSpiralCursor(Spiral*, int);
 
 /***
@@ -817,5 +1191,4 @@ void MoveSpiralCursor(Spiral*, int);
 
 void CreateWelcomePage(GtkNotebook*);
 
-
 #endif				/* DVDISASTER_H */
