@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2006 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2008 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -72,14 +72,14 @@ static void destroy(Method *method)
 
    if(wl)
    {  if(wl->fixCurve) FreeCurve(wl->fixCurve);
-      if(method->widgetList)
-       g_free(method->widgetList);
 
       if(wl->cmpSpiral)
 	FreeSpiral(wl->cmpSpiral);
 
       if(wl->cmpLayout)
 	g_object_unref(wl->cmpLayout);
+
+      g_free(wl);
    }
 }
 
