@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2006 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2008 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -30,10 +30,10 @@
 
 static inline int mod_fieldmax(int x)
 {
-  while (x >= FIELDMAX) 
+  while (x >= GF_FIELDMAX) 
   {
-    x -= FIELDMAX;
-    x = (x >> SYMBOLSIZE) + (x & FIELDMAX);
+    x -= GF_FIELDMAX;
+    x = (x >> GF_SYMBOLSIZE) + (x & GF_FIELDMAX);
   }
 
   return x;
