@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2006 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2008 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -23,6 +23,298 @@
 
 #include "help-dialogs.h"
 
+/***
+ *** Online help system for the preferences
+ ***/
+
+/*
+ * Create a help window
+ */
+
+/* Close button response */
+
+static void close_cb(GtkWidget *widget, gpointer data)
+{  LabelWithOnlineHelp *lwoh = (LabelWithOnlineHelp*)data;
+
+   gtk_widget_hide(lwoh->helpWindow);
+}
+
+/* Do not destroy the window when close via the window manager */
+
+static gboolean delete_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
+{  LabelWithOnlineHelp *lwoh = (LabelWithOnlineHelp*)data;
+
+   gtk_widget_hide(lwoh->helpWindow);
+
+   return TRUE;
+}
+
+/*
+ * Get a new integer variable from the lastSizes array 
+ */
+
+static int* get_new_int(LabelWithOnlineHelp* lwoh)
+{  int *var = g_malloc0(sizeof(int));
+
+   if(!lwoh->lastSizes)
+      lwoh->lastSizes = g_ptr_array_new();
+
+   g_ptr_array_add(lwoh->lastSizes, var);
+
+   return var;
+}
+
+/*
+ * Callback for the help link
+ */
+
+static gint help_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
+{  GtkWidget *lab = GTK_BIN(widget)->child;
+   LabelWithOnlineHelp *lwoh = (LabelWithOnlineHelp*)data;
+
+   switch(event->type)
+   {  case GDK_BUTTON_PRESS: 
+        if(!lwoh->inside) return FALSE; /* Bug in Gtk for Windows? */
+	gtk_widget_show_all(GTK_WIDGET(lwoh->helpWindow));
+	break; 
+
+      case GDK_ENTER_NOTIFY: 
+	gtk_label_set_markup(GTK_LABEL(lab), lwoh->highlitText);
+	lwoh->inside = TRUE;
+	break;
+
+      case GDK_LEAVE_NOTIFY: 
+	gtk_label_set_markup(GTK_LABEL(lab), lwoh->normalText);
+	lwoh->inside = FALSE;
+	break;
+
+      default: break;
+   }
+
+   return FALSE;
+}
+
+/*
+ * Create a frame labeled with a link to the help system
+ */
+
+LabelWithOnlineHelp* CreateLabelWithOnlineHelp(char *title, char *ascii_text)
+{  GtkWidget *window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+   GtkWidget *vbox, *hbox, *button;
+   GtkWidget *ebox  = gtk_event_box_new();
+   LabelWithOnlineHelp *lwoh;
+ 
+   /*** Initialize online help context */
+
+   lwoh = g_malloc0(sizeof(LabelWithOnlineHelp));
+   lwoh->normalLabel = gtk_label_new(NULL);
+   lwoh->linkLabel = gtk_label_new(NULL);
+   lwoh->linkBox = ebox;
+   lwoh->windowTitle = g_locale_to_utf8(title, -1, NULL, NULL, NULL);
+   SetOnlineHelpLinkText(lwoh, ascii_text);
+
+   gtk_label_set_markup(GTK_LABEL(lwoh->normalLabel), lwoh->normalText);
+
+   /*** Create the help window */
+
+   lwoh->helpWindow = window;
+   gtk_window_set_title(GTK_WINDOW(window), lwoh->windowTitle);
+   gtk_window_set_icon(GTK_WINDOW(window), Closure->windowIcon);
+   gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
+
+   lwoh->outerPadding = 12;
+   gtk_container_set_border_width(GTK_CONTAINER(window), lwoh->outerPadding);
+   lwoh->outerPadding *= 2;
+
+   /* Connect window with the close button from the window manager */
+
+   g_signal_connect(window, "delete_event", G_CALLBACK(delete_cb), lwoh);
+
+   /* Create the main layout of the window */
+
+   lwoh->vbox = vbox = gtk_vbox_new(FALSE, 0);
+   gtk_container_add(GTK_CONTAINER(window), vbox);
+   
+   hbox = gtk_hbox_new(FALSE, 0);
+   gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+
+   button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
+   gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+   g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(close_cb), lwoh);
+
+   gtk_box_pack_end(GTK_BOX(vbox), gtk_hseparator_new(), FALSE, FALSE, 6);
+
+   /*** Put link label into an event box */
+
+   gtk_widget_set_events(ebox, GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK | GDK_BUTTON_PRESS_MASK);
+   g_signal_connect(G_OBJECT(ebox), "button_press_event", G_CALLBACK(help_cb), (gpointer)lwoh);
+   g_signal_connect(G_OBJECT(ebox), "enter_notify_event", G_CALLBACK(help_cb), (gpointer)lwoh);
+   g_signal_connect(G_OBJECT(ebox), "leave_notify_event", G_CALLBACK(help_cb), (gpointer)lwoh);
+
+   gtk_label_set_markup(GTK_LABEL(lwoh->linkLabel), lwoh->normalText);
+   gtk_container_add(GTK_CONTAINER(ebox), lwoh->linkLabel);
+
+   return lwoh;
+}
+
+LabelWithOnlineHelp* CloneLabelWithOnlineHelp(LabelWithOnlineHelp *orig, char *ascii_text)
+{  LabelWithOnlineHelp *lwoh;
+   GtkWidget *ebox  = gtk_event_box_new();
+ 
+   /*** Initialize online help context from given one */
+
+   lwoh = g_malloc0(sizeof(LabelWithOnlineHelp));
+   lwoh->helpWindow = orig->helpWindow;
+
+   /*** Only replace the labels */
+
+   lwoh->normalLabel = gtk_label_new(NULL);
+   lwoh->linkLabel   = gtk_label_new(NULL);
+   lwoh->linkBox     = ebox;
+   lwoh->windowTitle = g_strdup("ignore");
+
+   SetOnlineHelpLinkText(lwoh, ascii_text);
+
+   /*** Put link label into an event box */
+
+   gtk_widget_set_events(ebox, GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK | GDK_BUTTON_PRESS_MASK);
+   g_signal_connect(G_OBJECT(ebox), "button_press_event", G_CALLBACK(help_cb), (gpointer)lwoh);
+   g_signal_connect(G_OBJECT(ebox), "enter_notify_event", G_CALLBACK(help_cb), (gpointer)lwoh);
+   g_signal_connect(G_OBJECT(ebox), "leave_notify_event", G_CALLBACK(help_cb), (gpointer)lwoh);
+
+   gtk_label_set_markup(GTK_LABEL(lwoh->normalLabel), lwoh->normalText);
+   gtk_label_set_markup(GTK_LABEL(lwoh->linkLabel), lwoh->normalText);
+   gtk_container_add(GTK_CONTAINER(ebox), lwoh->linkLabel);
+
+   return lwoh;
+}
+
+void SetOnlineHelpLinkText(LabelWithOnlineHelp *lwoh, char *ascii_text)
+{  char text[strlen(ascii_text)+80];
+
+   if(lwoh->normalText) g_free(lwoh->normalText);
+   if(lwoh->highlitText) g_free(lwoh->highlitText);
+
+   lwoh->normalText  = g_locale_to_utf8(ascii_text, -1, NULL, NULL, NULL);
+   g_sprintf(text, "<span underline=\"single\" color=\"blue\">%s</span>", ascii_text);
+   lwoh->highlitText = g_locale_to_utf8(text, -1, NULL, NULL, NULL);
+}
+
+void FreeLabelWithOnlineHelp(LabelWithOnlineHelp *lwoh)
+{  
+   if(lwoh->lastSizes)
+   {  int i;
+
+      for(i=0; i<lwoh->lastSizes->len; i++)
+      {  int *var = g_ptr_array_index(lwoh->lastSizes, i);
+
+	 g_free(var);
+      }
+      g_ptr_array_free(lwoh->lastSizes, FALSE);
+   }
+
+   g_free(lwoh->windowTitle);
+   g_free(lwoh->normalText);
+   g_free(lwoh->highlitText);
+   g_free(lwoh);
+}
+
+/*
+ * Add a paragraph of text to the help window
+ */
+
+static gboolean wrapper_fix_cb(GtkWidget *widget, GdkEventExpose *event, gpointer data)
+{  int *last_width = (int*)data;
+   int label_width = widget->allocation.width;
+   
+   if(*last_width == label_width)  /* short circuit expose events */ 
+      return FALSE;                /* without size changes */
+
+   *last_width = label_width;
+
+   /* This is a hack. We feed the label its own allocation to make it redraw.
+      Note that we subtract 4 or else the window would never shrink again. */
+
+   if(label_width<0 || label_width>200)
+      gtk_widget_set_size_request(widget, label_width-4, -1);
+
+   return FALSE;
+}
+
+void AddHelpParagraph(LabelWithOnlineHelp *lwoh, char *format, ...)
+{  GtkWidget *label = gtk_label_new(NULL);
+   va_list argp;
+   char *text,*utf;
+
+   va_start(argp, format);
+   text = g_strdup_vprintf(format, argp);
+   va_end(argp);
+
+   utf = g_locale_to_utf8(text, -1, NULL, NULL, NULL);
+   gtk_label_set_markup(GTK_LABEL(label), utf);
+   g_free(utf);
+   g_free(text);
+
+   gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
+   gtk_box_pack_start(GTK_BOX(lwoh->vbox), label, FALSE, FALSE, 0);
+
+   /* Work around some bugs in the gtk line wrapper code.
+      By default lines are wrapped at the length of 
+      "This long string gives a good enough length for any line to have."
+      which is, well, stupid. */ 
+
+   gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
+   g_signal_connect(label, "expose_event", G_CALLBACK(wrapper_fix_cb), get_new_int(lwoh));
+}
+
+/*
+ * Add an item list to the help window.
+ * The list may be preceeded by an optional paragraph of text.
+ */
+
+void AddHelpListItem(LabelWithOnlineHelp *lwoh, char *format, ...)
+{  GtkWidget *label = gtk_label_new(NULL);
+   GtkWidget *bullet = gtk_label_new(" - ");
+   GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
+   va_list argp;
+   char *text,*utf;
+
+   gtk_box_pack_start(GTK_BOX(lwoh->vbox), hbox, FALSE, FALSE, 0);
+
+   gtk_misc_set_alignment(GTK_MISC(bullet), 0.0, 0.0);
+   gtk_box_pack_start(GTK_BOX(hbox), bullet, FALSE, FALSE, 0);
+
+   va_start(argp, format);
+   text = g_strdup_vprintf(format, argp);
+   va_end(argp);
+
+   utf = g_locale_to_utf8(text, -1, NULL, NULL, NULL);
+   gtk_label_set_markup(GTK_LABEL(label), utf);
+   g_free(utf);
+   g_free(text);
+
+   gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
+   gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
+
+   /* Work around some bugs in the gtk line wrapper code.
+      By default lines are wrapped at the length of 
+      "This long string gives a good enough length for any line to have."
+      which is, well, stupid. */ 
+
+   gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
+   g_signal_connect(label, "expose_event", G_CALLBACK(wrapper_fix_cb), get_new_int(lwoh));
+}
+
+/*
+ * Add a (fully functional!) widget set to the help window 
+ */
+
+void AddHelpWidget(LabelWithOnlineHelp *lwoh, GtkWidget *widget)
+{  
+   gtk_box_pack_start(GTK_BOX(lwoh->vbox), widget, FALSE, FALSE, 10);
+   gtk_box_pack_start(GTK_BOX(lwoh->vbox), gtk_hseparator_new(), FALSE, FALSE, 10);
+}
+
 /***
  *** The log viewer
  ***/
@@ -33,21 +325,28 @@ static void log_destroy_cb(GtkWidget *widget, gpointer data)
    Closure->logBuffer = NULL;
 }
 
-static gboolean log_idle_func(gpointer data)
+static gboolean log_jump_func(gpointer data)
 {  GtkAdjustment *a;
    GtkTextIter end;
 
-   gtk_text_buffer_set_text(Closure->logBuffer, Closure->logString->str, Closure->logString->len);
    gtk_text_buffer_get_end_iter(Closure->logBuffer, &end);
    gtk_text_buffer_place_cursor(Closure->logBuffer, &end);
 
    a = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(Closure->logScroll));
-   gtk_adjustment_set_value(a, a->upper);
+   gtk_adjustment_set_value(a, a->upper - a->page_size);
    gtk_scrolled_window_set_vadjustment(GTK_SCROLLED_WINDOW(Closure->logScroll), a);
 
    return FALSE;
 }
 
+static gboolean log_idle_func(gpointer data)
+{
+   gtk_text_buffer_set_text(Closure->logBuffer, Closure->logString->str, Closure->logString->len);
+
+   g_idle_add(log_jump_func, NULL);
+   return FALSE;
+}
+
 
 void UpdateLog()
 {  static int unique_addr;
@@ -78,7 +377,7 @@ void ShowLog()
 
 
 /***
- *** Help dialogs 
+ *** Specific help dialogs 
  ***/
 
 void ShowGPL()
@@ -96,7 +395,7 @@ void ShowGPL()
 char *find_file(char *file, size_t *size, char *lang)
 {  char *path;
    char lang_suffix[3];
-   struct stat mystat;
+   gint64 stat_size;
 
    lang_suffix[0] = lang_suffix[2] = 0;
 
@@ -113,8 +412,8 @@ char *find_file(char *file, size_t *size, char *lang)
            path = g_strdup_printf("%s/%s.%s",Closure->binDir, file, lang_suffix);
       else path = g_strdup_printf("%s/%s",Closure->binDir, file);
 
-      if(!stat(path, &mystat))
-      {	 *size = mystat.st_size;
+      if(LargeStat(path, &stat_size))
+      {	 *size = stat_size;
 	 return path;
       }
 
@@ -128,8 +427,8 @@ char *find_file(char *file, size_t *size, char *lang)
            path = g_strdup_printf("%s/%s.%s",Closure->docDir, file, lang_suffix);
       else path = g_strdup_printf("%s/%s",Closure->docDir, file);
 
-      if(!stat(path, &mystat))
-      {	 *size = mystat.st_size;
+      if(LargeStat(path, &stat_size))
+      {	 *size = stat_size;
 	 return path;
       }
 
@@ -162,7 +461,7 @@ GtkWidget* ShowTextfile(char *title, char *explanation, char *file,
 	 size = strlen(buf);
       }
       else
-      {  FILE *file = fopen(path, "rb");
+      {  FILE *file = portable_fopen(path, "rb");
 
 	 buf = g_malloc(size);
 	 fread(buf, size, 1, file);
@@ -371,76 +670,6 @@ void AboutTextWithLink(GtkWidget *parent, char *text, char *action)
    g_free(copy);
 }
 
-void AboutTextWithLink2(GtkWidget *parent, char *text, char *actions)
-{  char *copy,*head,*end_of_line;
-   char *link_start,*link_end; 
-   char *utf;
-   char *action_separator;
-   char *action,*action_copy;
-
-   head = copy = g_strdup(text);
-   action = action_copy = g_strdup(actions);
-
-   while(*head)
-   {  end_of_line = strchr(head, '\n');
-      if(end_of_line && *end_of_line == '\n')
-        *end_of_line = 0;
-
-      link_start = strchr(head, '[');
-      link_end = strchr(head, ']');
-
-      if(!link_start || !link_end)
-	   AboutText(parent, head);
-      else
-      {  GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
-
-         gtk_box_pack_start(GTK_BOX(parent), hbox, FALSE, FALSE, 0);
-
-	 while(1)
-	 {  *link_start++ = *link_end++ = 0;
-
-	    if(*head) 
-	    {  GtkWidget *lab = gtk_label_new(NULL);
-
-	       utf = g_locale_to_utf8(head, -1, NULL, NULL, NULL);
-	       gtk_label_set_markup(GTK_LABEL(lab), utf);
-	       gtk_box_pack_start(GTK_BOX(hbox), lab, FALSE, FALSE, 0);
-	       g_free(utf);
-	    }
-
-	    action_separator = strchr(action, ',');
-	    if(action_separator) *action_separator = 0;
-	    AboutLink(hbox, link_start, g_strdup(action));
-	    if(action_separator) action = action_separator+1;
-
-	    head = link_end;
-	    if(head >= end_of_line) break;
-
-	    link_start = strchr(head, '[');   /* more links in this line? */
-	    link_end = strchr(head, ']');
-	    if(link_start && link_end)
-	      continue;
-	    
-	    if(*head)  /* no more links, but perhaps trailing text in this line */
-	    {  GtkWidget *lab = gtk_label_new(NULL);
-
-	       utf = g_locale_to_utf8(head, -1, NULL, NULL, NULL);
-	       gtk_label_set_markup(GTK_LABEL(lab), utf);
-	       gtk_box_pack_start(GTK_BOX(hbox), lab, FALSE, FALSE, 0);
-	       g_free(utf);
-	       break;
-	    }
-	 } 
-      }
-
-      if(end_of_line) head = end_of_line+1;
-      else break;
-   }
-
-   g_free(copy);
-   g_free(action_copy);
-}
-
 void AboutDialog()
 {  GtkWidget *about, *vbox, *sep;
    char *text; 
@@ -469,11 +698,11 @@ void AboutDialog()
 
 #ifdef MODIFIED_SOURCE
    AboutTextWithLink(vbox, 
-		     _("Modified version Copyright 2006 (please fill in - [directions])\n"
-		       "Copyright 2004-2006 Carsten Gnoerlich"),
+		     _("Modified version Copyright 2008 (please fill in - [directions])\n"
+		       "Copyright 2004-2008 Carsten Gnoerlich"),
 		     "MODIFYING");
 #else
-   AboutText(vbox, _("Copyright 2004-2006 Carsten Gnoerlich"));
+   AboutText(vbox, _("Copyright 2004-2008 Carsten Gnoerlich"));
 #endif
 
    sep = gtk_hseparator_new();
@@ -506,14 +735,16 @@ void AboutDialog()
    {    AboutTextWithLink(vbox, "\n[http://www.dvdisaster.com]", "http://www.dvdisaster.com");
    }
 
-   AboutText(vbox, _("\ne-mail: carsten@dvdisaster.com   -or-   cgnoerlich@fsfe.org")); 
+   AboutText(vbox, _("\ne-mail: carsten@dvdisaster.org   -or-   cgnoerlich@fsfe.org")); 
 
 #ifdef SYS_DARWIN
    AboutText(vbox, _("\nDarwin port (Mac OS X): Julian Einwag &lt;julian@einwag.de&gt;")); 
 #endif
+#ifdef SYS_NETBSD
+   AboutText(vbox, _("\nNetBSD port: Sergey Svishchev &lt;svs@ropnet.ru&gt;")); 
+#endif
 #endif
    /* Show it */
 
    gtk_widget_show_all(about);
 }
-
