@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2006 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2008 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -133,6 +133,60 @@ void CalcSectors(gint64 size, gint64 *sectors, int *in_last)
  *** Message output
  ***/
 
+/*
+ * Append message to the log window.
+ */
+
+#define MAX_LOG_WIN_SIZE 10240
+
+static void clamp_gstring(GString *string)
+{  gchar *ptr;
+   int cut;
+
+
+   if(string->len < MAX_LOG_WIN_SIZE)
+      return;
+
+   /* Remove head of the string so that it gets smaller than
+      the maximum size, and cut off until the next newline */
+
+   ptr = string->str;
+   cut = string->len - MAX_LOG_WIN_SIZE;
+   ptr += cut;
+
+   while(*ptr && *ptr != '\n')
+   {  ptr++;
+      cut++;
+   }
+
+   g_string_erase(string, 0, cut);
+}
+
+static void log_window_vprintf(char *format, va_list argp)
+{  char *tmp,*utf_tmp;
+
+   tmp = g_strdup_vprintf(format, argp);
+   utf_tmp = g_locale_to_utf8(tmp, -1, NULL, NULL, NULL);
+   g_string_append(Closure->logString, utf_tmp);
+   clamp_gstring(Closure->logString);
+
+   UpdateLog();
+   
+   g_free(tmp);
+   g_free(utf_tmp);
+}
+
+static void log_window_append(char *text)
+{  char *utf_tmp = g_locale_to_utf8(text, -1, NULL, NULL, NULL);
+
+   g_string_append(Closure->logString, utf_tmp);
+   clamp_gstring(Closure->logString);
+
+   UpdateLog();
+   
+   g_free(utf_tmp);
+}
+
 /*
  * Output of the greetings is delayed until the first message is printed.
  */
@@ -143,7 +197,7 @@ static void print_greetings(FILE *where)
    if(greetings_shown) return;
 
    greetings_shown = 1;
-   g_fprintf(where, _("dvdisaster-%s%sCopyright 2004-2006 Carsten Gnoerlich.\n"),
+   g_fprintf(where, _("dvdisaster-%s%sCopyright 2004-2008 Carsten Gnoerlich.\n"),
 	     VERSION, strstr(VERSION,"pl") ? " " : "  ");
    /* TRANSLATORS: Excluding all kinds of warranty might be harmful under your
       legislature. If in doubt, just translate the following like "This is free
@@ -156,14 +210,27 @@ static void print_greetings(FILE *where)
 }
 
 /*
- * Print to stdout if run from the command line; do nothing in GUI mode.
+ * Print to stdout if run from the command line;
+ * do nothing in GUI mode unless Closure->verbose is set.
  */
 
 void PrintCLI(char *format, ...)
 {  va_list argp;
 
+   if(Closure->logFileEnabled)
+   {  va_start(argp, format);
+      VPrintLogFile(format, argp);
+      va_end(argp);
+   }
+
    if(Closure->guiMode)
-     return;
+   {  if(Closure->verbose)
+      {  va_start(argp, format);
+	 log_window_vprintf(format, argp);
+	 va_end(argp);
+      }
+      return;
+   }
 
    va_start(argp, format);
    g_vfprintf(stdout, format, argp);
@@ -221,20 +288,16 @@ void PrintProgress(char *format, ...)
 void PrintLog(char *format, ...)
 {  va_list argp;
 
+   if(Closure->logFileEnabled)
+   {  va_start(argp, format);
+      VPrintLogFile(format, argp);
+      va_end(argp);
+   }
+
    va_start(argp, format);
 
    if(Closure->guiMode)
-   {  char *tmp,*utf_tmp;
-
-      tmp = g_strdup_vprintf(format, argp);
-      utf_tmp = g_locale_to_utf8(tmp, -1, NULL, NULL, NULL);
-      g_string_append(Closure->logString, utf_tmp);
-
-      UpdateLog();
-   
-      g_free(tmp);
-      g_free(utf_tmp);
-   }
+      log_window_vprintf(format, argp);
    else 
    {
       print_greetings(stderr);
@@ -253,23 +316,19 @@ void PrintLog(char *format, ...)
 void Verbose(char *format, ...)
 {  va_list argp;
 
+   if(Closure->logFileEnabled)
+   {  va_start(argp, format);
+      VPrintLogFile(format, argp);
+      va_end(argp);
+   }
+
    if(!Closure->verbose)
     return;
 
    va_start(argp, format);
 
    if(Closure->guiMode)
-   {  char *tmp,*utf_tmp;
-
-      tmp = g_strdup_vprintf(format, argp);
-      utf_tmp = g_locale_to_utf8(tmp, -1, NULL, NULL, NULL);
-      g_string_append(Closure->logString, utf_tmp);
-
-      UpdateLog();
-   
-      g_free(tmp);
-      g_free(utf_tmp);
-   }
+      log_window_vprintf(format, argp);
    else 
    {
       print_greetings(stderr);
@@ -281,32 +340,6 @@ void Verbose(char *format, ...)
    va_end(argp);
 }
 
-/*
- * Print a message to the log file.
- * Tries hard to make the log messages survive a system crash.
- */
-
-#ifdef WITH_LOGFILE_YES
-void PrintLogFile(char *format, ...)
-{  FILE *file;
-   va_list argp;
-
-   va_start(argp, format);
-
-   file = fopen(Closure->logFile, "a");
-   if(!file)
-     return;
-
-   g_vfprintf(file, format, argp);
-   fflush(file);
-   fclose(file);
-
-   va_end(argp);
-}
-#else
-void PrintLogFile(char *format, ...) { }
-#endif
-
 /*
  * Print timing results to console and log window 
  */
@@ -318,24 +351,19 @@ void PrintTimeToLog(GTimer *timer, char *format, ...)
    double seconds = fmod(elapsed,60.0);
    int minutes = (int)fmod(elapsed / 60.0, 60.0);
    int hours = (int)(elapsed / 3600.0);
-   char *tmp1,*tmp2,*utf_tmp;
+   char *tmp1,*tmp2;
 
    if(!Closure->verbose)
      return;
 
    va_start(argp, format);
-
    tmp1 = g_strdup_vprintf(format, argp);
    tmp2 = g_strdup_printf("%02d:%02d:%04.1f %s", hours, minutes, seconds, tmp1);
+   va_end(argp);
 
    if(Closure->guiMode)
    { 
-      utf_tmp = g_locale_to_utf8(tmp2, -1, NULL, NULL, NULL);
-      g_string_append(Closure->logString, utf_tmp);
-
-      UpdateLog();
-   
-      g_free(utf_tmp);
+      log_window_append(tmp2);
    }
    else
    {  g_fprintf(stderr, tmp2);
@@ -343,8 +371,6 @@ void PrintTimeToLog(GTimer *timer, char *format, ...)
       fflush(stderr);   /* at least needed for Windows */
    }
 
-   va_end(argp);
-
    g_free(tmp1);
    g_free(tmp2);
 }
@@ -357,6 +383,12 @@ void PrintTimeToLog(GTimer *timer, char *format, ...)
 void PrintCLIorLabel(GtkLabel *label, char *format, ...)
 {  va_list argp;
 
+   if(Closure->logFileEnabled)
+   {  va_start(argp, format);
+      VPrintLogFile(format, argp);
+      va_end(argp);
+   }
+
    va_start(argp, format);
       
    if(Closure->guiMode)
@@ -364,6 +396,9 @@ void PrintCLIorLabel(GtkLabel *label, char *format, ...)
 	
       tmp = g_strdup_vprintf(format, argp);
 
+      if(Closure->verbose)
+	 log_window_append(tmp);
+
       c   = tmp + strlen(tmp) - 1; 
       while(*c == '\n')          /* Remove trailing newlines */
 	*c-- = 0;
@@ -390,7 +425,7 @@ static void vlog_warning(char *format, va_list argp)
    int len = strlen(warn)+4;
    char prefix[len+1]; 
    char *c,*line;
-   char *body,*utf_msg;
+   char *body;
    GString *str;
 
    body = g_strdup_vprintf(format, argp);
@@ -414,12 +449,11 @@ static void vlog_warning(char *format, va_list argp)
       if(c) line = c+1;
    } while(c && *line);
 
-   if(Closure->guiMode)
-   {  utf_msg = g_locale_to_utf8(str->str, -1, NULL, NULL, NULL);
-      g_string_append(Closure->logString, utf_msg);
-      g_free(utf_msg);
+   if(Closure->logFileEnabled)
+      PrintLogFile("%s", str->str);
 
-      UpdateLog();
+   if(Closure->guiMode)
+   {  log_window_append(str->str);
    }
    else
    {  print_greetings(stderr);
@@ -448,6 +482,13 @@ void Stop(char *format, ...)
 
    /*** Show message depending on commandline / GUI mode  */ 
 
+   if(Closure->logFileEnabled)
+   {  va_start(argp, format);
+      PrintLogFile(_("\n*\n* dvdisaster - can not continue:\n*\n"));
+      VPrintLogFile(format, argp);
+      va_end(argp);
+   }
+
    if(!Closure->guiMode) 
    {  g_fprintf(stderr, _("\n*\n* dvdisaster - can not continue:\n*\n"));
       va_start(argp, format);
@@ -466,11 +507,7 @@ void Stop(char *format, ...)
 
       if(Closure->errorTitle)
       {
-#if GTK_MINOR_VERSION >= 4
 	 titled = g_strdup_printf("<b>%s</b>\n\n%s", Closure->errorTitle,msg);
-#else
-	 titled = g_strdup_printf("%s\n\n%s", Closure->errorTitle,msg);
-#endif
 	 g_free(msg);
 	 msg = titled;
       }
@@ -628,19 +665,11 @@ static gboolean message_idle_func(gpointer data)
 {  message_info *mi = (message_info*)data;
    GtkWidget *dialog;
 
-#if GTK_MINOR_VERSION >= 4
    dialog = gtk_message_dialog_new_with_markup(mi->window,
 					       GTK_DIALOG_DESTROY_WITH_PARENT,
 					       mi->type,
 					       GTK_BUTTONS_CLOSE,
 					       mi->msg, NULL);
-#else
-   dialog = gtk_message_dialog_new(mi->window, 
-				   GTK_DIALOG_DESTROY_WITH_PARENT,
-				   mi->type,
-				   GTK_BUTTONS_CLOSE,
-				   mi->msg, NULL);
-#endif
 
    gtk_label_set_line_wrap(GTK_LABEL(((struct _GtkMessageDialog*)dialog)->label), FALSE);
    g_signal_connect_swapped(dialog, "response", G_CALLBACK(gtk_widget_destroy), dialog);
@@ -715,14 +744,16 @@ static gboolean label_idle_func(gpointer data)
 void SetLabelText(GtkLabel *label, char *format, ...)
 {  label_info *li = g_malloc(sizeof(label_info));
    va_list argp;
-   char *tmp;
 
    li->label = label;
 
    va_start(argp, format);
-   tmp  = g_strdup_vprintf(format, argp);
-   li->text = g_locale_to_utf8(tmp, -1, NULL, NULL, NULL);
-   g_free(tmp);
+   if(format)
+   {  char *tmp  = g_strdup_vprintf(format, argp);
+      li->text = g_locale_to_utf8(tmp, -1, NULL, NULL, NULL);
+      g_free(tmp);
+   }
+   else li->text = g_locale_to_utf8("(null)", -1, NULL, NULL, NULL);
    va_end(argp);
 
    g_idle_add(label_idle_func, li);
@@ -885,9 +916,13 @@ int ModalWarning(GtkMessageType mt, GtkButtonsType bt,
 
    va_start(argp, msg);
    vlog_warning(msg, argp);
-   if(Closure->guiMode)
-     result = vmodal_dialog(mt, bt, button_fn, msg, argp);
    va_end(argp);
+   
+   if(Closure->guiMode)
+   { va_start(argp, msg);
+     result = vmodal_dialog(mt, bt, button_fn, msg, argp);
+     va_end(argp);
+   }
 
    return result;
 }
@@ -964,19 +999,21 @@ void SwitchAndSetFootline(GtkWidget *notebook, int page, GtkWidget *label, char
  * 
  * gtk_dialog_set_alternative_button_order()
  * has been introduced since gtk+2.6,
- * but seems to work only in 2.8.
+ * but does not seem to work correctly.
  */
 
 void ReverseCancelOK(GtkDialog *dialog)
-{  
-#if GTK_MINOR_VERSION <= 6
-   GtkWidget *box, *button ;
+{  GtkWidget *box, *button ;
+
+   if(!Closure->reverseCancelOK)
+      return;
 
    box = dialog->action_area; 
    button = ((GtkBoxChild*)(g_list_first(GTK_BOX(box)->children)->data))->widget;
 
    gtk_box_reorder_child(GTK_BOX(box), button, 1);
-#else
+
+#if 0
    gtk_dialog_set_alternative_button_order(GTK_DIALOG(dialog),
 					   GTK_RESPONSE_OK,
 					   GTK_RESPONSE_CANCEL,
@@ -1003,3 +1040,51 @@ void TimedInsensitive(GtkWidget *widget, int delay)
 
   g_timeout_add(delay, insensitive_timeout_func, (gpointer)widget);
 }
+
+/*
+ * Get the width of a label text
+ */
+
+int GetLabelWidth(GtkLabel *label, char *format, ...)
+{  PangoLayout *layout;
+   PangoRectangle rect;
+   va_list argp;
+   char *text;
+
+   va_start(argp, format);
+   text = g_strdup_vprintf(format, argp);
+   va_end(argp);
+
+   layout = gtk_label_get_layout(label);
+   pango_layout_set_text(layout, text, -1);
+   pango_layout_get_pixel_extents(layout, NULL, &rect);
+
+   g_free(text);
+
+   return rect.width;
+}
+ 
+
+/*
+ * Lock the size of a label to that of the given sample text.
+ */
+
+void LockLabelSize(GtkLabel *label, char *format, ...)
+{  PangoLayout *layout;
+   PangoRectangle rect;
+   va_list argp;
+   char *text;
+
+   va_start(argp, format);
+   text = g_strdup_vprintf(format, argp);
+   va_end(argp);
+
+   layout = gtk_label_get_layout(label);
+   pango_layout_set_text(layout, text, -1);
+   pango_layout_get_pixel_extents(layout, NULL, &rect);
+
+   gtk_widget_set_size_request(GTK_WIDGET(label), rect.width, rect.height);
+   gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
+
+   g_free(text);
+}
