@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2006 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2008 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -51,7 +51,7 @@ static const char *snstext[] = {
     "Volume Overflow",          /* Medium full with still data to be written */
     "Miscompare",               /* Source data and data on the medium
                                    do not agree */
-    "Key=15"                    /* Reserved */
+    "Key=15",                   /* Reserved */
 };
 
 struct error_info{
@@ -534,10 +534,25 @@ static struct error_info additional[] =
   {0x73,0x04,R,"Program memory area update failure"},
   {0x73,0x05,R,"Program memory area is full"},
   {0x73,0x06,R,"RMA/PMA is full"},
+
+
+  /*
+   * Faked errors by software L-EC
+   */
+  {0xff,0x00,R,"No data returned"},
+  {0xff,0x01,R,"EDC failure in RAW sector"},
+  {0xff,0x02,R,"Wrong MSF in RAW sector"},
+  {0xff,0x03,R,"RAW reading > 16 sectors at once not supported"},
+  {0xff,0x04,R,"RAW buffer not allocated"},
+  {0xff,0x05,R,"Invalid zero sector"},
+  {0xff,0x06,R,"Sector accumulated for analysis"},
+  {0xff,0x07,R,"Recovery failed"},
+  {0xff,0x08,R,"Invalid sector; possibly random data returned"},
+
   /*
    * Faked error by defect simulation mode
    */
-  {0xff,0xff,R,"Simulated medium defect"},
+  {0xff,0xff,R,"[dvdisaster: Simulated medium defect]"},
 
   {0, 0, 0, NULL}
 };
@@ -567,13 +582,30 @@ void RememberSense(int k, int a, int aq)
 char *GetSenseString(int sense_key, int asc, int ascq, int verbose)
 {  static char text[256];
    struct error_info *ei;
+   int raw_reader_error;
+   const char *sns_text;
+   char sep;
    int idx,len;
 
+   /* Special treatment for our fake SCSI errors */
+
+   raw_reader_error = (asc == 255) && (ascq < 255);
+   if(!raw_reader_error) 
+   {  sep = ';';
+      sns_text = snstext[sense_key];
+   }
+   else
+   {  sep = ':';
+      sns_text = "Raw Reader";
+   }
+
+   /* Go print them */
+
    if(sense_key <0 || sense_key > 15) 
         g_snprintf(text, 255, _("Sense error (0x%02x); "),sense_key);
    else 
-   {  if(verbose) g_snprintf(text, 255, _("Sense error: %s; "),snstext[sense_key]);
-      else        g_snprintf(text, 255, "%s; ",snstext[sense_key]);
+   {   if(verbose) g_snprintf(text, 255, _("Sense error: %s%c "),sns_text, sep);
+       else        g_snprintf(text, 255, "%s%c ",sns_text, sep);
    }
 
    idx = strlen(text); 
@@ -599,3 +631,9 @@ char *GetSenseString(int sense_key, int asc, int ascq, int verbose)
 char* GetLastSenseString(int verbose)
 {  return GetSenseString(sense_key, asc, ascq, verbose);
 }
+
+void GetLastSense(int *key_out, int *asc_out, int *ascq_out)
+{  *key_out  = sense_key;
+   *asc_out  = asc;
+   *ascq_out = ascq;
+}
