@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2006 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2008 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -45,8 +45,10 @@ typedef struct
    int rs01LayerSectors;        /* length of each RS01 ecc layer in sectors */
    RS02Layout *lay;             /* layout of RS02 type ecc data */
 
+   AlignedBuffer *ab;           /* buffer suitable for reading from the drive */
+   unsigned char *buf;          /* buffer component from above */
    Bitmap *map;                 /* bitmap for keeping track of read sectors */
-   guint32 *crcbuf;             /* crc sum buffer space */
+   CrcBuf *crcBuf;              /* preloaded CRC info from ecc data */
 
    gint64 readable;             /* current outcome of reading process */
    gint64 unreadable;
@@ -89,6 +91,12 @@ static void cleanup(gpointer data)
 
    Closure->cleanupProc = NULL;
 
+   /* Reset temporary ignoring of fatal errors.
+      User has to set this in the preferences to make it permanent. */
+
+   if(Closure->ignoreFatalSense == 2)
+      Closure->ignoreFatalSense = 0;
+
    /* Rewrite the header sectors if we were reading an RS02 image;
       otherwise the image will not be recognized later. */
 
@@ -129,7 +137,7 @@ static void cleanup(gpointer data)
 bail_out:
    if(Closure->guiMode)
    {  if(rc->earlyTermination)
-        SetAdaptiveReadFootline(_("Aborted by unrecoverable error."), Closure->red);
+        SetAdaptiveReadFootline(_("Aborted by unrecoverable error."), Closure->redText);
 
       AllowActions(TRUE);
    }
@@ -144,15 +152,18 @@ bail_out:
    if(rc->subtitle) g_free(rc->subtitle);
    if(rc->segmentState) g_free(rc->segmentState);
 
+   if(rc->ab) FreeAlignedBuffer(rc->ab);
+   
    if(rc->intervals) g_free(rc->intervals);
-   if(rc->crcbuf) g_free(rc->crcbuf);
+   if(rc->crcBuf) FreeCrcBuf(rc->crcBuf);
 
    if(rc->map)
      FreeBitmap(rc->map);
 
    g_free(rc);
 
-   g_thread_exit(0);
+   if(Closure->guiMode)
+      g_thread_exit(0);
 }
 
 /***
@@ -317,13 +328,15 @@ static void mark_sector(read_closure *rc, gint64 sector, GdkColor *color)
    {  GdkColor *old = Closure->readAdaptiveSpiral->segmentColor[segment];
       GdkColor *new = old;
 
-      if(color == Closure->red && old != Closure->red)
+      if(color == Closure->redSector && old != Closure->redSector)
 	new = color;
 
-      if(color == Closure->yellow && old != Closure->red && old != Closure->yellow)
+      if(   color == Closure->yellowSector
+	 && old != Closure->redSector
+	 && old != Closure->yellowSector)
 	new = color;
 
-      if(color == Closure->green)
+      if(color == Closure->greenSector)
       {  rc->segmentState[segment]++;
 
 	 if(rc->segmentState[segment] >= rc->sectorsPerSegment)
@@ -454,7 +467,7 @@ static void check_size(read_closure *rc)
 			    rc->sectors);
 
       if(!answer)
-      {  SetAdaptiveReadFootline(_("Aborted by user request!"), Closure->red);
+      {  SetAdaptiveReadFootline(_("Aborted by user request!"), Closure->redText);
 	 rc->earlyTermination = FALSE;
 	 cleanup((gpointer)rc);
       }
@@ -470,7 +483,7 @@ static void check_size(read_closure *rc)
 			     rc->sectors);
 
       if(!answer)
-      {  SetAdaptiveReadFootline(_("Aborted by user request!"), Closure->red);
+      {  SetAdaptiveReadFootline(_("Aborted by user request!"), Closure->redText);
 	 rc->earlyTermination = FALSE;
 	 cleanup((gpointer)rc);
       }
@@ -483,67 +496,67 @@ static void check_size(read_closure *rc)
  * Limit reading to user selected range
  */
 
-static void get_reading_range(read_closure *rc)
-{
+void GetReadingRange(gint64 sectors, gint64 *firstSector, gint64 *lastSector)
+{  gint64 first, last;
+
    if(Closure->readStart || Closure->readEnd)
    {  if(!Closure->guiMode) /* more range checks are made below */ 
-      {  rc->firstSector = Closure->readStart;
-         rc->lastSector  = Closure->readEnd < 0 ? rc->sectors-1 : Closure->readEnd;
+      {  first = Closure->readStart;
+         last  = Closure->readEnd < 0 ? sectors-1 : Closure->readEnd;
       }
       else  /* be more permissive in GUI mode */
-      {  rc->firstSector = 0;
- 	 rc->lastSector  = rc->sectors-1;
+      {  first = 0;
+ 	 last  = sectors-1;
 
 	 if(Closure->readStart <= Closure->readEnd)
-	 {  rc->firstSector = Closure->readStart < rc->sectors ? Closure->readStart : rc->sectors-1;
-	    rc->lastSector  = Closure->readEnd   < rc->sectors ? Closure->readEnd   : rc->sectors-1;
+	 {  first = Closure->readStart < sectors ? Closure->readStart : sectors-1;
+	    last  = Closure->readEnd   < sectors ? Closure->readEnd   : sectors-1;
 	 }
       }
 
-      if(rc->firstSector > rc->lastSector || rc->firstSector < 0 || rc->lastSector >= rc->sectors)
-	Stop(_("Sectors must be in range [0..%lld].\n"),rc->sectors-1);
+      if(first > last || first < 0 || last >= sectors)
+	Stop(_("Sectors must be in range [0..%lld].\n"), sectors-1);
 
-      PrintLog(_("Limiting sector range to [%lld,%lld].\n"),rc->firstSector,rc->lastSector);
+      PrintLog(_("Limiting sector range to [%lld,%lld].\n"), first, last);
    }
    else 
-   {  rc->firstSector = 0; rc->lastSector = rc->sectors-1;
+   {  first = 0; 
+      last  = sectors-1;
    }
+
+   *firstSector = first;
+   *lastSector = last;
 }
 
 /*
- * Compare image fingerprint against fingerprint stored in error correction file 
+ * Compare medium fingerprint against fingerprint stored in error correction file 
  */
 
-static void check_fingerprint(read_closure *rc)
-{  unsigned char digest[16];
-   int status;
-   
-   status = ReadSectors(rc->dh, Closure->scratchBuf, rc->eh->fpSector, 1);
+static void check_ecc_fingerprint(read_closure *rc)
+{  guint8 digest[16];
+   int fp_read;
 
-   if(status) /* Not readable. Bad luck. */
+   fp_read = GetMediumFingerprint(rc->dh, digest, rc->eh->fpSector);
+
+   if(!fp_read) /* Not readable. Bad luck. */
    {  int answer;
 
       answer = ModalWarning(GTK_MESSAGE_WARNING, GTK_BUTTONS_OK_CANCEL, NULL,
-			    _("Sector %d is missing. Can not compare image and ecc fingerprints.\n"
-			      "Double check that image and ecc file belong together.\n"),
+			    _("Sector %d is missing. Can not compare medium and ecc fingerprints.\n"
+			      "Double check that the medium and the ecc file belong together.\n"),
 			    rc->eh->fpSector);
 
       if(!answer)
-      {  SetAdaptiveReadFootline(_("Aborted by user request!"), Closure->red);
+      {  SetAdaptiveReadFootline(_("Aborted by user request!"), Closure->redText);
 	 rc->earlyTermination = FALSE;
 	 cleanup((gpointer)rc);
       }
    }
    else 
-   {  struct MD5Context image_md5;
-
-      MD5Init(&image_md5);
-      MD5Update(&image_md5, (unsigned char*)Closure->scratchBuf, 2048);
-      MD5Final(digest, &image_md5);
-
+   {  
       if(memcmp(digest, rc->eh->mediumFP, 16))
-	Stop(_("Fingerprints of image and ecc file do not match.\n"
-	       "Image and ecc file do not belong together.\n"));
+	Stop(_("Fingerprints of medium and ecc file do not match.\n"
+	       "Medium and ecc file do not belong together.\n"));
    }
 }
 
@@ -552,8 +565,10 @@ static void check_fingerprint(read_closure *rc)
  */
 
 int check_image_fingerprint(read_closure *rc)
-{  gint32 fingerprint_sector;
-   int status,n;
+{  struct MD5Context md5ctxt;
+   guint8 image_fp[16], medium_fp[16];
+   gint32 fingerprint_sector;
+   int fp_read,n;
   
    /* Determine fingerprint sector */
 
@@ -566,15 +581,19 @@ int check_image_fingerprint(read_closure *rc)
    if(!LargeSeek(rc->image, (gint64)(2048*fingerprint_sector)))
      return 0; /* can't tell, assume okay */
 
-   n = LargeRead(rc->image, Closure->scratchBuf, 2048);
-   status = ReadSectors(rc->dh, Closure->scratchBuf+2048, fingerprint_sector, 1);
+   n = LargeRead(rc->image, rc->buf, 2048);
+   MD5Init(&md5ctxt);
+   MD5Update(&md5ctxt, rc->buf, 2048);
+   MD5Final(image_fp, &md5ctxt);
 
-   if(n != 2048 || status || !memcmp(Closure->scratchBuf, Closure->deadSector, 2048))
+   fp_read = GetMediumFingerprint(rc->dh, medium_fp, fingerprint_sector);
+
+   if(n != 2048 || !fp_read || !memcmp(rc->buf, Closure->deadSector, 2048))
      return 0; /* can't tell, assume okay */
 
    /* If both could be read, compare them */
 
-   if(memcmp(Closure->scratchBuf, Closure->scratchBuf+2048, 2048))
+   if(memcmp(image_fp, medium_fp, 16))
    {  	  
      if(!Closure->guiMode)
        Stop(_("Image file does not match the CD/DVD."));
@@ -586,7 +605,7 @@ int check_image_fingerprint(read_closure *rc)
         if(!answer)
 	{  rc->earlyTermination = FALSE;
 	   SetAdaptiveReadFootline(_("Reading aborted. Please select a different image file."), 
-				   Closure->red);
+				   Closure->redText);
 	   cleanup((gpointer)rc);
 	}
 	else
@@ -616,7 +635,7 @@ void check_image_size(read_closure *rc, gint64 image_file_sectors)
 			    image_file_sectors-rc->sectors, image_file_sectors, rc->sectors);
 
       if(!answer)
-      {  SetAdaptiveReadFootline(_("Aborted by user request!"), Closure->red);
+      {  SetAdaptiveReadFootline(_("Aborted by user request!"), Closure->redText);
 	 rc->earlyTermination = FALSE;
 	 cleanup((gpointer)rc);
       }
@@ -626,31 +645,40 @@ void check_image_size(read_closure *rc, gint64 image_file_sectors)
    else rc->highestWrittenSector = image_file_sectors-1;
 }
 
+/***
+ *** Load the crc buf from RS01/RS02 error correction data.
+ ***/
+
+static void load_crc_buf(read_closure *rc)
+{
+   switch(rc->readMode)
+   {  case ECC_IN_FILE:
+	 SetAdaptiveReadSubtitle(_utf("Loading CRC data."));
+	 rc->crcBuf = GetCRCFromRS01(rc->ei);
+	 break;
+      case ECC_IN_IMAGE:
+	 SetAdaptiveReadSubtitle(_utf("Loading CRC data."));
+	 rc->crcBuf = GetCRCFromRS02(rc->lay, rc->dh, rc->image);
+	 break;
+      default:
+	 rc->crcBuf = NULL;
+	 break;
+   }
+}
+
 /***
  *** Examine existing image file.
  ***
  * Build an initial interval list from it.
  */
 
-#define CRCBUFSIZE (1024*256)
-
-void build_interval_from_image(read_closure *rc)
+static void build_interval_from_image(read_closure *rc)
 {  gint64 s;
    gint64 first_missing, last_missing, current_missing;
    int tail_included = FALSE;
-   guint32 crc;
-   int crcidx = CRCBUFSIZE;
    int last_percent = 0;
+   int crc_result;
 
-   /*** If we have a separate ecc file we can compare against the
-        CRC sum it contains. Position ecc file behind the ecc header. */
-  
-   if(rc->readMode == ECC_IN_FILE)
-   {
-     if(!LargeSeek(rc->ei->file, (gint64)sizeof(EccHeader)))
-       Stop(_("Failed skipping the ecc header: %s"),strerror(errno));
-   }
-  
    /*** Rewind image file */
 
    LargeSeek(rc->image, 0);
@@ -669,51 +697,42 @@ void build_interval_from_image(read_closure *rc)
       /* Check for user interruption. */
 
       if(Closure->stopActions)   
-      {  SetAdaptiveReadFootline(_("Aborted by user request!"), Closure->red);
+      {  SetAdaptiveReadFootline(_("Aborted by user request!"), Closure->redText);
 	 rc->earlyTermination = FALSE;
 	 cleanup((gpointer)rc);
       }
 
       /* Read the next sector */
 
-      n = LargeRead(rc->image, Closure->scratchBuf, 2048);
+      n = LargeRead(rc->image, rc->buf, 2048);
       if(n != 2048) /* && (s != rc->sectors - 1 || n != ii->inLast)) */
 	Stop(_("premature end in image (only %d bytes): %s\n"),n,strerror(errno));
 
       /* Look for the dead sector marker */
 
-      current_missing = !memcmp(Closure->scratchBuf, Closure->deadSector, n);
+      current_missing = !memcmp(rc->buf, Closure->deadSector, n);
 
       if(current_missing)
-	mark_sector(rc, s, Closure->red);
+	mark_sector(rc, s, Closure->redSector);
 
       /* Compare checksums if available */
 
-      if(rc->readMode == ECC_IN_FILE)
-      {
-	/* If the CRC buf is exhausted, refill it. */
+      if(rc->crcBuf)
+	   crc_result = CheckAgainstCrcBuffer(rc->crcBuf, s, rc->buf);
+      else crc_result = CRC_UNKNOWN;
 
-	 if(crcidx >= CRCBUFSIZE)
-	 {  int remain = rc->highestWrittenSector-s+1;
-	    int size;
-
-	    if(remain < CRCBUFSIZE)
-	         size = remain*sizeof(guint32);
-	    else size = CRCBUFSIZE*sizeof(guint32);
-
-	    if(LargeRead(rc->ei->file, rc->crcbuf, size) != size)
-	      Stop(_("Error reading CRC information: %s"),strerror(errno));
-
-	    crcidx = 0;
-	 }
-
-	 /* compare the checksums */
-
-	 crc = Crc32(Closure->scratchBuf, 2048); 
-	 if(crc != rc->crcbuf[crcidx++] && !current_missing)
-	 {  current_missing = 1;
-	    mark_sector(rc, s, Closure->yellow);
-	 }
+      switch(crc_result)
+      {  case CRC_GOOD:
+	    break;
+	 case CRC_UNKNOWN:
+	    break;
+	 case CRC_BAD:
+	    /* If its not already missing because of a read error,
+	       make it missing due to the CRC failure. */
+	    if(!current_missing)
+	    {  current_missing = 1;
+	       mark_sector(rc, s, Closure->yellowSector);
+	    }
       }
 
       /* Remember sector state */
@@ -727,7 +746,7 @@ void build_interval_from_image(read_closure *rc)
 	 if(rc->map)
 	   SetBit(rc->map, s);
 
-	 mark_sector(rc, s, Closure->green);
+	 mark_sector(rc, s, Closure->greenSector);
 
 #ifdef CHECK_VISITED
 	 rc->count[s]++;
@@ -809,9 +828,9 @@ void build_interval_from_image(read_closure *rc)
 	       {  SetBit(rc->map, layer_idx);
 		  rc->correctable++;
 #ifdef CHECK_VISITED
-		  count[layer_idx]++;
+		  rc->count[layer_idx]++;
 #endif
-		  mark_sector(rc, layer_idx, Closure->green);
+		  mark_sector(rc, layer_idx, Closure->greenSector);
 	       }
 
 	       layer_idx += rc->rs01LayerSectors;
@@ -844,7 +863,7 @@ void build_interval_from_image(read_closure *rc)
 		   && !GetBit(rc->map, sector))
 	       {  SetBit(rc->map, sector);
 		  rc->correctable++;
-		  mark_sector(rc, sector, Closure->green);
+		  mark_sector(rc, sector, Closure->greenSector);
 	       }
 	    }
 	 }
@@ -886,12 +905,12 @@ static void mark_rs02_headers(read_closure *rc)
    while(hpos < end)
    {  if(!GetBit(rc->map, hpos))
       {  SetBit(rc->map, hpos);
-	 mark_sector(rc, hpos, Closure->green);
+	 mark_sector(rc, hpos, Closure->greenSector);
 	 rc->correctable++;
       }
       if(!GetBit(rc->map, hpos+1))
       {  SetBit(rc->map, hpos+1);
-         mark_sector(rc, hpos+1, Closure->green);
+         mark_sector(rc, hpos+1, Closure->greenSector);
 	 rc->correctable++;
       }
 
@@ -934,7 +953,12 @@ void fill_gap(read_closure *rc)
   gint64 i,j;
   gint64 firstUnwritten;
 
-  firstUnwritten = rc->highestWrittenSector + 1;
+  /*** Start filling after rc->highestWrittenSector unless we are
+       skipping from Sector 0 to the user selected start area. */
+
+  if(rc->firstSector > 0 && rc->highestWrittenSector == 0)
+       firstUnwritten = 0;
+  else firstUnwritten = rc->highestWrittenSector + 1;
 
   /*** Tell user what's going on */
 
@@ -943,7 +967,7 @@ void fill_gap(read_closure *rc)
   clear_progress(rc);
   if(Closure->guiMode)
   {  SetAdaptiveReadSubtitle(t);
-     ChangeSpiralCursor(-1); 
+     ChangeSpiralCursor(Closure->readAdaptiveSpiral, -1); 
   }
   PrintCLI(t);
   g_free(t);
@@ -970,7 +994,7 @@ void fill_gap(read_closure *rc)
 	     
      if(Closure->stopActions)
      {  if(Closure->guiMode)
-	 SetAdaptiveReadFootline(_("Aborted by user request!"), Closure->red);
+	 SetAdaptiveReadFootline(_("Aborted by user request!"), Closure->redText);
 
         rc->earlyTermination = FALSE;  /* suppress respective error message */
 	cleanup((gpointer)rc);
@@ -980,8 +1004,11 @@ void fill_gap(read_closure *rc)
 
      if(j++ % 2000)
      {  int seq = (j/2000)%10;
-       
-        PrintCLI(anim[seq]);
+
+	if(!Closure->guiMode)
+        {  g_printf(anim[seq]);
+	   fflush(stdout);   /* at least needed for Windows */
+	}
      }
 	
      /* Show progress in the spiral */
@@ -990,7 +1017,7 @@ void fill_gap(read_closure *rc)
      {  int segment = i / rc->sectorsPerSegment;
        
         if(Closure->readAdaptiveSpiral->segmentColor[segment] == Closure->background)
-	  ChangeSegmentColor(Closure->white, segment);
+	  ChangeSegmentColor(Closure->whiteSector, segment);
      }
   }
 
@@ -1034,17 +1061,16 @@ void fill_correctable_gap(read_closure *rc, gint64 correctable)
 
 void ReadMediumAdaptive(gpointer data)
 {  read_closure *rc;
-   int crcidx = 0;
    gint64 s;
    gint64 image_file_size;
-   guint32 *crcbuf;
    int status,i,n;
 
    /*** Initialize the read closure. */
 
    rc = g_malloc0(sizeof(read_closure));
 
-   crcbuf = rc->crcbuf = g_malloc(sizeof(guint32)*CRCBUFSIZE);
+   rc->ab = CreateAlignedBuffer(MAX_CLUSTER_SIZE);
+   rc->buf = rc->ab->buf;
 
    memset(rc->progressBs, '\b', 256);
    memset(rc->progressSp, ' ', 256);
@@ -1063,22 +1089,22 @@ void ReadMediumAdaptive(gpointer data)
 
    open_and_determine_mode(rc);
 
+   /*** Compare image and ecc fingerprints (only if RS01 type .ecc is available) */
+
+   if(rc->readMode == ECC_IN_FILE)
+      check_ecc_fingerprint(rc);
+
    /*** Validate image size against ecc data */
    
    check_size(rc);
 
    /*** Limit the read range from users choice */
 
-   get_reading_range(rc);
+   GetReadingRange(rc->sectors, &rc->firstSector, &rc->lastSector);
    rc->intervalStart = rc->firstSector;
    rc->intervalEnd   = rc->lastSector;
    rc->intervalSize  = rc->intervalEnd - rc->intervalStart + 1;
 
-   /*** Compare image and ecc fingerprints (only if RS01 type .ecc is available) */
-
-   if(rc->readMode == ECC_IN_FILE)
-     check_fingerprint(rc);
-
    /*** Initialize the sector bitmap if ecc information is present.
 	This is used to stop when sufficient data for error correction
 	becomes available. */
@@ -1121,9 +1147,13 @@ reopen_image:
 		     _("Reading new medium image."),
 		     rc->dh->mediumDescr);
 
-   /* Mark RS02 header sectors as correctable. */
+      /* Mark RS02 header sectors as correctable. */
 
       mark_rs02_headers(rc);
+
+      /* Preload the CRC buffer */
+
+      load_crc_buf(rc);
    }
 
    /*** else examine the existing image file ***/
@@ -1152,6 +1182,10 @@ reopen_image:
 
       check_image_size(rc, image_file_size / 2048);
 
+      /* Preload the CRC buffer */
+
+      load_crc_buf(rc);
+
       /* Build the interval list */
 
       build_interval_from_image(rc);
@@ -1166,7 +1200,7 @@ reopen_image:
       {  char *t = _("\nSufficient data for reconstructing the image is available.\n");
 	 PrintLog(t);
 	 if(Closure->guiMode)
-	   SetAdaptiveReadFootline(t, Closure->darkgreen);
+	   SetAdaptiveReadFootline(t, Closure->greenText);
 	 goto finished;
       }
 
@@ -1187,7 +1221,8 @@ reopen_image:
      SetAdaptiveReadSubtitle(rc->subtitle);
 
    for(;;)
-   {  
+   {  int cluster_mask = rc->dh->clusterSize-1;
+
       /* If we jumped beyond the highest writtensector, 
 	 fill the gap with dead sector markers. */
 
@@ -1197,30 +1232,28 @@ reopen_image:
       /*** Try reading the next interval */
 
       print_progress(rc, TRUE);
-      crcidx = CRCBUFSIZE;
 
       for(s=rc->intervalStart; s<=rc->intervalEnd; ) /* s is incremented elsewhere */
       {  int nsectors,cnt;
-	 int ignore_fatal = 0;
  
 	 if(Closure->stopActions)          /* somebody hit the Stop button */
 	 {  if(Closure->guiMode)
-	       SetAdaptiveReadFootline(_("Aborted by user request!"), Closure->red);
+	       SetAdaptiveReadFootline(_("Aborted by user request!"), Closure->redText);
 
 	    rc->earlyTermination = FALSE;  /* suppress respective error message */
 	    goto terminate;
 	 }
 
 	 if(Closure->guiMode)
-	   ChangeSpiralCursor(s / rc->sectorsPerSegment);
+	    ChangeSpiralCursor(Closure->readAdaptiveSpiral, s / rc->sectorsPerSegment);
 	    
-	 /* Determine number of sectors to read. Read the next 16 sectors
+	 /* Determine number of sectors to read. Read the next dh->clusterSize sectors
 	    unless we're at the end of the interval or at a position which is
-	    not divideable by 16. */
+	    not divideable by the cluster size. */
 
-	 if(s & 15)
+	 if(s & cluster_mask)
                nsectors = 1;
-	 else  nsectors = 16;
+	 else  nsectors = rc->dh->clusterSize;
 
 	 if(s+nsectors > rc->intervalEnd) nsectors = rc->intervalEnd-s+1;
 
@@ -1243,7 +1276,6 @@ reopen_image:
 
 	       if(GetBit(rc->map, s))
 	       {  s++;
-		  crcidx++;
 		  continue;  /* restart reading loop with next sector */
 	       }
 	    }
@@ -1251,18 +1283,19 @@ reopen_image:
 
 	 /* Try to actually read the next sector(s) */
 
-	 status = ReadSectors(rc->dh, Closure->scratchBuf, s, nsectors);
+	 status = ReadSectors(rc->dh, rc->buf, s, nsectors);
 
 	 /* Medium Error (3) and Illegal Request (5) may result from 
 	    a medium read problem, but other errors are regarded as fatal. */
 
-	 if(status && !ignore_fatal
+	 if(status && !Closure->ignoreFatalSense
 	    && rc->dh->sense.sense_key 
 	    && rc->dh->sense.sense_key != 3 && rc->dh->sense.sense_key != 5)
 	 {  int answer;
 
 	    if(!Closure->guiMode)
-	      Stop(_("Sector %lld: %s\nCan not recover from above error."),
+	      Stop(_("Sector %lld: %s\nCan not recover from above error.\n"
+		     "Use the --ignore-fatal-sense option to override."),
 		   s, GetLastSenseString(FALSE));
 
 	    answer = ModalDialog(GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, insert_buttons,
@@ -1272,10 +1305,10 @@ reopen_image:
 				 s, GetLastSenseString(FALSE));
 
 	    if(answer == 2)
-	      ignore_fatal = TRUE;
+	      Closure->ignoreFatalSense = 2;
 
 	    if(!answer)
-	    {  SetAdaptiveReadFootline(_("Aborted by unrecoverable error."), Closure->red);
+	    {  SetAdaptiveReadFootline(_("Aborted by unrecoverable error."), Closure->redText);
 
 	       rc->earlyTermination = FALSE;  /* suppress respective error message */
 	       goto terminate;
@@ -1295,80 +1328,49 @@ reopen_image:
 	       otherwise treat them as unprocessed. */
 
 	    for(i=0, b=s; i<nsectors; i++,b++)
-	    {  guint32 crc;
+	    {  int result;
 
-	       if(rc->readMode == ECC_IN_FILE) /* we have crc information */
-	       { 
-		 /* If the CRC buf is exhausted, refill it. */
+	       /* Calculate and compare CRC sums.
+		  Sectors with bad CRC sums are marked unvisited,
+		  but do not terminate the current interval. */
 
-		 if(crcidx >= CRCBUFSIZE)  
-		 {  int remain = rc->sectors-b;
-		    int size;
+	       if(rc->crcBuf) /* we have crc information */
+		    result = CheckAgainstCrcBuffer(rc->crcBuf, b, rc->buf+i*2048);
+	       else result = CRC_UNKNOWN;
 
-		    if(remain < CRCBUFSIZE)
-		         size = remain*sizeof(guint32);
-	            else size = CRCBUFSIZE*sizeof(guint32);
+	       switch(result)
+	       {  case CRC_BAD:
+		     PrintCLI("\n");
+		     PrintCLI(_("CRC error in sector %lld\n"),b);
+		     print_progress(rc, TRUE);
 
-		    if(!LargeSeek(rc->ei->file, (gint64)sizeof(EccHeader)+b*4))
-		      Stop(_("Failed seeking in crc area: %s"),strerror(errno));
+		     n = LargeWrite(rc->image, Closure->deadSector, 2048);
+		     if(n != 2048)
+			Stop(_("Failed writing to sector %lld in image [%s]: %s"),
+			     b, "unv", strerror(errno));
 
-		    if(LargeRead(rc->ei->file, crcbuf, size) != size)
-		      Stop(_("Error reading CRC information: %s"),strerror(errno));
+		     mark_sector(rc, b, Closure->yellowSector);
+		     
+		     if(rc->highestWrittenSector < b)
+			rc->highestWrittenSector = b;
+		     break;
 
-		    crcidx = 0;
-		 }
+		  case CRC_UNKNOWN:
+		  case CRC_GOOD:
+		     n = LargeWrite(rc->image, rc->buf+i*2048, 2048);
+		     if(n != 2048)
+			Stop(_("Failed writing to sector %lld in image [%s]: %s"),
+			     b, "store", strerror(errno));
 
-		 /* Calculate and compare CRC sums.
-		    Sectors with bad CRC sums are marked unvisited,
-		    but do not terminate the current interval. */
+		     if(rc->map)
+			SetBit(rc->map, b);
+		     rc->readable++;
 
-		 crc = Crc32(Closure->scratchBuf+i*2048, 2048); 
-
-		 if(crc != crcbuf[crcidx++])  /* failed CRC test */
-		 {  PrintCLI("\n");
-		    PrintCLI(_("CRC error in sector %lld\n"),b);
-		    print_progress(rc, TRUE);
-
-		    n = LargeWrite(rc->image, Closure->deadSector, 2048);
-		    if(n != 2048)
-		      Stop(_("Failed writing to sector %lld in image [%s]: %s"),
-			   b, "unv", strerror(errno));
-
-		    mark_sector(rc, b, Closure->yellow);
-
-		    if(rc->highestWrittenSector < b)
-		      rc->highestWrittenSector = b;
-		 }
-		 else /* good sector */
-		 {  n = LargeWrite(rc->image, Closure->scratchBuf+i*2048, 2048);
-		    if(n != 2048)
-		      Stop(_("Failed writing to sector %lld in image [%s]: %s"),
-			   b, "store", strerror(errno));
-
-		    SetBit(rc->map, b);
-		    rc->readable++;
-
-		    mark_sector(rc, b, Closure->green);
+		     mark_sector(rc, b, Closure->greenSector);
 		    
-		    if(rc->highestWrittenSector < b)
-		      rc->highestWrittenSector = b;
-		 }
-	       }
-
-	       else /* no crc information available */
-	       {  n = LargeWrite(rc->image, Closure->scratchBuf+i*2048, 2048);
-		  if(n != 2048)
-		    Stop(_("Failed writing to sector %lld in image [%s]: %s"),
-			 b, "store", strerror(errno));
-
-		  if(rc->map)
-		    SetBit(rc->map, b);
-		  rc->readable++;
-
-		  mark_sector(rc, b, Closure->green);
-
-		  if(rc->highestWrittenSector < b)
-		    rc->highestWrittenSector = b;
+		     if(rc->highestWrittenSector < b)
+			rc->highestWrittenSector = b;
+		     break;
 	       }
 	    }
 
@@ -1380,7 +1382,7 @@ reopen_image:
 		  gint64 layer_idx;
 
 #ifdef CHECK_VISITED
-		  count[b]++;
+		  rc->count[b]++;
 #endif
 		  /* Count available sectors. */
 
@@ -1403,10 +1405,10 @@ reopen_image:
 			   && !GetBit(rc->map, layer_idx))
 			{  SetBit(rc->map, layer_idx);
 			   rc->correctable++;
-			   mark_sector(rc, layer_idx, Closure->green);
+			   mark_sector(rc, layer_idx, Closure->greenSector);
 
 #ifdef CHECK_VISITED
-			   count[layer_idx]++;
+			   rc->count[layer_idx]++;
 #endif
 
 			   /* If the correctable sector lies beyond the highest written sector,
@@ -1446,7 +1448,7 @@ reopen_image:
 			   && !GetBit(rc->map, sector))
 			{  SetBit(rc->map, sector);
 			   rc->correctable++;
-			   mark_sector(rc, sector, Closure->green);
+			   mark_sector(rc, sector, Closure->greenSector);
 			   fill_correctable_gap(rc, sector);
 			}
 		     }
@@ -1471,7 +1473,9 @@ reopen_image:
 	       print_progress(rc, TRUE);
 	       PrintLog(t);
 	       if(Closure->guiMode && rc->ei)
-		 SetAdaptiveReadFootline(t, Closure->black);
+		  SetAdaptiveReadFootline(t, Closure->foreground);
+	       if(Closure->eject)
+		  LoadMedium(rc->dh, FALSE);
 	       goto finished;
 	    }
 	 }  /* end of if(!status) (successful reading of sector(s)) */
@@ -1499,7 +1503,7 @@ reopen_image:
 		 Stop(_("Failed writing to sector %lld in image [%s]: %s"),
 		      s, "nds", strerror(errno));
 
-	       mark_sector(rc, s+i, Closure->red);
+	       mark_sector(rc, s+i, Closure->redSector);
 	    }
 
 	    if(rc->highestWrittenSector < s+nsectors)
@@ -1519,7 +1523,7 @@ reopen_image:
 		       rc->intervalSize, rc->intervalStart, rc->intervalStart+rc->intervalSize-1);
 
 	       add_interval(rc, rc->intervalStart, rc->intervalSize);
-	       if(Closure->verbose) print_intervals(rc);
+	       //print_intervals(rc);
 	    }
 	    break; /* fall out of reading loop */
 	 }
@@ -1564,7 +1568,7 @@ reopen_image:
 	 Verbose("*** Popped [%lld]\n",rc->intervalStart);
       }
 
-      if(Closure->verbose) print_intervals(rc);
+      //print_intervals(rc);
 
       rc->intervalSize  = rc->intervalEnd-rc->intervalStart+1;
 
@@ -1580,25 +1584,23 @@ finished:
 #ifdef CHECK_VISITED
    {  int i,cnt=0;
       for(i=0; i<(int)rc->sectors; i++)
-      {  cnt+=count[i];
-         if(count[i] != 1)
-           printf("Sector %d: %d\n",i,count[i]);
+      {  cnt+=rc->count[i];
+         if(rc->count[i] != 1)
+           printf("Sector %d: %d\n",i,rc->count[i]);
       }
 
       printf("\nTotal visited %d (%d)\n",cnt,i);
 
       for(i=(int)rc->sectors; i<(int)rc->sectors+160; i++)
-        if(count[i] != 0)
-          printf("SECTOR %d: %d\n",i,count[i]);
-
-      g_free(count);
+        if(rc->count[i] != 0)
+          printf("SECTOR %d: %d\n",i,rc->count[i]);
    }
 #endif
 
    /* Force output of final results */
 
    if(Closure->guiMode)
-   {  ChangeSpiralCursor(-1);
+   {  ChangeSpiralCursor(Closure->readAdaptiveSpiral, -1);
       mark_sector(rc, 0, NULL);
    }
 
@@ -1617,7 +1619,7 @@ finished:
 		  "(%lld readable,  %lld correctable,  %lld still missing).\n"),
 		t, rc->readable, rc->correctable, rc->sectors-total);
       if(Closure->guiMode)
-	 SetAdaptiveReadFootline(t, Closure->black);
+	 SetAdaptiveReadFootline(t, Closure->foreground);
 
       g_free(t);
       exitCode = EXIT_FAILURE;
@@ -1630,7 +1632,9 @@ finished:
       {  char *t = _("\nGood! All sectors have been read.\n"); 
 	 PrintLog(t);
 	 if(Closure->guiMode)
-	   SetAdaptiveReadFootline(t, Closure->black);
+	   SetAdaptiveReadFootline(t, Closure->foreground);
+	 if(Closure->eject)
+	    LoadMedium(rc->dh, FALSE);
       }
       else
       {  int percent = (int)((1000LL*rc->readable)/rc->sectors);
@@ -1642,7 +1646,7 @@ finished:
 		   t, percent/10, percent%10, rc->readable);
 
 	 if(Closure->guiMode)
-	   SetAdaptiveReadFootline(t, Closure->black);
+	   SetAdaptiveReadFootline(t, Closure->foreground);
 	 g_free(t);
 	 exitCode = EXIT_FAILURE;
       }
