@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2006 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2008 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -55,6 +55,7 @@ static void read_crc(LargeFile *ecc, guint32 *buf, int first_sector, int n_secto
 typedef struct
 {  RS01Widgets *wl;
    GaloisTables *gt;
+   ReedSolomonTables *rt;
    int earlyTermination;
    char *msg;
    ImageInfo *ii;
@@ -73,7 +74,8 @@ static void fix_cleanup(gpointer data)
    {  if(fc->earlyTermination)
          SwitchAndSetFootline(fc->wl->fixNotebook, 1,
 			      fc->wl->fixFootline,
-			      _("<span color=\"red\">Aborted by unrecoverable error.</span>")); 
+			      _("<span %s>Aborted by unrecoverable error.</span>"),
+			      Closure->redMarkup); 
       AllowActions(TRUE);
    }
 
@@ -91,10 +93,12 @@ static void fix_cleanup(gpointer data)
    }
 
    if(fc->gt) FreeGaloisTables(fc->gt);
-
+   if(fc->rt) FreeReedSolomonTables(fc->rt);
+ 
    g_free(fc);
 
-   g_thread_exit(0);
+   if(Closure->guiMode)
+      g_thread_exit(0);
 }
 
 /*
@@ -104,6 +108,7 @@ static void fix_cleanup(gpointer data)
 void RS01Fix(Method *method)
 {  RS01Widgets *wl = (RS01Widgets*)method->widgetList;
    GaloisTables *gt;
+   ReedSolomonTables *rt;
    fix_closure *fc = g_malloc0(sizeof(fix_closure)); 
    ImageInfo *ii = NULL;
    EccInfo   *ei = NULL;
@@ -180,7 +185,8 @@ void RS01Fix(Method *method)
         if(!answer)
         {  SwitchAndSetFootline(fc->wl->fixNotebook, 1,
 				fc->wl->fixFootline,
-				_("<span color=\"red\">Aborted by user request!</span>")); 
+				_("<span %s>Aborted by user request!</span>"),
+				Closure->redMarkup); 
 	   fc->earlyTermination = FALSE;  /* suppress respective error message */
 	   goto terminate;
 	}
@@ -201,7 +207,8 @@ void RS01Fix(Method *method)
        if(!answer)
        {  SwitchAndSetFootline(fc->wl->fixNotebook, 1,
 			       fc->wl->fixFootline,
-			       _("<span color=\"red\">Aborted by user request!</span>")); 
+			       _("<span %s>Aborted by user request!</span>"),
+			       Closure->redMarkup); 
 	  fc->earlyTermination = FALSE;  /* suppress respective error message */
 	  goto terminate;
        }
@@ -246,7 +253,8 @@ void RS01Fix(Method *method)
 	 if(!answer)
 	 {  SwitchAndSetFootline(fc->wl->fixNotebook, 1,
 				 fc->wl->fixFootline,
-				 _("<span color=\"red\">Aborted by user request!</span>")); 
+				 _("<span %s>Aborted by user request!</span>"),
+				 Closure->redMarkup); 
 	    fc->earlyTermination = FALSE;  /* suppress respective error message */
 	    goto terminate;
 	 }
@@ -275,7 +283,8 @@ void RS01Fix(Method *method)
       if(!answer)
       {  SwitchAndSetFootline(fc->wl->fixNotebook, 1,
 			      fc->wl->fixFootline,
-			      _("<span color=\"red\">Aborted by user request!</span>")); 
+			      _("<span %s>Aborted by user request!</span>"),
+			      Closure->redMarkup); 
 	 fc->earlyTermination = FALSE;  /* suppress respective error message */
 	 goto terminate;
       }
@@ -291,7 +300,8 @@ void RS01Fix(Method *method)
       if(!answer)
       {  SwitchAndSetFootline(fc->wl->fixNotebook, 1,
 			      fc->wl->fixFootline,
-			      _("<span color=\"red\">Aborted by user request!</span>")); 
+			      _("<span %s>Aborted by user request!</span>"),
+			      Closure->redMarkup); 
 	 fc->earlyTermination = FALSE;  /* suppress respective error message */
 	 goto terminate;
       }
@@ -302,11 +312,14 @@ void RS01Fix(Method *method)
 
    /*** Set up the Galois field arithmetic */
 
-   gt = fc->gt = CreateGaloisTables(eh->eccBytes);
-   nroots      = gt->nroots;
-   ndata       = gt->ndata;
-   gf_index_of = gt->index_of;
-   gf_alpha_to = gt->alpha_to;
+   gt = fc->gt = CreateGaloisTables(RS_GENERATOR_POLY);
+   rt = fc->rt = CreateReedSolomonTables(gt, RS_FIRST_ROOT, RS_PRIM_ELEM, eh->eccBytes);
+
+   gf_index_of = gt->indexOf;
+   gf_alpha_to = gt->alphaTo;
+
+   nroots      = rt->nroots;
+   ndata       = rt->ndata;
 
    /*** Prepare buffers for ecc code processing.
 	Our ecc blocks are built from ndata medium sectors spread over the full medium size.
@@ -340,7 +353,8 @@ void RS01Fix(Method *method)
      if(Closure->stopActions) /* User hit the Stop button */
      {   SwitchAndSetFootline(fc->wl->fixNotebook, 1,
 			      fc->wl->fixFootline,
-			      _("<span color=\"red\">Aborted by user request!</span>")); 
+			      _("<span %s>Aborted by user request!</span>"),
+			      Closure->redMarkup); 
          fc->earlyTermination = FALSE;  /* suppress respective error message */
 	 goto terminate;
      }
@@ -464,12 +478,12 @@ void RS01Fix(Method *method)
 	   for(i=0; i<nroots; i++)
 	     s[i] = fc->imgBlock[0][offset];
 
-	   for(j=1; j<FIELDMAX; j++)
+	   for(j=1; j<GF_FIELDMAX; j++)
 	   {  int data = j>=ndata ? parity[j-ndata] : fc->imgBlock[j][offset];
 
 	      for(i=0;i<nroots;i++)
               {  if(s[i] == 0) s[i] = data;
-	         else s[i] = data ^ gf_alpha_to[mod_fieldmax(gf_index_of[s[i]] + (FIRST_ROOT+i)*PRIM_ELEM)];
+	         else s[i] = data ^ gf_alpha_to[mod_fieldmax(gf_index_of[s[i]] + (RS_FIRST_ROOT+i)*RS_PRIM_ELEM)];
 	      }
 	   }
 
@@ -497,12 +511,12 @@ void RS01Fix(Method *method)
 	   memset(lambda+1, 0, nroots*sizeof(lambda[0]));
 	   lambda[0] = 1;
 
-	   lambda[1] = gf_alpha_to[mod_fieldmax(PRIM_ELEM*(FIELDMAX-1-erasure_list[0]))];
+	   lambda[1] = gf_alpha_to[mod_fieldmax(RS_PRIM_ELEM*(GF_FIELDMAX-1-erasure_list[0]))];
 	   for(i=1; i<erasure_count; i++) 
-	   {  u = mod_fieldmax(PRIM_ELEM*(FIELDMAX-1-erasure_list[i]));
+	   {  u = mod_fieldmax(RS_PRIM_ELEM*(GF_FIELDMAX-1-erasure_list[i]));
 	      for(j=i+1; j>0; j--) 
 	      {  tmp = gf_index_of[lambda[j-1]];
-	         if(tmp != ALPHA0)
+	         if(tmp != GF_ALPHA0)
 		   lambda[j] ^= gf_alpha_to[mod_fieldmax(u + tmp)];
 	      }
 	   }
@@ -518,22 +532,22 @@ void RS01Fix(Method *method)
 	   {  
 	     discr_r = 0;
 	     for(i=0; i<r; i++)
-	       if((lambda[i] != 0) && (s[r-i-1] != ALPHA0))
+	       if((lambda[i] != 0) && (s[r-i-1] != GF_ALPHA0))
 		 discr_r ^= gf_alpha_to[mod_fieldmax(gf_index_of[lambda[i]] + s[r-i-1])];
 
 	     discr_r = gf_index_of[discr_r];	/* Index form */
 
-	     if(discr_r == ALPHA0) 
+	     if(discr_r == GF_ALPHA0) 
 	     {
 	        /* B(x) = x*B(x) */
 	        memmove(b+1, b, nroots*sizeof(b[0]));
-	        b[0] = ALPHA0;
+	        b[0] = GF_ALPHA0;
 	     } 
 	     else 
 	     {  /* T(x) = lambda(x) - discr_r*x*b(x) */
 	        t[0] = lambda[0];
 	        for(i=0; i<nroots; i++) 
-		{  if(b[i] != ALPHA0)
+		{  if(b[i] != GF_ALPHA0)
 		        t[i+1] = lambda[i+1] ^ gf_alpha_to[mod_fieldmax(discr_r + b[i])];
 		   else t[i+1] = lambda[i+1];
 		}
@@ -543,12 +557,12 @@ void RS01Fix(Method *method)
 
 		   /* B(x) <-- inv(discr_r) * lambda(x) */
 		   for(i=0; i<=nroots; i++)
-		     b[i] = (lambda[i] == 0) ? ALPHA0 : mod_fieldmax(gf_index_of[lambda[i]] - discr_r + FIELDMAX);
+		     b[i] = (lambda[i] == 0) ? GF_ALPHA0 : mod_fieldmax(gf_index_of[lambda[i]] - discr_r + GF_FIELDMAX);
 		} 
 		else 
 		{  /* 2 lines below: B(x) <-- x*B(x) */
 		   memmove(b+1, b, nroots*sizeof(b[0]));
-		   b[0] = ALPHA0;
+		   b[0] = GF_ALPHA0;
 		}
 
 		memcpy(lambda,t,(nroots+1)*sizeof(t[0]));
@@ -559,7 +573,7 @@ void RS01Fix(Method *method)
 	   deg_lambda = 0;
 	   for(i=0; i<nroots+1; i++)
 	   {  lambda[i] = gf_index_of[lambda[i]];
-	      if(lambda[i] != ALPHA0)
+	      if(lambda[i] != GF_ALPHA0)
 		deg_lambda = i;
 	   }
 
@@ -567,11 +581,11 @@ void RS01Fix(Method *method)
 	   memcpy(reg+1, lambda+1, nroots*sizeof(reg[0]));
 	   count = 0;		/* Number of roots of lambda(x) */
 
-	   for(i=1, k=PRIMTH_ROOT-1; i<=FIELDMAX; i++, k=mod_fieldmax(k+PRIMTH_ROOT))
+	   for(i=1, k=RS_PRIMTH_ROOT-1; i<=GF_FIELDMAX; i++, k=mod_fieldmax(k+RS_PRIMTH_ROOT))
 	   {  q=1; /* lambda[0] is always 0 */
 
 	      for(j=deg_lambda; j>0; j--)
-	      {  if(reg[j] != ALPHA0) 
+	      {  if(reg[j] != GF_ALPHA0) 
 		 {  reg[j] = mod_fieldmax(reg[j] + j);
 		    q ^= gf_alpha_to[reg[j]];
 		 }
@@ -593,7 +607,7 @@ void RS01Fix(Method *method)
 	   /* deg(lambda) unequal to number of roots => uncorrectable error detected */
 
 	   if(deg_lambda != count)
-	   {  PrintLog(_("Decoder problem (%d != %d) for %d sectors: "), deg_lambda, count, erasure_count);
+	   {  PrintLog("Decoder problem (%d != %d) for %d sectors: ", deg_lambda, count, erasure_count);
 
 	      for(i=0; i<erasure_count; i++)
 	      {  gint64 idx = block_idx[erasure_list[i]];
@@ -612,7 +626,7 @@ void RS01Fix(Method *method)
 	   for(i=0; i<=deg_omega; i++)
 	   {  tmp = 0;
 	      for(j=i; j>=0; j--)
-	      {  if((s[i - j] != ALPHA0) && (lambda[j] != ALPHA0))
+	      {  if((s[i - j] != GF_ALPHA0) && (lambda[j] != GF_ALPHA0))
 		   tmp ^= gf_alpha_to[mod_fieldmax(s[i - j] + lambda[j])];
 	      }
 
@@ -628,17 +642,17 @@ void RS01Fix(Method *method)
 	   {  num1 = 0;
 
 	      for(i=deg_omega; i>=0; i--) 
-	      {  if(omega[i] != ALPHA0)
+	      {  if(omega[i] != GF_ALPHA0)
 		    num1 ^= gf_alpha_to[mod_fieldmax(omega[i] + i * root[j])];
 	      }
 
-	      num2 = gf_alpha_to[mod_fieldmax(root[j] * (FIRST_ROOT - 1) + FIELDMAX)];
+	      num2 = gf_alpha_to[mod_fieldmax(root[j] * (RS_FIRST_ROOT - 1) + GF_FIELDMAX)];
 	      den = 0;
     
 	      /* lambda[i+1] for i even is the formal derivative lambda_pr of lambda[i] */
 
 	      for(i=MIN(deg_lambda, nroots-1) & ~1; i>=0; i-=2) 
-	      {  if(lambda[i+1] != ALPHA0)
+	      {  if(lambda[i+1] != GF_ALPHA0)
 		   den ^= gf_alpha_to[mod_fieldmax(lambda[i+1] + i * root[j])];
 	      }
 
@@ -650,7 +664,7 @@ void RS01Fix(Method *method)
 		 if(location >= 0 && location < ndata)
 		 {  if(erasure_map[location] == 3)
 		    {  int old = fc->imgBlock[location][offset];
-		       int new = old ^ gf_alpha_to[mod_fieldmax(gf_index_of[num1] + gf_index_of[num2] + FIELDMAX - gf_index_of[den])];
+		       int new = old ^ gf_alpha_to[mod_fieldmax(gf_index_of[num1] + gf_index_of[num2] + GF_FIELDMAX - gf_index_of[den])];
 
 		       PrintCLI(_("-> Error located in sector %lld at byte %4d (value %02x '%c', expected %02x '%c')\n"),
 				block_idx[location], bi, 
@@ -662,7 +676,7 @@ void RS01Fix(Method *method)
 		      PrintLog(_("Unexpected byte error in sector %lld, byte %d\n"),
 			       block_idx[location], bi);
 
-		    fc->imgBlock[location][offset] ^= gf_alpha_to[mod_fieldmax(gf_index_of[num1] + gf_index_of[num2] + FIELDMAX - gf_index_of[den])];
+		    fc->imgBlock[location][offset] ^= gf_alpha_to[mod_fieldmax(gf_index_of[num1] + gf_index_of[num2] + GF_FIELDMAX - gf_index_of[den])];
 		 }
 		 else
 		   PrintLog(_("Bad error location %d; corrupted .ecc file?\n"), location);
@@ -739,8 +753,8 @@ skip:
       if(Closure->guiMode)
         SwitchAndSetFootline(wl->fixNotebook, 1, wl->fixFootline,
 			     _("Image sectors could not be fully restored "
-			       "(%lld repaired; <span color=\"red\">%lld unrepaired</span>)"),
-			     corrected, uncorrected);
+			       "(%lld repaired; <span %s>%lld unrepaired</span>)"),
+			     corrected, Closure->redMarkup, uncorrected);
    }
    else
    {  if(!corrected)
