@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2006 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2008 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -229,7 +229,11 @@ RS02Layout *CalcRS02Layout(gint64 data_sectors, int requested_roots)
             lay->mediumCapacity = Closure->cdSize;   /* CDR */
       else if(data_sectors < Closure->dvdSize1)
             lay->mediumCapacity = Closure->dvdSize1; /* Single layered DVD */
-      else  lay->mediumCapacity = Closure->dvdSize2; /* Double layered DVD */
+      else if(data_sectors < Closure->dvdSize2)
+	    lay->mediumCapacity = Closure->dvdSize2; /* Double layered DVD */
+      else if(data_sectors < Closure->bdSize1)
+	    lay->mediumCapacity = Closure->bdSize1;  /* Single layered BD */
+      else  lay->mediumCapacity = Closure->bdSize2;  /* Double layered BD */
    }
 
    lay->dataSectors      = data_sectors;
@@ -243,7 +247,7 @@ RS02Layout *CalcRS02Layout(gint64 data_sectors, int requested_roots)
       lay->nroots = requested_roots;
    else
    {  lay->rsSectors        = lay->mediumCapacity - lay->protectedSectors;     /* just to start */
-      lay->nroots           = (FIELDMAX*lay->rsSectors) / lay->mediumCapacity; /* iteration below */
+      lay->nroots           = (GF_FIELDMAX*lay->rsSectors) / lay->mediumCapacity; /* iteration below */
    }
 
    if(lay->nroots > 170)   /* Cap redundancy to 200% */
@@ -253,7 +257,7 @@ RS02Layout *CalcRS02Layout(gint64 data_sectors, int requested_roots)
       we get about 20 to 40 copies of the header in the ecc section. */
 
    lay->headerModulo = 32;
-   lay->ndata        = FIELDMAX-lay->nroots;
+   lay->ndata        = GF_FIELDMAX-lay->nroots;
    ecc_area = lay->nroots * ((lay->protectedSectors + lay->ndata - 1) / lay->ndata); 
 
    while(ecc_area / lay->headerModulo > 40)
@@ -265,7 +269,7 @@ RS02Layout *CalcRS02Layout(gint64 data_sectors, int requested_roots)
    {  gint64 first_repeat;  /* first header which is interleaved with ecc sectors */
       gint64 interleaved;   /* number of ecc sectors after first header */
 
-      lay->ndata             = FIELDMAX-lay->nroots;
+      lay->ndata             = GF_FIELDMAX-lay->nroots;
       lay->rsSectors         = lay->nroots * ((lay->protectedSectors + lay->ndata - 1) / lay->ndata); 
       first_repeat  = (lay->protectedSectors + lay->headerModulo - 1) / lay->headerModulo;
       first_repeat *= lay->headerModulo;
@@ -277,6 +281,7 @@ RS02Layout *CalcRS02Layout(gint64 data_sectors, int requested_roots)
 
       lay->eccSectors         = 2 + lay->crcSectors + lay->rsSectors + 2*lay->headers;
       lay->sectorsPerLayer    = (lay->protectedSectors + lay->ndata - 1) / lay->ndata;
+
       lay->firstCrcLayerIndex = (2 + lay->dataSectors) % lay->sectorsPerLayer;
 
       if(requested_roots > 0)
