@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2006 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2008 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -32,6 +32,7 @@ typedef struct
    RS02Widgets *wl;
    RS02Layout *lay;
    GaloisTables *gt;
+   ReedSolomonTables *rt;
    ImageInfo *ii;
    EccHeader *eh;
    unsigned char *data;
@@ -54,7 +55,8 @@ static void ecc_cleanup(gpointer data)
    if(Closure->guiMode)
    {  if(ec->earlyTermination)
         SetLabelText(GTK_LABEL(ec->wl->encFootline),
-		     _("<span color=\"red\">Aborted by unrecoverable error.</span>")); 
+		     _("<span %s>Aborted by unrecoverable error.</span>"),
+		     Closure->redMarkup); 
       AllowActions(TRUE);
    }
 
@@ -67,6 +69,7 @@ static void ecc_cleanup(gpointer data)
    /*** Clean up */
 
    if(ec->gt) FreeGaloisTables(ec->gt);
+   if(ec->rt) FreeReedSolomonTables(ec->rt);
    if(ec->ii) FreeImageInfo(ec->ii);
    if(ec->eh) g_free(ec->eh);
    if(ec->lay) g_free(ec->lay);
@@ -81,7 +84,8 @@ static void ecc_cleanup(gpointer data)
 
    g_free(ec);
 
-   g_thread_exit(0);
+   if(Closure->guiMode)
+     g_thread_exit(0);
 }
 
 /***
@@ -100,11 +104,13 @@ static void abort_encoding(ecc_closure *ec, int truncate)
 	Stop(_("Could not truncate %s: %s\n"),Closure->imageName,strerror(errno));
 
       SetLabelText(GTK_LABEL(wl->encFootline), 
-		   _("<span color=\"red\">Aborted by user request!</span> (partial ecc data removed from image)")); 
+		   _("<span %s>Aborted by user request!</span> (partial ecc data removed from image)"),
+		   Closure->redMarkup); 
    }
    else
    {  SetLabelText(GTK_LABEL(wl->encFootline), 
-		   _("<span color=\"red\">Aborted by user request!</span>")); 
+		   _("<span %s>Aborted by user request!</span>"),
+		   Closure->redMarkup); 
    }
 
    ec->earlyTermination = FALSE;   /* suppress respective error message */
@@ -412,7 +418,7 @@ static void create_reed_solomon(ecc_closure *ec)
    int layer,chunk,i,j,k;
    unsigned char *par_ptr;
 static gint32 *gf_index_of;    /* These need to be static globals */
-static gint32 *gf_gpoly;       /* for optimization reasons. */
+static gint32 *rs_gpoly;       /* for optimization reasons. */
 static gint32 *enc_alpha_to;
 
    /*** Show the second progress bar */
@@ -428,11 +434,12 @@ static gint32 *enc_alpha_to;
 
    /*** Create table for Galois field math */
 
-   ec->gt = CreateGaloisTables(nroots);
+   ec->gt = CreateGaloisTables(RS_GENERATOR_POLY);
+   ec->rt = CreateReedSolomonTables(ec->gt, RS_FIRST_ROOT, RS_PRIM_ELEM, nroots);
 
-   gf_index_of  = ec->gt->index_of;
-   enc_alpha_to = ec->gt->enc_alpha_to;
-   gf_gpoly     = ec->gt->gpoly;
+   gf_index_of  = ec->gt->indexOf;
+   enc_alpha_to = ec->gt->encAlphaTo;
+   rs_gpoly     = ec->rt->gpoly;
 
    /*** Allocate buffers for the parity calculation and image data caching. 
 
@@ -454,7 +461,7 @@ static gint32 *enc_alpha_to;
    n_layer_sectors = n_parity_blocks/2048;
 
    if(n_layer_sectors*2048 != n_parity_blocks)
-     Stop(_("Internal error: parity blocks are not a multiple of sector size.\n"));
+     Stop("Internal error: parity blocks are not a multiple of sector size.\n");
 
    ec->parity = g_malloc(n_parity_bytes);
    ec->data   = g_malloc(n_layer_bytes);
@@ -533,9 +540,9 @@ static gint32 *enc_alpha_to;
 
 	    feedback = gf_index_of[ec->data[i] ^ par_idx[sp]];
 
-	    if(feedback != ALPHA0) /* non-zero feedback term */
+	    if(feedback != GF_ALPHA0) /* non-zero feedback term */
 	    {  register int spk = sp+1;
-	       register int *gpoly = gf_gpoly + nroots;
+	       register int *gpoly = rs_gpoly + nroots;
 
 	       switch(nroots-spk)  /* unrolled loop part1 */
 	       {  
@@ -887,7 +894,7 @@ static gint32 *enc_alpha_to;
 		   case  1: par_idx[spk++] ^= enc_alpha_to[feedback + *--gpoly];
 	       }
 
-	       par_idx[sp] = enc_alpha_to[feedback + gf_gpoly[0]];
+	       par_idx[sp] = enc_alpha_to[feedback + rs_gpoly[0]];
 	    }
 	    else                   /* zero feedback term */
 	      par_idx[sp] = 0;
