@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2006 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2008 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -60,45 +60,26 @@
 #ifdef SYS_LINUX
 #define MAX_CDB_SIZE CDROM_PACKET_SIZE
 
-typedef struct request_sense Sense;
+/* Now globally defined for all OSes here */
+//typedef struct request_sense Sense;
 #endif
 
 #ifdef SYS_FREEBSD
 #define MAX_CDB_SIZE SCSI_MAX_CDBLEN
-
-//typedef struct scsi_sense_data Sense;
-
-/* This is actually the little endian version of the
- * Linux Sense struct. FreeBSD does not have the asb entry.
- */
-
-typedef struct request_sense {
-	guint8 error_code		: 7;
-	guint8 valid			: 1;
-	guint8 segment_number;
-	guint8 sense_key		: 4;
-	guint8 reserved2		: 1;
-	guint8 ili			: 1;
-	guint8 reserved1		: 2;
-	guint8 information[4];
-	guint8 add_sense_len;
-	guint8 command_info[4];
-	guint8 asc;
-	guint8 ascq;
-	guint8 fruc;
-	guint8 sks[3];
-	guint8 asb[46];
-} Sense;
 #endif
 
-#ifdef SYS_MINGW
+#if defined(SYS_UNKNOWN) || defined(SYS_MINGW) || defined(SYS_NETBSD) || defined(SYS_SOLARIS) || defined(SYS_DARWIN)
 #define MAX_CDB_SIZE 16   /* longest possible SCSI command */
+#endif
 
-/* This is actually the little endian version of the
- * Linux Sense struct. Windows uses only the first 24 bytes.
+/* 
+ * The sense struct is named differently on various OSes,
+ * Some have subtle differences in the used data types.
+ * To avoid typecasting mayhem we simply reproduce the Linux
+ * version here and use it on all OS versions.
  */
 
-typedef struct request_sense {
+typedef struct _Sense {
 	guint8 error_code		: 7;
 	guint8 valid			: 1;
 	guint8 segment_number;
@@ -115,69 +96,18 @@ typedef struct request_sense {
 	guint8 sks[3];
 	guint8 asb[46];
 } Sense;
-#endif
-
-#ifdef SYS_UNKNOWN
-#define MAX_CDB_SIZE 16   /* longest possible SCSI command */
-
-/* This is actually the little endian version of the
- * Linux Sense struct. 
- */
-
-typedef struct request_sense {
-	guint8 error_code		: 7;
-	guint8 valid			: 1;
-	guint8 segment_number;
-	guint8 sense_key		: 4;
-	guint8 reserved2		: 1;
-	guint8 ili			: 1;
-	guint8 reserved1		: 2;
-	guint8 information[4];
-	guint8 add_sense_len;
-	guint8 command_info[4];
-	guint8 asc;
-	guint8 ascq;
-	guint8 fruc;
-	guint8 sks[3];
-	guint8 asb[46];
-} Sense;
-#endif
-
-#ifdef SYS_DARWIN
-#define MAX_CDB_SIZE 16   /* longest possible SCSI command */
-
-/* This is actually the little endian version of the
- * Linux Sense struct. 
- */
-
-typedef struct request_sense {
-	guint8 error_code		: 7;
-	guint8 valid			: 1;
-	guint8 segment_number;
-	guint8 sense_key		: 4;
-	guint8 reserved2		: 1;
-	guint8 ili			: 1;
-	guint8 reserved1		: 2;
-	guint8 information[4];
-	guint8 add_sense_len;
-	guint8 command_info[4];
-	guint8 asc;
-	guint8 ascq;
-	guint8 fruc;
-	guint8 sks[3];
-	guint8 asb[46];
-} Sense;
-#endif
 
 /***
  ***  The DeviceHandle is pretty much our device abstraction layer. 
  ***
- * It contains info about the opened device and the inserted medium
+ * It contains info about the opened device and the inserted medium.
  */
 
 typedef struct _DeviceHandle
-{
-#ifdef SYS_LINUX
+{  /*
+    * OS-specific data for device access
+    */
+#if defined(SYS_LINUX) || defined(SYS_NETBSD) || defined(SYS_SOLARIS)
    int fd;                    /* device file descriptor */
 #endif
 #ifdef SYS_FREEBSD
@@ -196,34 +126,112 @@ typedef struct _DeviceHandle
    SCSITaskInterface **taskInterface;
    IOVirtualRange *range;
 #endif
-   char *device;
-   char devinfo[34];          /* whole device info string */
+   
+   /*
+    * OS-independent data about the device
+    */
+
+   char *device;              /* /dev/foo or whatever the OS uses to name it */
+   char devinfo[34];          /* whole device info string from INQUIRY */
    char vendor[34];           /* vendor and product info only */
-   int category;
-   int (*read)(struct _DeviceHandle*, unsigned char*, int, int);
+
    Sense sense;               /* sense data from last operation */
-   gint64 userAreaSize;       /* size of user area according to DVD Info struct */
-   gint64 rs02Size;           /* size reported in RS02 header */
-   EccHeader *rs02Header;     /* copy of RS02 header */
-   int sessions;
-   int layers;
-   int mainType;
-   int subType;
-   char *typedescr;
-   int rewriteable;
-   gint64 sectors;            /* Number of sectors */
+
    double singleRate;         /* supposed KB/sec @ single speed */
    int maxRate;               /* guessed maximum transfer rate */
+   int clusterSize;           /* number of sectors per cluster */
+
+   /*
+    * Raw reading support
+    */
+
+   int canReadDefective;      /* TRUE if drive claims to raw read uncorrectable sectors */
+   int canC2Scan;             /* TRUE if drive supports C2 error scanning */
+   int c2[MAX_CLUSTER_SIZE];  /* C2 errors per sector */
+   unsigned char previousReadMode;/* read mode prior to switching to raw reads */
+   unsigned char previousRetries; /* retries prior to switching */
+   unsigned char currentReadMode; /* current raw read mode */
+   RawBuffer *rawBuffer;      /* for performing raw read analysis */
+   int (*read)(struct _DeviceHandle*, unsigned char*, int, int);
+   int (*readRaw)(struct _DeviceHandle*, unsigned char*, int, int);
+
+   /* 
+    * Information about currently inserted medium 
+    */
+
+   gint64 sectors;            /* actually used number of sectors */
+   int sessions;              /* number of sessions */
+   int layers;                /* and layers */
+   char manuID[20];           /* Manufacturer info from ADIP/lead-in */
+   int mainType;              /* CD or DVD */
+   int subType;               /* see enum below */
+   char *typeDescr;           /* human readable form of subType */
+   int bookType;              /* book type */
+   char *bookDescr;           /* human readable of above */
+   int profile;               /* profile selected by drive */
+   char *profileDescr;        /* human readable form of above */
+   int isDash;                /* DVD- */
+   int isPlus;                /* DVD+ */
+   int discStatus;            /* obtained from READ DISC INFORMATION query */
+   int rewriteable;
    char *mediumDescr;         /* textual description of medium */
+
+   guint8 mediumFP[16];       /* Medium fingerprint */
+   gint64 fpSector;           /* Sector used for calculating the fingerprint */
+   int fpState;               /* 0=unknown; 1=unreadable; 2=present */
+
+   /*
+    * size alternatives from different sources 
+    */
+
+   gint64 readCapacity;       /* value returned by READ CAPACITY */
+   gint64 userAreaSize;       /* size of user area according to DVD Info struct */
+   gint64 blankCapacity;      /* blank capacity (maybe 0 if query failed) */
+   gint64 rs02Size;           /* size reported in RS02 header */
+
+   /*
+    * file system(s) found on medium
+    */
+   
+   EccHeader *rs02Header;     /* copy of RS02 header */
    struct _IsoInfo *isoInfo;  /* Information gathered from ISO filesystem */
+
+   /*
+    * debugging stuff
+    */
+
    Bitmap *defects;           /* for defect simulation */
 } DeviceHandle;
 
-/* These seem not to be standardized anywhere,
- * so we make up our own enum here.
+/* 
+ * Media types seem not to be standardized anywhere,
+ * so we make up our own here.
  */
 
-enum { DATA1, XA21, CD, DVD, UNSUPPORTED };
+#define MAIN_TYPE_MASK 0xf0
+
+#define CD             0x10
+#define DATA1          0x11
+#define XA21           0x12
+
+#define DVD            0x20
+#define DVD_RAM        0x21
+#define DVD_DASH_R     0x22
+#define DVD_DASH_RW    0x23
+#define DVD_PLUS_R     0x24
+#define DVD_PLUS_RW    0x25
+#define DVD_DASH_R_DL  0x26
+#define DVD_DASH_RW_DL 0x27
+#define DVD_PLUS_R_DL  0x28
+#define DVD_PLUS_RW_DL 0x29
+
+#define BD             0x40
+#define BD_R           0x41
+#define BD_RE          0x42
+
+#define UNSUPPORTED    0x00 
+
+/* transport io modes */
 
 #define DATA_WRITE 0
 #define DATA_READ  1
@@ -252,10 +260,25 @@ int SendPacket(DeviceHandle*, unsigned char*, int, unsigned char*, int, Sense*,
  * The really user-visible stuff
  */
 
+enum 
+{  MODE_PAGE_UNSET, 
+   MODE_PAGE_SET
+};
+
 DeviceHandle* OpenAndQueryDevice(char*);
-int InquireDevice(DeviceHandle*, int); 
-void SpinupDevice(DeviceHandle*);
-int ReadSectors(DeviceHandle*, unsigned char*, gint64, int);
+DeviceHandle* QueryMediumInfo(char*);
+gint64 CurrentMediumSize(int);
+int  GetMediumFingerprint(DeviceHandle*, guint8*, gint64);
 void CloseDevice(DeviceHandle*);
 
+int InquireDevice(DeviceHandle*, int); 
+void SetRawMode(DeviceHandle*, int);
+
+void SpinupDevice(DeviceHandle*);
+void LoadMedium(struct _DeviceHandle*, int);
+int  TestUnitReady(DeviceHandle*);
+
+int ReadSectors(DeviceHandle*, unsigned char*, gint64, int);
+int ReadSectorsFast(DeviceHandle*, unsigned char*, gint64, int);
+
 #endif /* SCSI_LAYER_H */
