@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2006 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2008 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -32,7 +32,8 @@
  * to be more compatible.
  * Otoh, ASPI run without special priviledges and even on the older
  * Windows 9x versions. 
- * The APSI wrapper has only been tested against Adaptec ASPI so far.
+ * The ASPI wrapper has only been tested against the WNASPI32.DLL
+ * made by Adaptec.
  *
  * SPTI is tried first and then we fall back to ASPI.
  */
@@ -82,14 +83,17 @@ char* DefaultDevice()
       }
    }
 
-   /* If no drives appeared under SPTI, 
-      try looking for drives using ASPI. */
+   /* Try looking for drives using ASPI.
+      Changed behaviour since V0.72: 
+      We provide both SPTI and ASPI in the GUI. */
 
-   if(!Closure->deviceNodes->len && Closure->aspiLib)  
-   {  DeviceHandle *dh = open_aspi_device("A:", 2);
+   if(Closure->aspiLib)  
+   {  int none_picked = !Closure->deviceNodes->len;
+      DeviceHandle *dh = open_aspi_device("A:", 2);
       if(dh) CloseDevice(dh);
 
-      if(Closure->deviceNodes->len) /* pick first aspi drive */
+      if(none_picked 
+	 && Closure->deviceNodes->len) /* pick first aspi drive */
 	*picked = ((char*)g_ptr_array_index(Closure->deviceNodes,0))[0];
    }
 
@@ -105,12 +109,20 @@ char* DefaultDevice()
 
 void CloseDevice(DeviceHandle *dh)
 {
+  if(dh->canReadDefective)
+    SetRawMode(dh, MODE_PAGE_UNSET);
+
+  if(dh->rawBuffer)
+     FreeRawBuffer(dh->rawBuffer);
+
   if(!dh->aspiUsed)             /* SPTI cleanup */
   {  CloseHandle(dh->fd);
   }
 
   if(dh->rs02Header)
     g_free(dh->rs02Header);
+  if(dh->typeDescr) 
+    g_free(dh->typeDescr);
   if(dh->mediumDescr) 
      g_free(dh->mediumDescr);
   if(dh->isoInfo)
@@ -212,7 +224,7 @@ void OpenAspi()
   Closure->GetASPI32SupportInfo 
      = (DWORD(*)(void))GetProcAddress(Closure->aspiLib, "GetASPI32SupportInfo");
   if(!Closure->GetASPI32SupportInfo)
-  {  PrintLog(_("GetASPI32SupportInfo() not available."));
+  {  PrintLog("GetASPI32SupportInfo() not available.");
      FreeLibrary(Closure->aspiLib);
      Closure->aspiLib = NULL;
      return;
@@ -221,7 +233,7 @@ void OpenAspi()
   Closure->SendASPI32Command 
      = (DWORD(*)(void*))GetProcAddress(Closure->aspiLib, "SendASPI32Command");
   if(!Closure->SendASPI32Command)
-  {  PrintLog(_("SendASPI32Command() not available."));
+  {  PrintLog("SendASPI32Command() not available.");
      FreeLibrary(Closure->aspiLib);
      Closure->aspiLib = NULL;
      return;
@@ -280,7 +292,7 @@ DeviceHandle* open_aspi_device(char *device, int list_mode)
   max_ha = ret & 0xff;
 
   if(status != SS_COMP)
-  {  PrintLog(_("Could not determine number of host adapters\n"));
+  {  PrintLog("Could not determine number of host adapters\n");
      g_free(dh->device);
      g_free(dh); 
      return NULL;
@@ -301,7 +313,7 @@ DeviceHandle* open_aspi_device(char *device, int list_mode)
 
      Closure->SendASPI32Command(&ha_inq);
      if(ha_inq.Status != SS_COMP)
-       PrintLog(_("ASPI warning: Could not query host adapter %d\n"),ha);
+       PrintLog("ASPI warning: Could not query host adapter %d\n",ha);
 
 #if 0
      PrintLog("HA %d: %16s\n",ha,ha_inq.HA_Identifier);
@@ -322,7 +334,7 @@ DeviceHandle* open_aspi_device(char *device, int list_mode)
      /* Missing the following conditions is close to impossible. */
 
      if(ha_inq.HA_BufAlignMask >= 4096)
-     {  Stop(_("ASPI alignment = %d requested; can't handle that.\n"),
+     {  Stop("ASPI alignment = %d requested; can't handle that.\n",
 	     ha_inq.HA_BufAlignMask);
         g_free(dh);
 	return NULL;
@@ -331,8 +343,8 @@ DeviceHandle* open_aspi_device(char *device, int list_mode)
      /* Some ASPI drivers return 0 or -1 for actually working drives,
 	so this information is also useless. */
 
-     if(ha_inq.HA_MaxTransferLength < 32768)
-     {  Stop(_("ASPI max xfer length = %d; can't handle that.\n"),
+     if(ha_inq.HA_MaxTransferLength < MAX_CLUSTER_SIZE)
+     {  Stop("ASPI max xfer length = %d; can't handle that.\n",
 	     ha_inq.HA_MaxTransferLength);
         g_free(dh);
 	return NULL;
@@ -460,7 +472,7 @@ static int send_aspi_packet(DeviceHandle *dh, unsigned char *cmd, int cdb_size,
 	srb.Flags = SRB_DIR_IN | SRB_EVENT_NOTIFY;
 	break;
       default:
-	Stop(_("illegal data_mode for ASPI: %d"),data_mode);
+	Stop("illegal data_mode for ASPI: %d", data_mode);
 	return -1;
    }
    srb.BufPtr  = buf;
@@ -496,11 +508,11 @@ static int send_aspi_packet(DeviceHandle *dh, unsigned char *cmd, int cdb_size,
 	return -1;
 
       case 0x08: /* BUSY */
-	PrintLog(_("ASPI layer: Target busy.\n"));
+	PrintLog("ASPI layer: Target busy.\n");
 	return -1;
 
       case 0x18: /* Reservation conflict */
-	PrintLog(_("ASPI layer: Reservation conflict.\n"));
+	PrintLog("ASPI layer: Reservation conflict.\n");
 	return -1;
    }
 
@@ -606,7 +618,7 @@ static int send_spti_packet(HANDLE fd, unsigned char *cmd, int cdb_size, char *b
 	ss.spt.DataIn          	= 2;  /* SCSI_IOCTL_DATA_UNSPECIFIED */
 	break;
       default:
-	Stop(_("illegal data_mode: %d"),data_mode);
+	Stop("illegal data_mode: %d",data_mode);
 	return -1;
    }
    ss.spt.DataTransferLength 	= size;
