@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2006 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2008 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -21,7 +21,7 @@
 
 #include "dvdisaster.h"
 
-#if defined(SYS_LINUX) || defined(SYS_FREEBSD) || defined(SYS_DARWIN)
+#if defined(SYS_LINUX) || defined(SYS_FREEBSD) || defined(SYS_DARWIN) || defined(SYS_NETBSD) || defined(SYS_SOLARIS)
 #include <sys/wait.h>
 #endif
 
@@ -34,7 +34,8 @@
  *** Ask user to specify his browser
  ***/
 
-#if defined(SYS_LINUX) || defined(SYS_FREEBSD) || defined(SYS_DARWIN)
+#if defined(SYS_LINUX) || defined(SYS_FREEBSD) || defined(SYS_DARWIN) || defined(SYS_NETBSD) || defined(SYS_SOLARIS)
+
 #define SEARCH_BUTTON 1
 
 typedef struct
@@ -172,7 +173,7 @@ static void msg_destroy_cb(GtkWidget *widget, gpointer data)
    bi->msg = NULL; 
 }
 
-#if defined(SYS_LINUX) || defined(SYS_FREEBSD) || defined(SYS_DARWIN)
+#if defined(SYS_LINUX) || defined(SYS_FREEBSD) || defined(SYS_DARWIN) || defined(SYS_NETBSD) || defined(SYS_SOLARIS)
 
 /* 
  * The following list of browsers and html wrappers
@@ -186,6 +187,7 @@ static void try_browser(browser_info*);
 
 static char *browsers[] = 
 {  "user-selection",
+   "xdg-open",
    "gnome-open",
    "htmlview",
    "firefox",
@@ -270,7 +272,7 @@ static gboolean browser_timeout_func(gpointer data)
  * Invoke the browser
  */
 
-#if defined(SYS_LINUX) || defined(SYS_FREEBSD) || defined(SYS_DARWIN)
+#if defined(SYS_LINUX) || defined(SYS_FREEBSD) || defined(SYS_DARWIN) || defined(SYS_NETBSD) || defined(SYS_SOLARIS)
 static void try_browser(browser_info *bi)
 {  pid_t pid;
 
@@ -311,7 +313,7 @@ static void try_browser(browser_info *bi)
 
 void ShowHTML(char *target)
 {  browser_info *bi = g_malloc0(sizeof(browser_info));
-   struct stat mystat;
+   gint64 ignore;
    const char *lang;
    char *path = NULL;
    int http_url;
@@ -334,11 +336,11 @@ void ShowHTML(char *target)
       lang = g_getenv("LANG");
 
       if(lang)
-      {  if(!strncmp(lang, "cs", 2)) 
+      {  if(!strncmp(lang, "ru", 2)) 
 #ifdef SYS_MINGW
-	     path = g_strdup_printf("%s\\cs\\%s",Closure->docDir,target); 
+	     path = g_strdup_printf("%s\\ru\\%s",Closure->docDir,target); 
 #else
-	     path = g_strdup_printf("%s/cs/%s",Closure->docDir,target); 
+	     path = g_strdup_printf("%s/ru/%s",Closure->docDir,target); 
 #endif
          else if(!strncmp(lang, "de", 2)) 
 #ifdef SYS_MINGW
@@ -358,7 +360,7 @@ void ShowHTML(char *target)
       }
 
 #ifdef SYS_MINGW      
-      if(stat(path, &mystat) == -1)
+      if(!LargeStat(path, &ignore))
       {  
 	 g_free(path);  /* the local dir is Windows specific */
 	 path = g_strdup_printf("%s\\local\\%s",Closure->docDir,target);
@@ -369,7 +371,7 @@ void ShowHTML(char *target)
    }
    else bi->url = target;
 
-   if(!http_url && stat(bi->url, &mystat) == -1)
+   if(!http_url && !LargeStat(bi->url, &ignore))
    {  
       CreateMessage(_("Documentation file\n%s\nnot found.\n"), GTK_MESSAGE_ERROR, bi->url);
       g_free(bi);
@@ -390,7 +392,7 @@ void ShowHTML(char *target)
    g_timeout_add(1000, browser_timeout_func, (gpointer)bi);
 #endif
 
-#if defined(SYS_LINUX) || defined(SYS_FREEBSD) || defined(SYS_DARWIN)
+#if defined(SYS_LINUX) || defined(SYS_FREEBSD) || defined(SYS_DARWIN) || defined(SYS_NETBSD) || defined(SYS_SOLARIS)
    /* Try the first browser */
 
    browser_index = 0;
