@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2006 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2008 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -28,24 +28,9 @@
  *** Look for ecc headers in RS02 style media
  ***/
 
-static int read_fingerprint(DeviceHandle *dh, unsigned char *fingerprint, gint64 sector)
-{  struct MD5Context md5ctxt;
-   int status;
-
-   status = ReadSectors(dh, Closure->scratchBuf, sector, 1);
-
-   if(status) return FALSE;
-
-   MD5Init(&md5ctxt);
-   MD5Update(&md5ctxt, Closure->scratchBuf, 2048);
-   MD5Final(fingerprint, &md5ctxt);
-
-   return TRUE;
-}
-
 enum { HEADER_FOUND, TRY_NEXT_HEADER, TRY_NEXT_MODULO};
 
-static int try_sector(DeviceHandle *dh, gint64 pos, EccHeader **ehptr)
+static int try_sector(DeviceHandle *dh, gint64 pos, EccHeader **ehptr, unsigned char *secbuf)
 {  EccHeader *eh;
    unsigned char fingerprint[16];
    guint32 recorded_crc;
@@ -56,12 +41,12 @@ static int try_sector(DeviceHandle *dh, gint64 pos, EccHeader **ehptr)
 
    Verbose("udf/try_sector: trying sector %lld\n", pos);
 
-   if(ReadSectors(dh, Closure->scratchBuf, pos, 2))
+   if(ReadSectorsFast(dh, secbuf, pos, 2))
    {  Verbose("udf/try_sector: read error, trying next header\n");
       return TRY_NEXT_HEADER;
    }
 
-   eh = (EccHeader*)Closure->scratchBuf;
+   eh = (EccHeader*)secbuf;
 
    /* See if the magic cookie is there. If not, searching within
       this modulo makes no sense for write-once media.
@@ -98,7 +83,7 @@ static int try_sector(DeviceHandle *dh, gint64 pos, EccHeader **ehptr)
    }
 
    eh = g_malloc(sizeof(EccHeader));
-   memcpy(eh, Closure->scratchBuf, sizeof(EccHeader));
+   memcpy(eh, secbuf, sizeof(EccHeader));
 #ifdef HAVE_BIG_ENDIAN
    SwapEccHeaderBytes(eh);
 #endif
@@ -109,14 +94,14 @@ static int try_sector(DeviceHandle *dh, gint64 pos, EccHeader **ehptr)
    /* Compare medium fingerprint with that recorded in Ecc header */
 
    if(last_fp != eh->fpSector)  /* fingerprint in different sector as before? */
-   {  int status;
+   {  int fp_read;
 
       /* read new fingerprint */
 
-      status = read_fingerprint(dh, fingerprint, eh->fpSector);
+      fp_read = GetMediumFingerprint(dh, fingerprint, eh->fpSector);
       last_fp = eh->fpSector;
 		  
-      if(!status)  /* be optimistic if fingerprint sector is unreadable */
+      if(!fp_read)  /* be optimistic if fingerprint sector is unreadable */
       {  *ehptr = eh;
 	 Verbose("udf/try_sector: read error in fingerprint sector\n");
 	 return HEADER_FOUND;
@@ -175,33 +160,39 @@ static void insert_buttons(GtkDialog *dialog)
  */
 
 EccHeader* FindHeaderInMedium(DeviceHandle *dh, gint64 max_sectors)
-{  EccHeader *eh = NULL;
+{  AlignedBuffer *ab = CreateAlignedBuffer(4096);
+   EccHeader *eh = NULL;
    Bitmap *try_next_header, *try_next_modulo;
    gint64 pos;
    gint64 header_modulo;
    int read_count = 0;
    int answered_continue = FALSE;
+   int warning_shown = FALSE;
 
    /*** Quick search at fixed offsets relative to ISO filesystem */
 
-   if(!max_sectors && dh->isoInfo)
-   {  gint64 iso_size = dh->isoInfo->volumeSize; 
+   if(!max_sectors)
+   {  if(dh->isoInfo)
+      {  gint64 iso_size = dh->isoInfo->volumeSize; 
 
-      /* Iso size is correct; look for root sector at +2 */
+	 /* Iso size is correct; look for root sector at +2 */
 
-      if(try_sector(dh, iso_size, &eh) == HEADER_FOUND)
-      {  Verbose("Root sector search at +0 successful\n");
-	 return eh;
+	 if(try_sector(dh, iso_size, &eh, ab->buf) == HEADER_FOUND)
+	 {  Verbose("Root sector search at +0 successful\n");
+	    FreeAlignedBuffer(ab);
+	    return eh;
+	 }
+
+	 /* Strange stuff. Sometimes the iso size is increased by 150
+	    sectors by the burning software. */
+
+	 if(try_sector(dh, iso_size-150, &eh, ab->buf) == HEADER_FOUND)
+	 {  Verbose("Root sector search at -150 successful\n");
+	    FreeAlignedBuffer(ab);
+	    return eh;
+	 }
       }
-
-      /* Strange stuff. Sometimes the iso size is increased by 150
-	 sectors by the burning software. */
-
-      if(try_sector(dh, iso_size-150, &eh) == HEADER_FOUND)
-      {  Verbose("Root sector search at -150 successful\n");
-	 return eh;
-      }
-
+      FreeAlignedBuffer(ab);
       return NULL;
    }
 
@@ -237,21 +228,30 @@ EccHeader* FindHeaderInMedium(DeviceHandle *dh, gint64 max_sectors)
 	     goto check_next_modulo;
 	 }
 
-	 result = try_sector(dh, pos, &eh);
+	 result = try_sector(dh, pos, &eh, ab->buf);
 
 	 switch(result)
 	 {  case TRY_NEXT_HEADER:
 	       SetBit(try_next_header, pos);
 	       read_count++;
 	       if(!answered_continue && read_count > 5)
-	       {  int answer = ModalDialog(GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, insert_buttons,
-					   _("Faster medium initialization\n\n"
-					     "Searching rewriteable media for error correction data may take a long time.\n"
-					     "Press \"Skip RS02 test\" if you are certain that this medium was\n"
-					     "not augmented with RS02 error correction data."));
+	       {  if(Closure->guiMode)
+		  {  int answer = ModalDialog(GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, insert_buttons,
+					      _("Faster medium initialization\n\n"
+						"Searching this medium for error correction data may take a long time.\n"
+						"Press \"Skip RS02 test\" if you are certain that this medium was\n"
+						"not augmented with RS02 error correction data."));
 		 
-		  if(answer) goto bail_out;
-		  answered_continue = TRUE;
+		    if(answer) goto bail_out;
+		    answered_continue = TRUE;
+		  }
+		  if(!Closure->guiMode && !warning_shown)
+		  {  PrintCLI(_("\nSearching this medium for error correction data may take a long time.\n"
+				"If you are certain that this medium was not augmented with RS02 error correction\n"
+				"data, you might wish to abort this command and re-run with the option\n"
+				"--query-size=udf\n"));
+		    warning_shown = TRUE;
+		  }
 	       }
 	       goto check_next_header;
 	    case TRY_NEXT_MODULO:
@@ -260,6 +260,7 @@ EccHeader* FindHeaderInMedium(DeviceHandle *dh, gint64 max_sectors)
 	    case HEADER_FOUND:
 	       FreeBitmap(try_next_header);
 	       FreeBitmap(try_next_modulo);
+	       FreeAlignedBuffer(ab);
 	       return eh;
 	 }
 
@@ -274,6 +275,7 @@ EccHeader* FindHeaderInMedium(DeviceHandle *dh, gint64 max_sectors)
 bail_out:
    FreeBitmap(try_next_header);
    FreeBitmap(try_next_modulo);
+   FreeAlignedBuffer(ab);
    return NULL;
 }
 
@@ -325,7 +327,11 @@ gint64 MediumLengthFromRS02(DeviceHandle *dh, gint64 max_size)
  * 16bit both byte order
  */
 
-#define bp_get_short_bbo(buf, idx) (*((guint16*)(buf+idx-1)))
+#ifdef HAVE_BIG_ENDIAN
+  #define bp_get_short_bbo(buf, idx) (*((guint16*)(buf+idx+1)))
+#else
+  #define bp_get_short_bbo(buf, idx) (*((guint16*)(buf+idx-1)))
+#endif
 
 static void bp_set_short_bbo(unsigned char *buf, int idx, guint16 value)
 {  unsigned char wx = (value >> 8) & 0xff;
@@ -360,7 +366,11 @@ static void bp_set_short_msb(unsigned char *buf, int idx, guint16 value)
  * 32bit single byte order
  */
 
-#define bp_get_long_lsb(buf, idx) (*((guint32*)(buf+idx-1)))
+#ifdef HAVE_BIG_ENDIAN
+  #define bp_get_long_lsb(buf, idx) (((guint32)buf[(idx)+2])<<24|((guint32)buf[(idx)+1])<<16|((guint32)buf[(idx)])<<8|((guint32)buf[(idx)-1]))
+#else
+  #define bp_get_long_lsb(buf, idx) (*((guint32*)(buf+idx-1)))
+#endif
 #define bp_get_long_msb(buf, idx) (((guint32)buf[(idx)-1])<<24|((guint32)buf[idx])<<16|((guint32)buf[(idx)+1])<<8|((guint32)buf[(idx)+2]))
 
 static void bp_set_long_lsb(unsigned char *buf, int idx, guint32 value)
@@ -393,7 +403,11 @@ static void bp_set_long_msb(unsigned char *buf, int idx, guint32 value)
  * 32bit both byte order 
  */
 
-#define bp_get_long_bbo(buf, idx) (*((guint32*)(buf+idx-1)))
+#ifdef HAVE_BIG_ENDIAN
+  #define bp_get_long_bbo(buf, idx) (*((guint32*)(buf+idx+3)))
+#else
+  #define bp_get_long_bbo(buf, idx) (*((guint32*)(buf+idx-1)))
+#endif
 
 static void bp_set_long_bbo(unsigned char *buf, int idx, guint32 value)
 {  unsigned char st = (value >> 24) & 0xff;
@@ -512,7 +526,7 @@ static void beautify_dchar(char *dchar)
 
   while(idx>0)
   {  if(dchar[idx-1] != ' ')
-       dchar[idx] = tolower(dchar[idx]);
+	dchar[idx] = tolower((int)dchar[idx]);
      idx--;
   }
 
@@ -603,17 +617,21 @@ static IsoInfo* examine_primary_vd(unsigned char *buf)
 
    ii->volumeSize = vss;
 
-   strcpy(ii->volumeLabel, (char*)vlabel);
-   beautify_dchar(ii->volumeLabel);
+   if(!Closure->screenShotMode)
+   {  strcpy(ii->volumeLabel, (char*)vlabel);
+      beautify_dchar(ii->volumeLabel);
+   }
+   else strcpy(ii->volumeLabel, _("Example disc"));
    strcpy(ii->creationDate, (char*)date);
-   ii->creationDate[11] = 0;
+   ii->creationDate[10] = 0;
    return ii;
 }
 
 static IsoInfo* examine_iso(DeviceHandle *dh)
-{  IsoInfo *ii = NULL;
+{  AlignedBuffer *ab = CreateAlignedBuffer(2048);
+   unsigned char *buf = ab->buf;
+   IsoInfo *ii = NULL;
    int sector,status;
-   unsigned char *buf = Closure->scratchBuf;
    int vdt,vdt_ver;
    unsigned char sid[6];
       
@@ -625,7 +643,7 @@ static IsoInfo* examine_iso(DeviceHandle *dh)
    {  if(Closure->stopActions) 
         continue;
 
-      status = ReadSectors(dh, buf, sector, 1);
+      status = ReadSectorsFast(dh, buf, sector, 1);
 
       if(status)
       {  Verbose("  Sector %2d: unreadable\n", sector);
@@ -665,6 +683,7 @@ static IsoInfo* examine_iso(DeviceHandle *dh)
    }
 
 finished:
+   FreeAlignedBuffer(ab);
    return ii;
 }
 
